@@ -30,14 +30,16 @@
     - optional pop up notices when GNOME2's system-tray-applet supports it */
 
 #include "internal.h"
-#include "gtkinternal.h"
+#include "gtkgaim.h"
 
 #include "core.h"
 #include "debug.h"
 #include "prefs.h"
 #include "signals.h"
 #include "sound.h"
+#include "version.h"
 
+#include "away.h"
 #include "gtkaccount.h"
 #include "gtkblist.h"
 #include "gtkft.h"
@@ -49,13 +51,14 @@
 #include "docklet.h"
 
 #include "gaim.h"
-#include "ui.h"
+#include "gtkdialogs.h"
 
 /* globals */
 
 GaimPlugin *handle = NULL;
 static struct docklet_ui_ops *ui_ops = NULL;
 static enum docklet_status status = offline;
+gboolean online_account_supports_chat = FALSE;
 #ifdef _WIN32
 __declspec(dllimport) GSList *unread_message_queue;
 __declspec(dllimport) GSList *away_messages;
@@ -87,12 +90,31 @@ docklet_auto_login()
 /* This is a workaround for a bug in windows GTK+. Clicking outside of the
    menu does not get rid of it, so instead we get rid of it as soon as the
    pointer leaves the menu. */
-static gboolean
-docklet_menu_leave(GtkWidget *menu, GdkEventCrossing *event, void *data)
+static gboolean hide_docklet_menu(gpointer data)
 {
-	if(event->detail == GDK_NOTIFY_ANCESTOR) {
+	if (data != NULL) {
+		gtk_menu_popdown(GTK_MENU(data));
+	}
+	return FALSE;
+}
+static gboolean
+docklet_menu_leave_enter(GtkWidget *menu, GdkEventCrossing *event, void *data)
+{
+	static guint hide_docklet_timer = 0;
+	if (event->type == GDK_LEAVE_NOTIFY && event->detail == GDK_NOTIFY_ANCESTOR) {
 		gaim_debug(GAIM_DEBUG_INFO, "tray icon", "menu leave-notify-event\n");
-		gtk_menu_popdown(GTK_MENU(menu));
+		/* Add some slop so that the menu doesn't annoyingly disappear when mousing around */
+		if (hide_docklet_timer == 0) {
+			hide_docklet_timer = gaim_timeout_add(500,
+					hide_docklet_menu, menu);
+		}
+	} else if (event->type == GDK_ENTER_NOTIFY && event->detail == GDK_NOTIFY_ANCESTOR) {
+		gaim_debug(GAIM_DEBUG_INFO, "tray icon", "menu enter-notify-event\n");
+		if (hide_docklet_timer != 0) {
+			/* Cancel the hiding if we reenter */
+			gaim_timeout_remove(hide_docklet_timer);
+			hide_docklet_timer = 0;
+		}
 	}
 	return FALSE;
 }
@@ -101,6 +123,7 @@ docklet_menu_leave(GtkWidget *menu, GdkEventCrossing *event, void *data)
 static void docklet_menu() {
 	static GtkWidget *menu = NULL;
 	GtkWidget *entry;
+	GtkWidget *menuitem;
 
 	if (menu) {
 		gtk_widget_destroy(menu);
@@ -114,8 +137,9 @@ static void docklet_menu() {
 			gaim_new_item_from_stock(menu, _("Auto-login"), GAIM_STOCK_SIGN_ON, G_CALLBACK(docklet_auto_login), NULL, 0, 0, NULL);
 			break;
 		default:
-			gaim_new_item_from_stock(menu, _("New Message..."), GAIM_STOCK_IM, G_CALLBACK(show_im_dialog), NULL, 0, 0, NULL);
-			gaim_new_item_from_stock(menu, _("Join A Chat..."), GAIM_STOCK_CHAT, G_CALLBACK(gaim_gtk_blist_joinchat_show), NULL, 0, 0, NULL);
+			gaim_new_item_from_stock(menu, _("New Message..."), GAIM_STOCK_IM, G_CALLBACK(gaim_gtkdialogs_im), NULL, 0, 0, NULL);
+			menuitem = gaim_new_item_from_stock(menu, _("Join A Chat..."), GAIM_STOCK_CHAT, G_CALLBACK(gaim_gtk_blist_joinchat_show), NULL, 0, 0, NULL);
+			gtk_widget_set_sensitive(menuitem, online_account_supports_chat);
 			break;
 	}
 
@@ -187,10 +211,13 @@ static void docklet_menu() {
 	gaim_new_item_from_stock(menu, _("Quit"), GTK_STOCK_QUIT, G_CALLBACK(gaim_core_quit), NULL, 0, 0, 0);
 
 #ifdef _WIN32
-	g_signal_connect(menu, "leave-notify-event", G_CALLBACK(docklet_menu_leave), NULL);
+	g_signal_connect(menu, "leave-notify-event", G_CALLBACK(docklet_menu_leave_enter), NULL);
+	g_signal_connect(menu, "enter-notify-event", G_CALLBACK(docklet_menu_leave_enter), NULL);
 #endif
 	gtk_widget_show_all(menu);
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
+				   ui_ops->position_menu,
+				   NULL, 0, gtk_get_current_event_time());
 }
 
 static gboolean
@@ -229,10 +256,12 @@ static gboolean
 docklet_update_status()
 {
 	enum docklet_status oldstatus;
+	GList *c;
 
 	oldstatus = status;
+	online_account_supports_chat = FALSE;
 
-	if (gaim_connections_get_all()) {
+	if ((c = gaim_connections_get_all())) {
 		if (unread_message_queue) {
 			status = online_pending;
 		} else if (awaymessage) {
@@ -246,6 +275,15 @@ docklet_update_status()
 		} else {
 			status = online;
 		}
+		/* Check if any online accounts support chats */
+		while (c != NULL) {
+			GaimConnection *gc = c->data;
+			if (GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl)->chat_info != NULL) {
+				online_account_supports_chat = TRUE;
+				break;
+			}
+			c = c->next;
+		}
 	} else {
 		if (gaim_connections_get_connecting()) {
 			status = offline_connecting;
@@ -396,6 +434,14 @@ gaim_new_conversation(GaimConversation *conv, void *data)
 	g_idle_add(docklet_update_status, &handle);
 }
 
+/* We need this because the blist purge_away_queue cb won't be called before the
+   plugin is unloaded, when quitting */
+static void gaim_quit_cb() {
+	gaim_debug(GAIM_DEBUG_INFO, "tray icon", "dealing with queued messages on exit\n");
+	docklet_flush_queue();
+}
+
+
 /* static void gaim_buddy_signon(GaimConnection *gc, char *who, void *data) {
 }
 
@@ -418,6 +464,7 @@ plugin_load(GaimPlugin *plugin)
 	void *conn_handle = gaim_connections_get_handle();
 	void *conv_handle = gaim_conversations_get_handle();
 	void *accounts_handle = gaim_accounts_get_handle();
+	void *core_handle = gaim_get_core();
 
 	gaim_debug(GAIM_DEBUG_INFO, "tray icon", "plugin loaded\n");
 
@@ -440,6 +487,8 @@ plugin_load(GaimPlugin *plugin)
 	gaim_signal_connect(conv_handle, "conversation-created",
 						plugin, GAIM_CALLBACK(gaim_new_conversation), NULL);
 
+	gaim_signal_connect(core_handle, "quitting",
+						plugin, GAIM_CALLBACK(gaim_quit_cb), NULL);
 /*	gaim_signal_connect(plugin, event_buddy_signon, gaim_buddy_signon, NULL);
 	gaim_signal_connect(plugin, event_buddy_signoff, gaim_buddy_signoff, NULL);
 	gaim_signal_connect(plugin, event_buddy_away, gaim_buddy_away, NULL);
@@ -495,7 +544,9 @@ static GaimGtkPluginUiInfo ui_info =
 
 static GaimPluginInfo info =
 {
-	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_STANDARD,                             /**< type           */
 	GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
 	0,                                                /**< flags          */
