@@ -472,12 +472,12 @@ static gint
 gaim_recv(GIOChannel *source, gchar *buf, gint len)
 {
 	gint total = 0;
-	guint cur;
+	gsize cur;
 
 	GError *error = NULL;
 
 	while (total < len) {
-		if (g_io_channel_read_chars(source, buf + total, len - total, (gsize *) &cur, &error) != G_IO_STATUS_NORMAL) {
+		if (g_io_channel_read_chars(source, buf + total, len - total, &cur, &error) != G_IO_STATUS_NORMAL) {
 			if (error)
 				g_error_free(error);
 			return -1;
@@ -663,6 +663,9 @@ open_socket(char **error)
 
 		g_snprintf(saddr.sun_path, sizeof(saddr.sun_path), "%s" G_DIR_SEPARATOR_S "gaim_%s.%d",
 				g_get_tmp_dir(), g_get_user_name(), gaim_session);
+		/* Unlink any existing socket for this session */
+		unlink (saddr.sun_path);
+
 		if (bind(fd, (struct sockaddr *)&saddr, sizeof(saddr)) != -1)
 			listen(fd, 100);
 		else {
