@@ -4,6 +4,13 @@
 #include "conversation.h"
 #include "debug.h"
 #include "signals.h"
+#include "version.h"
+
+#include "plugin.h"
+#include "pluginpref.h"
+#include "prefs.h"
+
+#define STATENOTIFY_PLUGIN_ID "core-statenotify"
 
 static void
 write_status(GaimBuddy *buddy, const char *message)
@@ -11,15 +18,18 @@ write_status(GaimBuddy *buddy, const char *message)
 	GaimConversation *conv;
 	const char *who;
 	char buf[256];
+	char *escaped;
 
 	conv = gaim_find_conversation_with_account(buddy->name, buddy->account);
 
 	if (conv == NULL)
 		return;
 
-	who = gaim_get_buddy_alias(buddy);
+	who = gaim_buddy_get_alias(buddy);
+	escaped = g_markup_escape_text(who, -1);
 
-	g_snprintf(buf, sizeof(buf), message, who);
+	g_snprintf(buf, sizeof(buf), message, escaped);
+	g_free(escaped);
 
 	gaim_conversation_write(conv, NULL, buf, GAIM_MESSAGE_SYSTEM, time(NULL));
 }
@@ -27,25 +37,49 @@ write_status(GaimBuddy *buddy, const char *message)
 static void
 buddy_away_cb(GaimBuddy *buddy, void *data)
 {
-	write_status(buddy, _("%s has gone away."));
+	if (gaim_prefs_get_bool("/plugins/core/statenotify/notify_away"))
+		write_status(buddy, _("%s has gone away."));
 }
 
 static void
 buddy_unaway_cb(GaimBuddy *buddy, void *data)
 {
-	write_status(buddy, _("%s is no longer away."));
+	if (gaim_prefs_get_bool("/plugins/core/statenotify/notify_away"))
+		write_status(buddy, _("%s is no longer away."));
 }
 
 static void
 buddy_idle_cb(GaimBuddy *buddy, void *data)
 {
-	write_status(buddy, _("%s has become idle."));
+	if (gaim_prefs_get_bool("/plugins/core/statenotify/notify_idle"))
+		write_status(buddy, _("%s has become idle."));
 }
 
 static void
 buddy_unidle_cb(GaimBuddy *buddy, void *data)
 {
-	write_status(buddy, _("%s is no longer idle."));
+	if (gaim_prefs_get_bool("/plugins/core/statenotify/notify_idle"))
+		write_status(buddy, _("%s is no longer idle."));
+}
+
+static GaimPluginPrefFrame *
+get_plugin_pref_frame(GaimPlugin *plugin)
+{
+	GaimPluginPrefFrame *frame;
+	GaimPluginPref *ppref;
+
+	frame = gaim_plugin_pref_frame_new();
+
+	ppref = gaim_plugin_pref_new_with_label(_("Notify When"));
+	gaim_plugin_pref_frame_add(frame, ppref);
+
+	ppref = gaim_plugin_pref_new_with_name_and_label("/plugins/core/statenotify/notify_away", _("Buddy Goes _Away"));
+	gaim_plugin_pref_frame_add(frame, ppref);
+	
+	ppref = gaim_plugin_pref_new_with_name_and_label("/plugins/core/statenotify/notify_idle", _("Buddy Goes _Idle"));
+	gaim_plugin_pref_frame_add(frame, ppref);
+	
+	return frame;
 }
 
 static gboolean
@@ -65,16 +99,23 @@ plugin_load(GaimPlugin *plugin)
 	return TRUE;
 }
 
+static GaimPluginUiInfo prefs_info =
+{
+	get_plugin_pref_frame
+};
+
 static GaimPluginInfo info =
 {
-	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_STANDARD,                             /**< type           */
 	NULL,                                             /**< ui_requirement */
 	0,                                                /**< flags          */
 	NULL,                                             /**< dependencies   */
 	GAIM_PRIORITY_DEFAULT,                            /**< priority       */
 
-	NULL,                                             /**< id             */
+	STATENOTIFY_PLUGIN_ID,                            /**< id             */
 	N_("Buddy State Notification"),                   /**< name           */
 	VERSION,                                          /**< version        */
 	                                                  /**  summary        */
@@ -92,13 +133,16 @@ static GaimPluginInfo info =
 
 	NULL,                                             /**< ui_info        */
 	NULL,                                             /**< extra_info     */
-	NULL,
+	&prefs_info,                                      /**< prefs_info     */
 	NULL
 };
 
 static void
 init_plugin(GaimPlugin *plugin)
 {
+	gaim_prefs_add_none("/plugins/core/statenotify");
+	gaim_prefs_add_bool("/plugins/core/statenotify/notify_away", TRUE);
+	gaim_prefs_add_bool("/plugins/core/statenotify/notify_idle", TRUE);
 }
 
 GAIM_INIT_PLUGIN(statenotify, init_plugin, info)
