@@ -161,8 +161,7 @@ static gboolean tcl_probe_plugin(GaimPlugin *plugin)
 	const char *next;
 	int len, found = 0, err = 0, nelems;
 	gboolean status = FALSE;
-
-	if ((fp = fopen(plugin->path, "r")) == NULL)
+	if ((fp = g_fopen(plugin->path, "r")) == NULL)
 		return FALSE;
 	if (fstat(fileno(fp), &st)) {
 		fclose(fp);
@@ -233,7 +232,7 @@ static gboolean tcl_probe_plugin(GaimPlugin *plugin)
 					info->version = g_strdup(Tcl_GetString(listitems[1]));
 					info->summary = g_strdup(Tcl_GetString(listitems[2]));
 					info->description = g_strdup(Tcl_GetString(listitems[3]));
-					info->author = g_strdup(Tcl_GetString(listitems[5]));
+					info->author = g_strdup(Tcl_GetString(listitems[4]));
 					info->homepage = g_strdup(Tcl_GetString(listitems[5]));
 
 					plugin->info = info;
