@@ -25,13 +25,14 @@
  * pluginized- Sean Egan, Summer 2002
  */
 #include "internal.h"
-#include "gtkinternal.h"
+#include "gtkgaim.h"
 
 #include "blist.h"
 #include "conversation.h"
 #include "debug.h"
 #include "prpl.h"
 #include "signals.h"
+#include "version.h"
 
 #include "gtkblist.h"
 #include "gtkplugin.h"
@@ -154,7 +155,7 @@ static void buddy_ticker_add_buddy(GaimBuddy *b) {
 	g_signal_connect(G_OBJECT(td->ebox), "button-press-event",
 		G_CALLBACK(buddy_click_cb), b);
 
-	td->label = gtk_label_new(gaim_get_buddy_alias(b));
+	td->label = gtk_label_new(gaim_buddy_get_alias(b));
 	gtk_box_pack_start(GTK_BOX(hbox), td->label, FALSE, FALSE, 5);
 
 	gtk_widget_show_all(td->ebox);
@@ -317,7 +318,9 @@ static GaimGtkPluginUiInfo ui_info =
 
 static GaimPluginInfo info =
 {
-	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_STANDARD,                             /**< type           */
 	GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
 	0,                                                /**< flags          */
