@@ -28,6 +28,7 @@
 #include "notify.h"
 #include "pounce.h"
 #include "prefs.h"
+#include "privacy.h"
 #include "prpl.h"
 #include "request.h"
 #include "signals.h"
@@ -145,6 +146,8 @@ gaim_account_new(const char *username, const char *protocol_id)
 	account->ui_settings = g_hash_table_new_full(g_str_hash, g_str_equal,
 				g_free, (GDestroyNotify)g_hash_table_destroy);
 	account->system_log = NULL;
+	/* 0 is not a valid privacy setting */
+	account->perm_deny = GAIM_PRIVACY_ALLOW_ALL;
 
 	return account;
 }
@@ -262,6 +265,24 @@ gaim_account_notify_added(GaimAccount *account, const char *id,
 		ui_ops->notify_added(account, remote_user, id, alias, message);
 }
 
+void
+gaim_account_request_add(GaimAccount *account, const char *id,
+                         const char *remote_user, const char *alias,
+                         const char *message)
+{
+	GaimAccountUiOps *ui_ops;
+
+	g_return_if_fail(account     != NULL);
+	g_return_if_fail(remote_user != NULL);
+
+	ui_ops = gaim_accounts_get_ui_ops();
+
+	/*
+	if (ui_ops != NULL && ui_ops->notify_added != NULL)
+		ui_ops->request_add(account, remote_user, id, alias, message);
+		*/
+}
+
 static void
 change_password_cb(GaimAccount *account, GaimRequestFields *fields)
 {
@@ -915,7 +936,7 @@ gaim_account_get_log(GaimAccount *account)
 
 		account->system_log	 = gaim_log_new(GAIM_LOG_SYSTEM,
 				gaim_account_get_username(account), account,
-				gc != NULL ? gc->login_time : time(NULL));
+				(gc != NULL && gc->login_time != 0) ? gc->login_time : time(NULL));
 	}
 
 	return account->system_log;
@@ -1339,17 +1360,30 @@ gaim_accounts_write(FILE *fp, GaimAccount *account)
 		if (proxy_type != GAIM_PROXY_USE_GLOBAL &&
 			proxy_type != GAIM_PROXY_NONE &&
 			proxy_type != GAIM_PROXY_USE_ENVVAR) {
+			char *esc;
 			if ((value = gaim_proxy_info_get_host(proxy_info)) != NULL)
-				fprintf(fp, "   <host>%s</host>\n", value);
+			{
+				esc = g_markup_escape_text(value, -1);
+				fprintf(fp, "   <host>%s</host>\n", esc);
+				g_free(esc);
+			}
 
 			if ((int_value = gaim_proxy_info_get_port(proxy_info)) != 0)
 				fprintf(fp, "   <port>%d</port>\n", int_value);
 
 			if ((value = gaim_proxy_info_get_username(proxy_info)) != NULL)
-				fprintf(fp, "   <username>%s</username>\n", value);
+			{
+				esc = g_markup_escape_text(value, -1);
+				fprintf(fp, "   <username>%s</username>\n", esc);
+				g_free(esc);
+			}
 
 			if ((value = gaim_proxy_info_get_password(proxy_info)) != NULL)
-				fprintf(fp, "   <password>%s</password>\n", value);
+			{
+				esc = g_markup_escape_text(value, -1);
+				fprintf(fp, "   <password>%s</password>\n", esc);
+				g_free(esc);
+			}
 		}
 
 		fprintf(fp, "  </proxy>\n");
@@ -1379,16 +1413,16 @@ gaim_accounts_sync(void)
 
 	gaim_debug(GAIM_DEBUG_INFO, "accounts", "Writing accounts to disk.\n");
 
-	fp = fopen(user_dir, "r");
+	fp = g_fopen(user_dir, "r");
 
 	if (fp == NULL)
-		mkdir(user_dir, S_IRUSR | S_IWUSR | S_IXUSR);
+		g_mkdir(user_dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	else
 		fclose(fp);
 
 	filename = g_build_filename(user_dir, "accounts.xml.save", NULL);
 
-	if ((fp = fopen(filename, "w")) != NULL) {
+	if ((fp = g_fopen(filename, "w")) != NULL) {
 		GList *l;
 
 		fprintf(fp, "<?xml version='1.0' encoding='UTF-8' ?>\n\n");
@@ -1409,16 +1443,16 @@ gaim_accounts_sync(void)
 		return;
 	}
 
-	if (stat(filename, &st) || (st.st_size == 0)) {
+	if (g_stat(filename, &st) || (st.st_size == 0)) {
 		gaim_debug_error("accounts", "Failed to save accounts\n");
-		unlink(filename);
+		g_unlink(filename);
 		g_free(filename);
 		return;
 	}
 
 	filename_real = g_build_filename(user_dir, "accounts.xml", NULL);
 
-	if (rename(filename, filename_real) < 0) {
+	if (g_rename(filename, filename_real) < 0) {
 		gaim_debug(GAIM_DEBUG_ERROR, "accounts", "Error renaming %s to %s\n",
 				   filename, filename_real);
 	}
