@@ -165,7 +165,8 @@ static gboolean blist_save_callback(gpointer data)
 	return FALSE;
 }
 
-static void schedule_blist_save()
+void
+gaim_blist_schedule_save()
 {
 	if (blist_save_timer != 0)
 		gaim_timeout_remove(blist_save_timer);
@@ -246,7 +247,8 @@ void gaim_blist_update_buddy_status(GaimBuddy *buddy, int status)
 		buddy->uc = status;
 		gaim_contact_compute_priority_buddy(gaim_buddy_get_contact(buddy));
 
-		if ((status & UC_UNAVAILABLE) != (old_status & UC_UNAVAILABLE)) {
+		if ((buddy->present != GAIM_BUDDY_SIGNING_OFF) && (buddy->present != GAIM_BUDDY_OFFLINE) &&
+			((status & UC_UNAVAILABLE) != (old_status & UC_UNAVAILABLE))) {
 			if (status & UC_UNAVAILABLE)
 				gaim_signal_emit(gaim_blist_get_handle(), "buddy-away", buddy);
 			else
@@ -405,7 +407,7 @@ void gaim_blist_rename_buddy(GaimBuddy *buddy, const char *name)
 	g_free(buddy->name);
 	buddy->name = g_strdup(name);
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && ops->update)
 		ops->update(gaimbuddylist, (GaimBlistNode *)buddy);
@@ -423,7 +425,7 @@ void gaim_blist_alias_chat(GaimChat *chat, const char *alias)
 	else
 		chat->alias = NULL;
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && ops->update)
 		ops->update(gaimbuddylist, (GaimBlistNode *)chat);
@@ -442,7 +444,7 @@ void gaim_blist_alias_buddy(GaimBuddy *buddy, const char *alias)
 	else
 		buddy->alias = NULL;
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && ops->update)
 		ops->update(gaimbuddylist, (GaimBlistNode *)buddy);
@@ -465,7 +467,7 @@ void gaim_blist_server_alias_buddy(GaimBuddy *buddy, const char *alias)
 	else
 		buddy->server_alias = NULL;
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && ops->update)
 		ops->update(gaimbuddylist, (GaimBlistNode *)buddy);
@@ -548,7 +550,7 @@ void gaim_blist_rename_group(GaimGroup *source, const char *new_name)
 	}
 
 	/* Save our changes */
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	/* Update the UI */
 	if (ops && ops->update)
@@ -652,9 +654,9 @@ gaim_buddy_set_icon(GaimBuddy *buddy, GaimBuddyIcon *icon)
 	if (buddy->icon)
 		gaim_buddy_icon_cache(icon, buddy);
 	else
-		gaim_blist_node_remove_setting((GaimBlistNode *)buddy, "buddy_icon");
+		gaim_buddy_icon_uncache(buddy);
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	gaim_blist_update_buddy_icon(buddy);
 }
@@ -707,7 +709,7 @@ void gaim_blist_add_chat(GaimChat *chat, GaimGroup *group, GaimBlistNode *node)
 
 		ops->remove(gaimbuddylist, cnode);
 
-		schedule_blist_save();
+		gaim_blist_schedule_save();
 	}
 
 	if (node != NULL) {
@@ -736,7 +738,7 @@ void gaim_blist_add_chat(GaimChat *chat, GaimGroup *group, GaimBlistNode *node)
 		}
 	}
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && ops->update)
 		ops->update(gaimbuddylist, (GaimBlistNode *)cnode);
@@ -808,7 +810,7 @@ void gaim_blist_add_buddy(GaimBuddy *buddy, GaimContact *contact, GaimGroup *gro
 
 		ops->remove(gaimbuddylist, bnode);
 
-		schedule_blist_save();
+		gaim_blist_schedule_save();
 
 		if (bnode->parent->parent != (GaimBlistNode*)g) {
 			hb = g_new(struct _gaim_hbuddy, 1);
@@ -865,7 +867,7 @@ void gaim_blist_add_buddy(GaimBuddy *buddy, GaimContact *contact, GaimGroup *gro
 
 	gaim_contact_compute_priority_buddy(gaim_buddy_get_contact(buddy));
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && ops->update)
 		ops->update(gaimbuddylist, (GaimBlistNode*)buddy);
@@ -902,7 +904,7 @@ void gaim_contact_set_alias(GaimContact *contact, const char *alias)
 	else
 		contact->alias = NULL;
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && ops->update)
 		ops->update(gaimbuddylist, (GaimBlistNode*)contact);
@@ -1044,8 +1046,11 @@ void gaim_blist_add_contact(GaimContact *contact, GaimGroup *group, GaimBlistNod
 		((GaimGroup*)cnode->parent)->totalsize--;
 
 		ops->remove(gaimbuddylist, cnode);
+		/* ops->remove() cleaned up the cnode's ui_data, so we need to
+		 * reinitialize it */
+		ops->new_node(cnode);
 
-		schedule_blist_save();
+		gaim_blist_schedule_save();
 	}
 
 	if (node && (GAIM_BLIST_NODE_IS_CONTACT(node) ||
@@ -1071,7 +1076,7 @@ void gaim_blist_add_contact(GaimContact *contact, GaimGroup *group, GaimBlistNod
 		g->currentsize++;
 	g->totalsize++;
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && cnode->child)
 		ops->update(gaimbuddylist, cnode);
@@ -1162,7 +1167,7 @@ void gaim_blist_add_group(GaimGroup *group, GaimBlistNode *node)
 		gaimbuddylist->root = gnode;
 	}
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	if (ops && ops->update) {
 		ops->update(gaimbuddylist, gnode);
@@ -1205,7 +1210,7 @@ void gaim_blist_remove_contact(GaimContact *contact)
 		if (node->next)
 			node->next->prev = node->prev;
 
-		schedule_blist_save();
+		gaim_blist_schedule_save();
 
 		/* Update the UI */
 		if (ops && ops->remove)
@@ -1254,7 +1259,7 @@ void gaim_blist_remove_buddy(GaimBuddy *buddy)
 	}
 	contact->totalsize--;
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	/* Re-sort the contact */
 	if (contact->priority == buddy) {
@@ -1317,7 +1322,7 @@ void gaim_blist_remove_chat(GaimChat *chat)
 	}
 	group->totalsize--;
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	/* Update the UI */
 	if (ops && ops->remove)
@@ -1371,7 +1376,7 @@ void gaim_blist_remove_group(GaimGroup *group)
 	if (node->next)
 		node->next->prev = node->prev;
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 
 	/* Update the UI */
 	if (ops && ops->remove)
@@ -1827,6 +1832,9 @@ static void parse_buddy(GaimGroup *group, GaimContact *contact, xmlnode *bnode)
 	if (!name)
 		return;
 
+	if(gaim_find_buddy_in_group(account, name, group))
+		return;
+
 	if ((x = xmlnode_get_child(bnode, "alias")))
 		alias = xmlnode_get_data(x);
 
@@ -1946,7 +1954,7 @@ static void parse_group(xmlnode *groupnode)
 
 static gboolean gaim_blist_read(const char *filename)
 {
-	GError *error;
+	GError *error = NULL;
 	gchar *contents = NULL;
 	gsize length;
 	xmlnode *gaim, *blist, *privacy;
@@ -1969,7 +1977,7 @@ static gboolean gaim_blist_read(const char *filename)
 				filename);
 		name = g_build_filename(gaim_user_dir(), "blist.xml~", NULL);
 
-		if ((backup = fopen(name, "w"))) {
+		if ((backup = g_fopen(name, "w"))) {
 			fwrite(contents, length, 1, backup);
 			fclose(backup);
 			chmod(name, S_IRUSR | S_IWUSR);
@@ -1999,6 +2007,7 @@ static gboolean gaim_blist_read(const char *filename)
 		for (anode = privacy->child; anode; anode = anode->next) {
 			xmlnode *x;
 			GaimAccount *account;
+			int imode;
 			const char *acct_name, *proto, *mode, *protocol;
 
 			acct_name = xmlnode_get_attrib(anode, "name");
@@ -2014,7 +2023,8 @@ static gboolean gaim_blist_read(const char *filename)
 			if (!account)
 				continue;
 
-			account->perm_deny = atoi(mode);
+			imode = atoi(mode);
+			account->perm_deny = (imode != 0 ? imode : GAIM_PRIVACY_ALLOW_ALL);
 
 			for (x = anode->child; x; x = x->next) {
 				char *name;
@@ -2399,15 +2409,15 @@ void gaim_blist_sync()
 		return;
 	}
 
-	file = fopen(user_dir, "r");
+	file = g_fopen(user_dir, "r");
 	if (!file)
-		mkdir(user_dir, S_IRUSR | S_IWUSR | S_IXUSR);
+		g_mkdir(user_dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	else
 		fclose(file);
 
 	filename = g_build_filename(user_dir, "blist.xml.save", NULL);
 
-	if ((file = fopen(filename, "w"))) {
+	if ((file = g_fopen(filename, "w"))) {
 		gaim_blist_write(file, NULL);
 		fclose(file);
 		chmod(filename, S_IRUSR | S_IWUSR);
@@ -2418,16 +2428,16 @@ void gaim_blist_sync()
 		return;
 	}
 
-	if (stat(filename, &st) || (st.st_size == 0)) {
+	if (g_stat(filename, &st) || (st.st_size == 0)) {
 		gaim_debug_error("blist", "Failed to save blist\n");
-		unlink(filename);
+		g_unlink(filename);
 		g_free(filename);
 		return;
 	}
 
 	filename_real = g_build_filename(user_dir, "blist.xml", NULL);
 
-	if (rename(filename, filename_real) < 0)
+	if (g_rename(filename, filename_real) < 0)
 		gaim_debug(GAIM_DEBUG_ERROR, "blist save",
 				   "Error renaming %s to %s\n", filename, filename_real);
 
@@ -2435,7 +2445,6 @@ void gaim_blist_sync()
 	g_free(filename_real);
 }
 
-
 static void gaim_blist_node_setting_free(struct gaim_blist_node_setting *setting)
 {
 	switch(setting->type) {
@@ -2466,7 +2475,7 @@ void gaim_blist_node_remove_setting(GaimBlistNode *node, const char *key)
 
 	g_hash_table_remove(node->settings, key);
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 }
 
 
@@ -2484,7 +2493,7 @@ void gaim_blist_node_set_bool(GaimBlistNode* node, const char *key, gboolean val
 
 	g_hash_table_replace(node->settings, g_strdup(key), setting);
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 }
 
 gboolean gaim_blist_node_get_bool(GaimBlistNode* node, const char *key)
@@ -2519,7 +2528,7 @@ void gaim_blist_node_set_int(GaimBlistNode* node, const char *key, int value)
 
 	g_hash_table_replace(node->settings, g_strdup(key), setting);
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 }
 
 int gaim_blist_node_get_int(GaimBlistNode* node, const char *key)
@@ -2555,7 +2564,7 @@ void gaim_blist_node_set_string(GaimBlistNode* node, const char *key,
 
 	g_hash_table_replace(node->settings, g_strdup(key), setting);
 
-	schedule_blist_save();
+	gaim_blist_schedule_save();
 }
 
 const char *gaim_blist_node_get_string(GaimBlistNode* node, const char *key)
