@@ -192,7 +192,8 @@ gaim_connection_connect(GaimConnection *gc)
 		gaim_debug(GAIM_DEBUG_INFO, "connection", "Requesting password\n");
 		gaim_connection_destroy(gc);
 		escaped = g_markup_escape_text(username, strlen(username));
-		primary = g_strdup_printf(_("Enter password for %s"), escaped);
+		primary = g_strdup_printf(_("Enter password for %s (%s)"), escaped,
+								  gaim_account_get_protocol_name(account));
 		gaim_request_input(gc, NULL, primary, NULL, NULL, FALSE, TRUE, NULL,
 						   _("OK"), G_CALLBACK(request_pass_ok_cb),
 						   _("Cancel"), NULL, account);
@@ -262,6 +263,9 @@ gaim_connection_disconnect(GaimConnection *gc)
 		gaim_notify_close_with_handle(gc);
 	}
 
+	if (!gaim_account_get_remember_password(account))
+		gaim_account_set_password(account,NULL);
+
 	gaim_connection_destroy(gc);
 }
 
@@ -271,7 +275,10 @@ gaim_connection_disconnect_cb(gpointer data)
 	GaimAccount *account = data;
 	GaimConnection *gc = gaim_account_get_connection(account);
 
-	if(gc)
+	if (!gaim_account_get_remember_password(account))
+		gaim_account_set_password(account,NULL);
+
+	if (gc)
 		gaim_connection_disconnect(gc);
 
 	return FALSE;
@@ -311,7 +318,7 @@ gaim_connection_set_state(GaimConnection *gc, GaimConnectionState state)
 	if (gc->state == GAIM_CONNECTED) {
 		GaimBlistNode *gnode,*cnode,*bnode;
 		GList *wins;
-		GList *add_buds=NULL;
+		GList *add_buds = NULL;
 		GaimAccount *account = gaim_connection_get_account(gc);
 
 		/* Set the time the account came online */
@@ -377,7 +384,7 @@ gaim_connection_set_state(GaimConnection *gc, GaimConnectionState state)
 
 					b = (GaimBuddy *)bnode;
 					if(b->account == gc->account) {
-						add_buds = g_list_append(add_buds, b->name);
+						add_buds = g_list_append(add_buds, b);
 					}
 				}
 			}
@@ -401,13 +408,13 @@ gaim_connection_set_state(GaimConnection *gc, GaimConnectionState state)
 			gaim_log_write(log, GAIM_MESSAGE_SYSTEM,
 						   gaim_account_get_username(account), time(NULL),
 						   msg);
-			g_free(msg);						   
+			g_free(msg);
 		}
 
 		gaim_account_destroy_log(account);
 
 		if (ops != NULL && ops->disconnected != NULL)
-			ops->disconnected(gc);		
+			ops->disconnected(gc);
 	}
 }
 
@@ -503,9 +510,6 @@ gaim_connection_error(GaimConnection *gc, const char *text)
 	if (ops != NULL) {
 		if (ops->report_disconnect != NULL)
 			ops->report_disconnect(gc, text);
-
-		if (ops->disconnected != NULL)
-			ops->disconnected(gc);
 	}
 
 	gc->disconnect_timeout = gaim_timeout_add(0, gaim_connection_disconnect_cb,
