@@ -614,8 +614,6 @@ gaim_conv_window_switch_conversation(GaimConvWindow *win, unsigned int index)
 	if (ops != NULL && ops->switch_conversation != NULL)
 		ops->switch_conversation(win, index);
 
-	gaim_conversation_set_unseen(conv, GAIM_UNSEEN_NONE);
-
 	gaim_signal_emit(gaim_conversations_get_handle(),
 					 "conversation-switched", old_conv, conv);
 }
@@ -1514,7 +1512,13 @@ gaim_conversation_write(GaimConversation *conv, const char *who,
 			gaim_conv_im_set_typing_state(GAIM_CONV_IM(conv), GAIM_NOT_TYPING);
 	}
 
-	if (gaim_conv_window_get_active_conversation(win) != conv) {
+	if (gaim_conv_window_has_focus(win) &&
+		gaim_conv_window_get_active_conversation(win) == conv)
+	{
+		unseen = GAIM_UNSEEN_NONE;
+	}
+	else
+	{
 		if ((flags & GAIM_MESSAGE_NICK) == GAIM_MESSAGE_NICK ||
 				gaim_conversation_get_unseen(conv) == GAIM_UNSEEN_NICK)
 			unseen = GAIM_UNSEEN_NICK;
@@ -1525,10 +1529,36 @@ gaim_conversation_write(GaimConversation *conv, const char *who,
 		else
 			unseen = GAIM_UNSEEN_TEXT;
 	}
-	else
-		unseen = GAIM_UNSEEN_NONE;
 
 	gaim_conversation_set_unseen(conv, unseen);
+
+	/*
+	 * TODO: This is #if 0'ed out because we don't have a way of
+	 *       telling if a conversation window is minimized.  This
+	 *       should probably be done in gtkconv.c anyway.
+	 */
+#if 0
+	/*
+	 * This is auto-tab switching.
+	 *
+	 * If we received an IM, and the GaimConvWindow is inactive and
+	 * minimized, then make this conversation the active tab in this
+	 * GaimConvWindow.
+	 *
+	 * We do this so that, when the user comes back to the conversation
+	 * window, the first thing they'll see is the new message.  This is
+	 * especially important when the IM window is flashing in their
+	 * taskbar--we want the title of the window to be set to the name
+	 * of the person that IMed them most recently.
+	 */
+	if ((gaim_conversation_get_type(conv) == GAIM_CONV_IM) &&
+		(flags & (GAIM_MESSAGE_RECV | GAIM_MESSAGE_ERROR)) &&
+		(!gaim_conv_window_has_focus(win)) &&
+		(gaim_conv_window_is_minimized(win)))
+	{
+		gaim_conv_window_switch_conversation(win, gaim_conversation_get_index(conv));
+	}
+#endif
 }
 
 void
@@ -1768,6 +1798,8 @@ gboolean gaim_conv_present_error(const char *who, GaimAccount *account, const ch
 	else
 		return FALSE;
 	window = gaim_conversation_get_window(conv);
+
+	/* TODO: There's a good chance this is no longer necessary */
 	if (!gaim_conv_window_has_focus(window)) /* don't change the active conversation if the user is using this window */
 		gaim_conv_window_switch_conversation(window, gaim_conversation_get_index(conv));
 
@@ -2022,7 +2054,8 @@ gaim_conv_chat_add_user(GaimConvChat *chat, const char *user, const char *extra_
 	ops  = gaim_conversation_get_ui_ops(conv);
 
 	quiet = GPOINTER_TO_INT(gaim_signal_emit_return_1(gaim_conversations_get_handle(),
-					 "chat-buddy-joining", conv, user, flags));
+					 "chat-buddy-joining", conv, user, flags)) ||
+			gaim_conv_chat_is_user_ignored(chat, user);
 
 	cb = gaim_conv_chat_cb_new(user, flags);
 
@@ -2130,7 +2163,9 @@ gaim_conv_chat_rename_user(GaimConvChat *chat, const char *old_user,
 		its_me = TRUE;
 	}
 
-	if (gaim_prefs_get_bool("/core/conversations/chat/show_nick_change")) {
+	if (gaim_prefs_get_bool("/core/conversations/chat/show_nick_change") ||
+		gaim_conv_chat_is_user_ignored(chat, new_user))
+	{
 		if(its_me) {
 			g_snprintf(tmp, sizeof(tmp),
 					_("You are now known as %s"), new_user);
@@ -2159,7 +2194,8 @@ gaim_conv_chat_remove_user(GaimConvChat *chat, const char *user, const char *rea
 	ops  = gaim_conversation_get_ui_ops(conv);
 
 	quiet = GPOINTER_TO_INT(gaim_signal_emit_return_1(gaim_conversations_get_handle(),
-				"chat-buddy-leaving", conv, user, reason));
+				"chat-buddy-leaving", conv, user, reason)) ||
+			gaim_conv_chat_is_user_ignored(chat, user);
 
 	if (ops != NULL && ops->chat_remove_user != NULL)
 		ops->chat_remove_user(conv, user);
@@ -2239,11 +2275,15 @@ gaim_conv_chat_remove_users(GaimConvChat *chat, GList *users, const char *reason
 
 		*tmp = '\0';
 
-		for (l = users, i = 0; i < max; i++, l = l->next) {
-			g_strlcat(tmp, (char *)l->data, sizeof(tmp));
+		for (l = users, i = 0; i < max; i++, l = l->next)
+		{
+			if (!gaim_conv_chat_is_user_ignored(chat, (char*)l->data))
+			{
+				g_strlcat(tmp, (char *)l->data, sizeof(tmp));
 
-			if (i < max - 1)
-				g_strlcat(tmp, ", ", sizeof(tmp));
+				if (i < max - 1)
+					g_strlcat(tmp, ", ", sizeof(tmp));
+			}
 		}
 
 		if (size > 10)
