@@ -113,8 +113,7 @@ gaim_xfer_set_status(GaimXfer *xfer, GaimXferStatusType status)
 	xfer->status = status;
 }
 
-static void
-gaim_xfer_conversation_write(GaimXfer *xfer, char *message, gboolean is_error)
+void gaim_xfer_conversation_write(GaimXfer *xfer, char *message, gboolean is_error)
 {
 	GaimConversation *conv = NULL;
 	GString *gs_message = NULL;
@@ -139,23 +138,26 @@ gaim_xfer_conversation_write(GaimXfer *xfer, char *message, gboolean is_error)
 
 static void gaim_xfer_show_file_error(GaimXfer *xfer, const char *filename)
 {
-	gchar *msg = NULL;
+	int err = errno;
+	gchar *msg = NULL, *utf8;
 	GaimXferType xfer_type = gaim_xfer_get_type(xfer);
 
+	utf8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
 	switch(xfer_type) {
 		case GAIM_XFER_SEND:
 			msg = g_strdup_printf(_("Error reading %s: \n%s.\n"),
-								  filename, strerror(errno));
+								  utf8, strerror(err));
 			break;
 		case GAIM_XFER_RECEIVE:
 			msg = g_strdup_printf(_("Error writing %s: \n%s.\n"),
-								  filename, strerror(errno));
+								  utf8, strerror(err));
 			break;
 		default:
 			msg = g_strdup_printf(_("Error accessing %s: \n%s.\n"),
-								  filename, strerror(errno));
+								  utf8, strerror(err));
 			break;
-		}
+	}
+	g_free(utf8);
 
 	gaim_xfer_conversation_write(xfer, msg, TRUE);
 	gaim_xfer_error(xfer_type, xfer->who, msg);
@@ -170,7 +172,7 @@ gaim_xfer_choose_file_ok_cb(void *user_data, const char *filename)
 
 	xfer = (GaimXfer *)user_data;
 
-	if (stat(filename, &st) != 0) {
+	if (g_stat(filename, &st) != 0) {
 		/* File not found. */
 		if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
 			gaim_xfer_request_accepted(xfer, filename);
@@ -200,8 +202,11 @@ gaim_xfer_choose_file_ok_cb(void *user_data, const char *filename)
 	}
 	else if ((gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) &&
 			 S_ISDIR(st.st_mode)) {
-				char *msg = g_strdup_printf(
-					_("%s is not a regular file. Cowardly refusing to overwrite it.\n"), filename);
+		char *utf8, *msg;
+		utf8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+		msg = g_strdup_printf(
+					_("%s is not a regular file. Cowardly refusing to overwrite it.\n"), utf8);
+		g_free(utf8);
 		gaim_notify_error(NULL, NULL, msg, NULL);
 		g_free(msg);
 		gaim_xfer_request_denied(xfer);
@@ -320,7 +325,7 @@ gaim_xfer_ask_accept(GaimXfer *xfer)
 
 void
 gaim_xfer_request(GaimXfer *xfer)
-{
+{	
 	g_return_if_fail(xfer != NULL);
 	g_return_if_fail(xfer->ops.init != NULL);
 
@@ -328,10 +333,16 @@ gaim_xfer_request(GaimXfer *xfer)
 
 	if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
 		if (gaim_xfer_get_filename(xfer) ||
-		    gaim_xfer_get_status(xfer) == GAIM_XFER_STATUS_ACCEPTED)
+		    gaim_xfer_get_status(xfer) == GAIM_XFER_STATUS_ACCEPTED) {
+		    	gchar* message = NULL;
+		    	message = g_strdup_printf(_("%s is offering to send file %s"),
+				xfer->who, gaim_xfer_get_filename(xfer));
+			gaim_xfer_conversation_write(xfer, message, FALSE);
+			g_free(message);
 			gaim_xfer_ask_recv(xfer);
-		else
+		} else {
 			gaim_xfer_ask_accept(xfer);
+		}
 	} else
 		gaim_xfer_choose_file(xfer);
 }
@@ -354,31 +365,36 @@ gaim_xfer_request_accepted(GaimXfer *xfer, const char *filename)
 	}
 
 	if (type == GAIM_XFER_SEND) {
-		char *msg;
+		char *msg, *utf8;
 
 		/* Check the filename. */
 		if (g_strrstr(filename, "..")) {
+			char *utf8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
 
-			msg = g_strdup_printf(_("%s is not a valid filename.\n"), filename);
+			msg = g_strdup_printf(_("%s is not a valid filename.\n"), utf8);
 			gaim_xfer_error(type, xfer->who, msg);
+			g_free(utf8);
 			g_free(msg);
 
 			gaim_xfer_unref(xfer);
 			return;
 		}
 
-		if (stat(filename, &st) == -1) {
+		if (g_stat(filename, &st) == -1) {
 		  	gaim_xfer_show_file_error(xfer, filename);
 			gaim_xfer_unref(xfer);
 			return;
 		}
 
 		gaim_xfer_set_local_filename(xfer, filename);
-		gaim_xfer_set_filename(xfer, g_basename(filename));
 		gaim_xfer_set_size(xfer, st.st_size);
-		
+
+		utf8 = g_filename_to_utf8(g_basename(filename), -1, NULL, NULL, NULL);
+		gaim_xfer_set_filename(xfer, utf8);
+
 		msg = g_strdup_printf(_("Offering to send %s to %s"),
-							  filename, xfer->who);
+							  utf8, xfer->who);
+		g_free(utf8);
 		gaim_xfer_conversation_write(xfer, msg, FALSE);
 		g_free(msg);
 	}
@@ -789,7 +805,7 @@ begin_transfer(GaimXfer *xfer, GaimInputCondition cond)
 {
 	GaimXferType type = gaim_xfer_get_type(xfer);
 
-	xfer->dest_fp = fopen(gaim_xfer_get_local_filename(xfer),
+	xfer->dest_fp = g_fopen(gaim_xfer_get_local_filename(xfer),
 						  type == GAIM_XFER_RECEIVE ? "wb" : "rb");
 
 	if (xfer->dest_fp == NULL) {
@@ -1058,7 +1074,6 @@ gaim_xfer_update_progress(GaimXfer *xfer)
 		ui_ops->update_progress(xfer, gaim_xfer_get_progress(xfer));
 }
 
-
 /**************************************************************************
  * File Transfer Subsystem API
  **************************************************************************/
