@@ -22,10 +22,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+#ifndef _GAIM_GTKBLIST_H_
+#define _GAIM_GTKBLIST_H_
 
-#ifndef _GAIM_GTK_LIST_H_
-#define _GAIM_GTK_LIST_H_
-
+#include "gtkgaim.h"
 #include "blist.h"
 
 enum {
@@ -129,6 +129,14 @@ GaimBlistUiOps *gaim_gtk_blist_get_ui_ops(void);
  */
 GdkPixbuf *create_prpl_icon(GaimAccount *account);
 
+/**
+ * Populates a menu with the items shown on the buddy list for a buddy.
+ *
+ * @param menu  The menu to populate
+ * @param buddy The buddy who's menu to get
+ */
+void gaim_gtk_blist_make_buddy_menu(GtkWidget *menu, GaimBuddy *buddy);
+
 /**
  * Refreshes all the nodes of the buddy list.
  * This should only be called when something changes to affect most of the nodes (such as a ui preference changing)
@@ -236,4 +244,4 @@ void gaim_gtk_append_blist_node_proto_menu (GtkWidget *menu, GaimConnection *gc,
  */
 void gaim_gtk_append_blist_node_extended_menu(GtkWidget *menu, GaimBlistNode *node);
 
-#endif /* _GAIM_GTK_LIST_H_ */
+#endif /* _GAIM_GTKBLIST_H_ */
