@@ -19,7 +19,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include "gtkinternal.h"
+#include "internal.h"
+#include "gtkgaim.h"
 
 #include "account.h"
 #include "debug.h"
@@ -28,11 +29,11 @@
 #include "stock.h"
 #include "util.h"
 
+#include "away.h"
 #include "gtkblist.h"
+#include "gtkdialogs.h"
 #include "gtkutils.h"
 
-#include "ui.h"
-
 /*
  * The next couple of functions deal with the connection dialog
  */
@@ -78,7 +79,7 @@ static void cancel_signon(GtkWidget *button, struct signon_meter *meter)
 		kill_meter(meter, _("Done."));
 
 		if (gaim_connections_get_all() == NULL) {
-			destroy_all_dialogs();
+			gaim_gtkdialogs_destroy_all();
 
 			gaim_blist_destroy();
 
@@ -201,7 +202,6 @@ static void gaim_gtk_connection_connect_progress(GaimConnection *gc,
 		gtk_window_set_role(GTK_WINDOW(meter_win->window), "signon");
 		gtk_container_set_border_width(GTK_CONTAINER(meter_win->window), 5);
 		gtk_window_set_title(GTK_WINDOW(meter_win->window), _("Signon"));
-		gtk_widget_realize(meter_win->window);
 
 		vbox = gtk_vbox_new (FALSE, 0);
 		gtk_container_add(GTK_CONTAINER(meter_win->window), vbox);
@@ -263,7 +263,7 @@ static void gaim_gtk_connection_disconnected(GaimConnection *gc)
 	if (gaim_connections_get_all() != NULL)
 		return;
 
-	destroy_all_dialogs();
+	gaim_gtkdialogs_destroy_all();
 
 	gaim_blist_destroy();
 
@@ -313,14 +313,20 @@ static void disconnect_window_hide()
  * is more than one account then make sure the GtkTreeView is shown.  
  * If there are no accounts disconnected then hide the dialog.
  */
-static void disconnect_window_update_buttons(GtkTreeModel *model)
+static void disconnect_window_update_buttons()
 {
 	GtkTreeIter iter;
 	GtkTreeSelection *sel;
-	const char *label_text;
+	GtkTreeModel *model;
+	char *label_text;
 	GaimAccount *account = NULL;
 
-	if ((disconnect_window == NULL) || (model == NULL))
+	if (disconnect_window == NULL)
+		return;
+
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(disconnect_window->treeview));
+
+	if (model == NULL)
 		return;
 
 	if (!gtk_tree_model_get_iter_first(model, &iter)) {
@@ -366,6 +372,7 @@ static void disconnect_window_update_buttons(GtkTreeModel *model)
 		gtk_widget_hide(disconnect_window->reconnect_btn);
 	else
 		gtk_widget_show(disconnect_window->reconnect_btn);
+	g_free(label_text);
 }
 
 static void disconnect_response_cb(GtkDialog *dialog, gint id, GtkWidget *widget)
@@ -423,7 +430,7 @@ static void disconnect_response_cb(GtkDialog *dialog, gint id, GtkWidget *widget
 
 		}
 
-		disconnect_window_update_buttons(model);
+		disconnect_window_update_buttons();
 
 		break;
 
@@ -474,7 +481,7 @@ static void disconnect_response_cb(GtkDialog *dialog, gint id, GtkWidget *widget
 		}
 
 		gaim_account_connect(account);
-		disconnect_window_update_buttons(model);
+		disconnect_window_update_buttons();
 
 		break;
 
@@ -491,7 +498,7 @@ static void disconnect_response_cb(GtkDialog *dialog, gint id, GtkWidget *widget
  */
 static void disconnect_tree_cb(GtkTreeSelection *sel, GtkTreeModel *model) 
 {
-	disconnect_window_update_buttons(model);
+	disconnect_window_update_buttons();
 }
 
 /*
@@ -542,7 +549,7 @@ static void disconnect_connection_change_cb(GaimConnection *gc, void *data) {
 	if (scale  != NULL)
 		g_object_unref(G_OBJECT(scale));
 
-	disconnect_window_update_buttons(model);
+	disconnect_window_update_buttons();
 }
 
 static void
@@ -658,7 +665,7 @@ gaim_gtk_connection_report_disconnect(GaimConnection *gc, const char *text)
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(disconnect_window->treeview));
 	gtk_tree_selection_select_iter(sel, &new_iter);
 
-	disconnect_window_update_buttons(GTK_TREE_MODEL(list_store));
+	disconnect_window_update_buttons();
 
 	g_free(label_text);
 }
