@@ -72,6 +72,7 @@ static struct developer developers[] = {
 	{"Luke 'LSchiere' Schierer",	N_("support"), "lschiere@users.sf.net"},
 	{"Stu 'nosnilmot' Tomlinson",	N_("developer"), NULL},
 	{"Gary 'grim' Kramlich",		N_("developer"), NULL},
+	{"Richard 'rlaager' Laager",	N_("developer"), NULL},
 	{NULL, NULL, NULL}
 };
 
@@ -79,6 +80,7 @@ static struct developer patch_writers[] = {
 	{"Ka-Hing 'javabsp' Cheung",	NULL,	NULL},
 	{"Felipe 'shx' Contreras",		NULL,	NULL},
 	{"Decklin Foster",				NULL,	NULL},
+	{"Peter 'Bleeter' Lawler",      NULL,   NULL},
 	{"Robert 'Robot101' McQueen",	NULL,	NULL},
 	{"Benjamin Miller",				NULL,	NULL},
 	{"Kevin 'SimGuy' Stange",		NULL,	NULL},
@@ -97,7 +99,7 @@ static struct developer retired_developers[] = {
 };
 
 static struct translator current_translators[] = {
-	{"Azerbaijani",				"az", "Metin Amiroff", "metin@karegen.com"},
+	{N_("Azerbaijani"),				"az", "Metin Amiroff", "metin@karegen.com"},
     {N_("Bulgarian"),           "bg", N_("Vladimira Girginova and Vladimir (Kaladan) Petkov"), "missing@here.is,vpetkov@i-space.org"},
 	{N_("Catalan"),				"ca", "Josep Puigdemont", "tradgnome@softcatala.org"},
 	{N_("Czech"),				"cs", "Miloslav Trmac", "mitr@volny.cz"},
@@ -111,8 +113,8 @@ static struct translator current_translators[] = {
 	{N_("Finnish"),				"fi", "Arto Alakulju", "arto@alakulju.net"},
 	{N_("French"),				"fr", "Éric Boumaour", "zongo_fr@users.sourceforge.net"},
 	{N_("Hebrew"),				"he", "Pavel Bibergal", "cyberkm203@hotmail.com"},
-	{N_("Hindi"),				"hi", "Ravishankar Shrivastava", "raviratlami@yahoo.com"},
-	{N_("Hungarian"),			"hu", "Zoltan Sutto", "sutto.zoltan@rutinsoft.hu"},
+	{N_("Hindi"),				"hi", "Akash Mahajan", "akashm@users.sf.net"},
+	{N_("Hungarian"),			"hu", "Gabor Kelemen", "kelemeng@gnome.hu"},
 	{N_("Italian"),				"it", "Claudio Satriano", "satriano@na.infn.it"},
 	{N_("Japanese"),			"ja", "Takashi Aihana", "aihana@gnome.gr.jp"},
 	{N_("Georgian"),            "ka", "Temuri Doghonadze", "admin@security.gov.ge"},
@@ -120,12 +122,14 @@ static struct translator current_translators[] = {
 	{N_("Lithuanian"),			"lt", "Gediminas Čičinskas", "gediminas@parok.lt"},
 	{N_("Macedonian"),			"mk", "Tomislav Markovski", "herrera@users.sf.net"},
 	{N_("Burmese"),             "my_MM", "Minn Myat Soe", "mmyatsoe@gmail.com"},
+	{N_("Norwegian"),			"nb", "Kyrre Ness Sjøbæk", "kyrsjo@users.sf.net"},
 	{N_("Dutch; Flemish"),		"nl", "Vincent van Adrighem", "V.vanAdrighem@dirck.mine.nu"},
-	{N_("Norwegian"),			"no", "Kyrre Ness Sjøbæk", "kyrsjo@users.sf.net"},
+	{N_("Norwegian (Nynorsk)"), "nn", "Kurt-Rune Bergset", ""},
+	{N_("Punjabi"),             "pa", "Amanpreet Singh Alam", "aalam@redhat.com"},
 	{N_("Polish"),				"pl", "Krzysztof Foltman, Emil Nowak", "krzysztof@foltman.com;emil5@go2.pl"},
 	{N_("Portuguese"),			"pt", "Duarte Henriques", "duarte_henriques@myrealbox.com"},
 	{N_("Portuguese-Brazil"),	"pt_BR", "Maurício de Lemos Rodrigues Collares Neto", "mauricioc@gmail.com"},
-	{N_("Romanian"),			"ro", "Mişu Moldovan", "dumol@go.ro"},
+	{N_("Romanian"),			"ro", "Mişu Moldovan", "dumol@gnome.ro"},
 	{N_("Russian"),				"ru", "Dmitry Beloglazov", "dmaa@users.sf.net"},
 	{N_("Slovenian"),			"sl", "Martin Srebotnjak", "miles@filmsi.net"},
 	{N_("Albanian"),            "sq", "Programe Shqip", "besnik@spymac.com"},
@@ -134,6 +138,7 @@ static struct translator current_translators[] = {
 	{N_("Turkish"),             "tr", "Onur Kucuktunc", "onurcc@gmail.com"},
 	{N_("Ukrainian"),           "uk", "Maxim V. Dziumanenko", "mvd@mylinux.com.ua"},
 	{N_("Vietnamese"),			"vi", N_("T.M.Thanh and the Gnome-Vi Team"), "gnomevi-list@lists.sf.net"},
+	{N_("Xhosa"),               "xh", "Canonical Ltd", "translations@canonical.com"},
 	{N_("Simplified Chinese"),	"zh_CN", "Funda Wang", "fundawang@linux.net.cn"},
 	{N_("Traditional Chinese"),	"zh_TW", "Ambrose C. Li, Paladin R. Liu", "acli@ada.dhs.org;paladin@ms1.hinet.net"},
 	{NULL, NULL, NULL, NULL}
@@ -150,6 +155,8 @@ static struct translator past_translators[] = {
 	{N_("Spanish"),				"es", "Amaya Rodrigo, Alejandro G Villar, Nicolás Lichtmaier, JM Pérez Cáncer", NULL},
 	{N_("Finnish"),				"fi", "Tero Kuusela", NULL},
 	{N_("French"),				"fr", "Sébastien François, Stéphane Pontier, Stéphane Wirtel, Loïc Jeannin", NULL},
+	{N_("Hindi"),				"hi", "Ravishankar Shrivastava", "raviratlami@yahoo.com"},
+	{N_("Hungarian"),			"hu", "Zoltan Sutto", NULL},
 	{N_("Italian"),				"it", "Salvatore di Maggio", NULL},
 	{N_("Japanese"),			"ja", "Ryosuke Kutsuna, Taku Yasui, Junichi Uekawa", NULL},
 	{N_("Korean"),				"ko", "Sang-hyun S, A Ho-seok Lee", NULL},
@@ -158,7 +165,7 @@ static struct translator past_translators[] = {
 	{N_("Russian"),				"ru", "Sergey Volozhanin", NULL},
 	{N_("Russian"),				"ru", "Alexandre Prokoudine", NULL},
 	{N_("Slovak"),				"sk", "Daniel Režný", NULL},
-	{N_("Swedish"),				"sv", "Tore Lundqvist", "tlt@mima.x.se"},
+	{N_("Swedish"),				"sv", "Tore Lundqvist", NULL},
 	{N_("Swedish"),				"sv", "Christian Rose", NULL},
 	{N_("Chinese"),				"zh_CN, zh_TW", "Hashao, Rocky S. Lee", NULL},
 	{NULL, NULL, NULL, NULL}
