@@ -64,6 +64,7 @@ struct _GaimGtkXferDialog
 	GtkWidget *remote_user_label;
 	GtkWidget *protocol_label;
 	GtkWidget *filename_label;
+	GtkWidget *localfile_label;
 	GtkWidget *status_label;
 	GtkWidget *speed_label;
 	GtkWidget *time_elapsed_label;
@@ -186,7 +187,7 @@ update_detailed_info(GaimGtkXferDialog *dialog, GaimXfer *xfer)
 {
 	GaimGtkXferUiData *data;
 	char *kbsec, *time_elapsed, *time_remaining;
-	char *status;
+	char *status, *utf8;
 
 	if (dialog == NULL || xfer == NULL)
 		return;
@@ -239,11 +240,19 @@ update_detailed_info(GaimGtkXferDialog *dialog, GaimXfer *xfer)
 					   gaim_xfer_get_filename(xfer));
 	} else {
 		char *tmp;
+
 		tmp = g_path_get_basename(gaim_xfer_get_local_filename(xfer));
-		gtk_label_set_text(GTK_LABEL(dialog->filename_label), tmp);
+		utf8 = g_filename_to_utf8(tmp, -1, NULL, NULL, NULL);
 		g_free(tmp);
+
+		gtk_label_set_text(GTK_LABEL(dialog->filename_label), utf8);
+		g_free(utf8);
 	}
 
+	utf8 = g_filename_to_utf8((gaim_xfer_get_local_filename(xfer)), -1, NULL, NULL, NULL);
+	gtk_label_set_text(GTK_LABEL(dialog->localfile_label), utf8);
+	g_free(utf8);
+
 	gtk_label_set_text(GTK_LABEL(dialog->status_label), status);
 
 	gtk_label_set_text(GTK_LABEL(dialog->speed_label), kbsec);
@@ -284,11 +293,11 @@ update_buttons(GaimGtkXferDialog *dialog, GaimXfer *xfer)
 		gtk_widget_show(dialog->remove_button);
 
 #ifdef _WIN32 /* Only supported in Win32 right now */
-        if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
-            gtk_widget_set_sensitive(dialog->open_button, TRUE);
-        } else {
-            gtk_widget_set_sensitive(dialog->open_button, FALSE);
-        }
+		if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
+			gtk_widget_set_sensitive(dialog->open_button, TRUE);
+		} else {
+			gtk_widget_set_sensitive(dialog->open_button, FALSE);
+		}
 #endif
 		gtk_widget_set_sensitive(dialog->pause_button,  FALSE);
 		gtk_widget_set_sensitive(dialog->resume_button, FALSE);
@@ -398,20 +407,41 @@ static void
 open_button_cb(GtkButton *button, GaimGtkXferDialog *dialog)
 {
 #ifdef _WIN32 /* Only supported in Win32 right now */
-    int code = (int)ShellExecute(NULL, NULL,
-                                 gaim_xfer_get_local_filename(dialog->selected_xfer),
-                                 NULL, NULL, SW_SHOW);
-    if (code == SE_ERR_ASSOCINCOMPLETE || code == SE_ERR_NOASSOC)
-    {
-        gaim_notify_error(NULL, NULL,
-                          _("There is no application configured to open this type of file."), NULL);
-    }
-    else if (code < 32)
-    {
-        gaim_notify_error(NULL, NULL,
-                          _("An error occurred while opening the file."), NULL);
-        gaim_debug_warning("ft", "filename: %s; code: %d\n", gaim_xfer_get_local_filename(dialog->selected_xfer), code);
-    }
+	int code;
+	if (G_WIN32_HAVE_WIDECHAR_API ()) {
+		wchar_t *wc_filename = g_utf8_to_utf16(
+				gaim_xfer_get_local_filename(
+					dialog->selected_xfer),
+				-1, NULL, NULL, NULL);
+	
+		code = (int) ShellExecuteW(NULL, NULL, wc_filename, NULL, NULL,
+				SW_SHOW);
+
+		g_free(wc_filename);
+	} else {
+		char *l_filename = g_locale_from_utf8(
+				gaim_xfer_get_local_filename(
+					dialog->selected_xfer),
+				-1, NULL, NULL, NULL);
+	
+		code = (int) ShellExecuteA(NULL, NULL, l_filename, NULL, NULL,
+				SW_SHOW);
+
+		g_free(l_filename);
+	}
+
+	if (code == SE_ERR_ASSOCINCOMPLETE || code == SE_ERR_NOASSOC)
+	{
+		gaim_notify_error(NULL, NULL,
+				_("There is no application configured to open this type of file."), NULL);
+	}
+	else if (code < 32)
+	{
+		gaim_notify_error(NULL, NULL,
+				_("An error occurred while opening the file."), NULL);
+		gaim_debug_warning("ft", "filename: %s; code: %d\n",
+				gaim_xfer_get_local_filename(dialog->selected_xfer), code);
+	}
 #endif
 }
 
@@ -552,8 +582,9 @@ make_info_table(GaimGtkXferDialog *dialog)
 	{
 		{ &dialog->local_user_desc_label, &dialog->local_user_label, NULL },
 		{ &dialog->remote_user_desc_label, &dialog->remote_user_label, NULL },
-		{ &label, &dialog->protocol_label,		 _("Protocol:") },
+		{ &label, &dialog->protocol_label,       _("Protocol:") },
 		{ &label, &dialog->filename_label,       _("Filename:") },
+		{ &label, &dialog->localfile_label,      _("Local File:") },
 		{ &label, &dialog->status_label,         _("Status:") },
 		{ &label, &dialog->speed_label,          _("Speed:") },
 		{ &label, &dialog->time_elapsed_label,   _("Time Elapsed:") },
@@ -810,7 +841,7 @@ gaim_gtkxfer_dialog_add_xfer(GaimGtkXferDialog *dialog, GaimXfer *xfer)
 	GaimXferType type;
 	GdkPixbuf *pixbuf;
 	char *size_str, *remaining_str;
-	char *lfilename;
+	char *lfilename, *utf8;
 
 	g_return_if_fail(dialog != NULL);
 	g_return_if_fail(xfer != NULL);
@@ -838,6 +869,9 @@ gaim_gtkxfer_dialog_add_xfer(GaimGtkXferDialog *dialog, GaimXfer *xfer)
 
 	gtk_list_store_append(dialog->model, &data->iter);
 	lfilename = g_path_get_basename(gaim_xfer_get_local_filename(xfer));
+	utf8 = g_filename_to_utf8(lfilename, -1, NULL, NULL, NULL);
+	g_free(lfilename);
+	lfilename = utf8;
 	gtk_list_store_set(dialog->model, &data->iter,
 					   COLUMN_STATUS, pixbuf,
 					   COLUMN_PROGRESS, 0.0,
