@@ -20,18 +20,19 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include "gtkinternal.h"
-
-#include "gtkimhtmltoolbar.h"
-#include "gtkutils.h"
+#include "internal.h"
+#include "gtkgaim.h"
 
 #include "imgstore.h"
 #include "notify.h"
 #include "request.h"
 #include "stock.h"
-#include "ui.h"
 #include "util.h"
 
+#include "gtkdialogs.h"
+#include "gtkimhtmltoolbar.h"
+#include "gtkutils.h"
+
 static GtkVBoxClass *parent_class = NULL;
 
 static void do_bold(GtkWidget *bold, GtkIMHtmlToolbar *toolbar)
@@ -123,6 +124,18 @@ destroy_toolbar_font(GtkWidget *widget, GdkEvent *event,
 	}
 }
 
+static void
+realize_toolbar_font(GtkWidget *widget, GtkIMHtmlToolbar *toolbar)
+{
+	GtkFontSelection *sel;
+
+	sel = GTK_FONT_SELECTION(GTK_FONT_SELECTION_DIALOG(toolbar->font_dialog)->fontsel);
+	gtk_widget_hide_all(gtk_widget_get_parent(sel->size_entry));
+	gtk_widget_show_all(sel->family_list);
+	gtk_widget_show(gtk_widget_get_parent(sel->family_list));
+	gtk_widget_show(gtk_widget_get_parent(gtk_widget_get_parent(sel->family_list)));
+}
+
 static void
 cancel_toolbar_font(GtkWidget *widget, GtkIMHtmlToolbar *toolbar)
 {
@@ -153,7 +166,7 @@ static void apply_font(GtkWidget *widget, GtkFontSelection *fontsel)
 static void
 toggle_font(GtkWidget *font, GtkIMHtmlToolbar *toolbar)
 {
-	const char *fontname;
+	char *fontname;
 
 	g_return_if_fail(toolbar);
 
@@ -163,15 +176,16 @@ toggle_font(GtkWidget *font, GtkIMHtmlToolbar *toolbar)
 		toolbar->font_dialog = gtk_font_selection_dialog_new(_("Select Font"));
 
 		g_object_set_data(G_OBJECT(toolbar->font_dialog), "gaim_toolbar", toolbar);
-
+		
 		if(fontname) {
 			char fonttif[128];
 			g_snprintf(fonttif, sizeof(fonttif), "%s 12", fontname);
 			gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(toolbar->font_dialog),
 													fonttif);
+			g_free(fontname);
 		} else {
 			gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(toolbar->font_dialog),
-													DEFAULT_FONT_FACE);
+													GTK_IMHTMLTOOLBAR_DEFAULT_FONT_FACE);
 		}
 
 		g_signal_connect(G_OBJECT(toolbar->font_dialog), "delete_event",
@@ -180,6 +194,8 @@ toggle_font(GtkWidget *font, GtkIMHtmlToolbar *toolbar)
 						 G_CALLBACK(apply_font), toolbar->font_dialog);
 		g_signal_connect(G_OBJECT(GTK_FONT_SELECTION_DIALOG(toolbar->font_dialog)->cancel_button), "clicked",
 						 G_CALLBACK(cancel_toolbar_font), toolbar);
+		g_signal_connect_after(G_OBJECT(toolbar->font_dialog), "realize",
+						 G_CALLBACK(realize_toolbar_font), toolbar);
 
 		gtk_window_present(GTK_WINDOW(toolbar->font_dialog));
 	} else {
@@ -231,7 +247,7 @@ toggle_fg_color(GtkWidget *color, GtkIMHtmlToolbar *toolbar)
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(color))) {
 		GtkWidget *colorsel;
 		GdkColor fgcolor;
-		const char *color = gtk_imhtml_get_current_forecolor(GTK_IMHTML(toolbar->imhtml));
+		char *color = gtk_imhtml_get_current_forecolor(GTK_IMHTML(toolbar->imhtml));
 
 		if (!toolbar->fgcolor_dialog) {
 
@@ -240,6 +256,7 @@ toggle_fg_color(GtkWidget *color, GtkIMHtmlToolbar *toolbar)
 			if (color) {
 				gdk_color_parse(color, &fgcolor);
 				gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(colorsel), &fgcolor);
+				g_free(color);
 			}
 
 			g_object_set_data(G_OBJECT(colorsel), "gaim_toolbar", toolbar);
@@ -256,7 +273,7 @@ toggle_fg_color(GtkWidget *color, GtkIMHtmlToolbar *toolbar)
 	} else if (toolbar->fgcolor_dialog != NULL) {
 		cancel_toolbar_fgcolor(color, toolbar);
 	} else {
-		//gaim_gtk_advance_past(gtkconv, "<FONT COLOR>", "</FONT>");
+		/* gaim_gtk_advance_past(gtkconv, "<FONT COLOR>", "</FONT>"); */
 	}
 	gtk_widget_grab_focus(toolbar->imhtml);
 }
@@ -304,7 +321,7 @@ toggle_bg_color(GtkWidget *color, GtkIMHtmlToolbar *toolbar)
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(color))) {
 		GtkWidget *colorsel;
 		GdkColor bgcolor;
-		const char *color = gtk_imhtml_get_current_backcolor(GTK_IMHTML(toolbar->imhtml));
+		char *color = gtk_imhtml_get_current_backcolor(GTK_IMHTML(toolbar->imhtml));
 
 		if (!toolbar->bgcolor_dialog) {
 
@@ -313,6 +330,7 @@ toggle_bg_color(GtkWidget *color, GtkIMHtmlToolbar *toolbar)
 			if (color) {
 				gdk_color_parse(color, &bgcolor);
 				gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(colorsel), &bgcolor);
+				g_free(color);
 			}
 
 			g_object_set_data(G_OBJECT(colorsel), "gaim_toolbar", toolbar);
@@ -329,7 +347,7 @@ toggle_bg_color(GtkWidget *color, GtkIMHtmlToolbar *toolbar)
 	} else if (toolbar->bgcolor_dialog != NULL) {
 		cancel_toolbar_bgcolor(color, toolbar);
 	} else {
-		//gaim_gtk_advance_past(gtkconv, "<FONT COLOR>", "</FONT>");
+		/* gaim_gtk_advance_past(gtkconv, "<FONT COLOR>", "</FONT>"); */
 	}
 	gtk_widget_grab_focus(toolbar->imhtml);
 }
@@ -382,7 +400,9 @@ insert_link_cb(GtkWidget *w, GtkIMHtmlToolbar *toolbar)
 		GaimRequestFields *fields;
 		GaimRequestFieldGroup *group;
 		GaimRequestField *field;
+		GtkTextIter start, end;
 		char *msg;
+		char *desc = NULL;
 
 		fields = gaim_request_fields_new();
 
@@ -394,8 +414,11 @@ insert_link_cb(GtkWidget *w, GtkIMHtmlToolbar *toolbar)
 		gaim_request_field_group_add_field(group, field);
 
 		if(GTK_IMHTML(toolbar->imhtml)->format_functions & GTK_IMHTML_LINKDESC) {
+			if (gtk_text_buffer_get_selection_bounds(GTK_IMHTML(toolbar->imhtml)->text_buffer, &start, &end)) {
+				desc = gtk_imhtml_get_text(GTK_IMHTML(toolbar->imhtml), &start, &end);
+			}
 			field = gaim_request_field_string_new("description", _("_Description"),
-							      NULL, FALSE);
+							      desc, FALSE);
 			gaim_request_field_group_add_field(group, field);
 			msg = g_strdup(_("Please enter the URL and description of the "
 							 "link that you want to insert. The description "
@@ -414,6 +437,7 @@ insert_link_cb(GtkWidget *w, GtkIMHtmlToolbar *toolbar)
 					    _("Cancel"), G_CALLBACK(cancel_link_cb),
 					    toolbar);
 		g_free(msg);
+		g_free(desc);
 	} else {
 		close_link_dialog(toolbar);
 	}
@@ -422,87 +446,103 @@ insert_link_cb(GtkWidget *w, GtkIMHtmlToolbar *toolbar)
 
 
 static void
-do_insert_image_cb(GtkWidget *widget, int resp, GtkIMHtmlToolbar *toolbar)
+do_insert_image_cb(GtkWidget *widget, int response, GtkIMHtmlToolbar *toolbar)
 {
-	char *name, *filename;
-	char *buf, *filedata;
+	gchar *filename, *name, *buf;
+	char *filedata;
 	size_t size;
 	GError *error = NULL;
 	int id;
 	GtkTextIter iter;
 	GtkTextMark *ins;
 
-	if (resp != GTK_RESPONSE_OK) {
-		//set_toggle(toolbar->image, FALSE);
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+	if (response != GTK_RESPONSE_ACCEPT) {
+#else /* FILECHOOSER */
+	if (response != GTK_RESPONSE_OK) {
+#endif /* FILECHOOSER */
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toolbar->image), FALSE);
 		return;
 	}
 
-	name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(widget)));
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+#else /* FILECHOOSER */
+	filename = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(widget)));
+#endif /* FILECHOOSER */
 
-	if (!name) {
+	if (filename == NULL) {
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toolbar->image), FALSE);
 		return;
 	}
 
-	if (gaim_gtk_check_if_dir(name, GTK_FILE_SELECTION(widget))) {
-		g_free(name);
+#if !GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+	if (gaim_gtk_check_if_dir(filename, GTK_FILE_SELECTION(widget))) {
+		g_free(filename);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toolbar->image), FALSE);
 		return;
 	}
+#endif /* FILECHOOSER */
 
+	/* The following triggers a callback that closes the widget */
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toolbar->image), FALSE);
 
-	if (!g_file_get_contents(name, &filedata, &size, &error)) {
+	if (!g_file_get_contents(filename, &filedata, &size, &error)) {
 		gaim_notify_error(NULL, NULL, error->message, NULL);
 
 		g_error_free(error);
-		g_free(name);
+		g_free(filename);
 
 		return;
 	}
 
-	filename = name;
-	while (strchr(filename, '/'))
-		filename = strchr(filename, '/') + 1;
+	name = strrchr(filename, G_DIR_SEPARATOR) + 1;
 
-	id = gaim_imgstore_add(filedata, size, filename);
+	id = gaim_imgstore_add(filedata, size, name);
 	g_free(filedata);
 
-	if (!id) {
-		buf = g_strdup_printf(_("Failed to store image: %s\n"), name);
+	if (id == 0) {
+		buf = g_strdup_printf(_("Failed to store image: %s\n"), filename);
 		gaim_notify_error(NULL, NULL, buf, NULL);
 
 		g_free(buf);
-		g_free(name);
+		g_free(filename);
 
 		return;
 	}
 
+	g_free(filename);
+
 	ins = gtk_text_buffer_get_insert(gtk_text_view_get_buffer(GTK_TEXT_VIEW(toolbar->imhtml)));
 	gtk_text_buffer_get_iter_at_mark(gtk_text_view_get_buffer(GTK_TEXT_VIEW(toolbar->imhtml)),
-	                                 &iter, ins);
+									 &iter, ins);
 	gtk_imhtml_insert_image_at_iter(GTK_IMHTML(toolbar->imhtml), id, &iter);
 	gaim_imgstore_unref(id);
-	
-	g_free(name);
 }
 
 
 static void
 insert_image_cb(GtkWidget *save, GtkIMHtmlToolbar *toolbar)
 {
-	char buf[BUF_LONG];
 	GtkWidget *window;
 
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toolbar->image))) {
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+		window = gtk_file_chooser_dialog_new(_("Insert Image"),
+						NULL,
+						GTK_FILE_CHOOSER_ACTION_OPEN,
+						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+						NULL);
+		gtk_dialog_set_default_response(GTK_DIALOG(window), GTK_RESPONSE_ACCEPT);
+		g_signal_connect(G_OBJECT(GTK_FILE_CHOOSER(window)),
+				"response", G_CALLBACK(do_insert_image_cb), toolbar);
+#else /* FILECHOOSER */
 		window = gtk_file_selection_new(_("Insert Image"));
-		g_snprintf(buf, sizeof(buf), "%s" G_DIR_SEPARATOR_S, gaim_home_dir());
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(window), buf);
-
 		gtk_dialog_set_default_response(GTK_DIALOG(window), GTK_RESPONSE_OK);
 		g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(window)),
 				"response", G_CALLBACK(do_insert_image_cb), toolbar);
+#endif /* FILECHOOSER */
 
 		gtk_widget_show(window);
 		toolbar->image_dialog = window;
@@ -510,6 +550,7 @@ insert_image_cb(GtkWidget *save, GtkIMHtmlToolbar *toolbar)
 		gtk_widget_destroy(toolbar->image_dialog);
 		toolbar->image_dialog = NULL;
 	}
+
 	gtk_widget_grab_focus(toolbar->imhtml);
 }
 
@@ -768,7 +809,7 @@ static void mark_set_cb(GtkTextBuffer *buffer, GtkTextIter *location,
 enum {
 	LAST_SIGNAL
 };
-//static guint signals [LAST_SIGNAL] = { 0 };
+/* static guint signals [LAST_SIGNAL] = { 0 }; */
 
 static void
 gtk_imhtmltoolbar_finalize (GObject *object)
