@@ -105,6 +105,7 @@ static void search_cb(GtkWidget *button, GaimGtkLogViewer *lv)
 					   0, title,
 					   1, log, -1);	
 		}
+		g_free(read);
 	}
 	
 	
@@ -159,6 +160,14 @@ static gboolean destroy_syslog_cb(GtkWidget *w, gint resp, void *cb)
 	return TRUE;
 }
 #endif
+
+static void log_row_activated_cb(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *col, GaimGtkLogViewer *viewer) {
+	if (gtk_tree_view_row_expanded(tv, path))
+		gtk_tree_view_collapse_row(tv, path);
+	else
+		gtk_tree_view_expand_row(tv, path, FALSE);
+}
+
 static void log_select_cb(GtkTreeSelection *sel, GaimGtkLogViewer *viewer) {
 	GtkTreeIter   iter;
 	GValue val = { 0, };
@@ -189,13 +198,16 @@ static void log_select_cb(GtkTreeSelection *sel, GaimGtkLogViewer *viewer) {
 	title = title_utf8;
 	gtk_window_set_title(GTK_WINDOW(viewer->window), title);
 	gtk_imhtml_clear(GTK_IMHTML(viewer->imhtml));
-       	gtk_imhtml_append_text(GTK_IMHTML(viewer->imhtml), read, 
-			       GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_TITLE | GTK_IMHTML_NO_SCROLL | 
+	gtk_imhtml_append_text(GTK_IMHTML(viewer->imhtml), read,
+			       GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_TITLE | GTK_IMHTML_NO_SCROLL |
 			       ((flags & GAIM_LOG_READ_NO_NEWLINE) ? GTK_IMHTML_NO_NEWLINE : 0));
 
 	if (viewer->search)
+	{
+		gtk_imhtml_search_clear(GTK_IMHTML(viewer->imhtml));
 		gtk_imhtml_search_find(GTK_IMHTML(viewer->imhtml), viewer->search);
-	
+	}
+
 	g_free(read);
 	g_free(title);
 }
@@ -225,6 +237,7 @@ static void populate_log_tree(GaimGtkLogViewer *lv)
 		/* do utf8 conversions */
 		utf8_tmp = gaim_utf8_try_convert(month);
 		strncpy(month, utf8_tmp, sizeof(month));
+		g_free(utf8_tmp);
 		utf8_tmp = gaim_utf8_try_convert(title);
 		strncpy(title, utf8_tmp, sizeof(title));
 		g_free(utf8_tmp);
@@ -283,6 +296,7 @@ void gaim_gtk_log_show(GaimLogType type, const char *screenname, GaimAccount *ac
 	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(lv->window)->vbox), 0);
 	g_signal_connect(G_OBJECT(lv->window), "response", 
 					 G_CALLBACK(destroy_cb), ht);
+	gtk_window_set_role(GTK_WINDOW(lv->window), "buddy_log");
 
 	hbox = gtk_hbox_new(FALSE, 6);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 6);
@@ -330,6 +344,9 @@ void gaim_gtk_log_show(GaimLogType type, const char *screenname, GaimAccount *ac
 	g_signal_connect (G_OBJECT (sel), "changed",
 			  G_CALLBACK (log_select_cb),
 			  lv);
+	g_signal_connect (G_OBJECT(lv->treeview), "row-activated",
+			  G_CALLBACK(log_row_activated_cb),
+			  lv);
 
 	/* Viewer ************/
 	vbox = gtk_vbox_new(FALSE, 6);
@@ -399,6 +416,7 @@ void gaim_gtk_syslog_show()
 	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(syslog_viewer->window)->vbox), 0);
 	g_signal_connect(G_OBJECT(syslog_viewer->window), "response", 
 					 G_CALLBACK(destroy_cb), NULL);
+	gtk_window_set_role(GTK_WINDOW(syslog_viewer->window), "system_log");
 
 	hbox = gtk_hbox_new(FALSE, 6);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 6);
