@@ -4,8 +4,10 @@
  *
  * gaim
  *
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
- * 
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -20,8 +22,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#ifndef _GAIM_GTK_NOTIFY_H_
-#define _GAIM_GTK_NOTIFY_H_
+#ifndef _GAIM_GTKNOTIFY_H_
+#define _GAIM_GTKNOTIFY_H_
 
 #include "notify.h"
 
@@ -30,6 +32,6 @@
  *
  * @return The GTK+ UI notify operations structure.
  */
-GaimNotifyUiOps *gaim_get_gtk_notify_ui_ops(void);
+GaimNotifyUiOps *gaim_gtk_notify_get_ui_ops(void);
 
-#endif /* _GAIM_GTK_NOTIFY_H_ */
+#endif /* _GAIM_GTKNOTIFY_H_ */
