@@ -4,8 +4,10 @@
  *
  * gaim
  *
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
- * 
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -21,21 +23,24 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include "gtkinternal.h"
+#include "internal.h"
+#include "gtkgaim.h"
 
+#include "account.h"
 #include "conversation.h"
 #include "debug.h"
 #include "notify.h"
 #include "prpl.h"
+#include "request.h"
 #include "server.h"
 #include "sound.h"
+#include "util.h"
 
 #include "gtkblist.h"
+#include "gtkdialogs.h"
 #include "gtkpounce.h"
 #include "gtkutils.h"
 
-#include "ui.h"
-
 typedef struct
 {
 	/* Pounce data */
@@ -71,7 +76,7 @@ typedef struct
 	GtkWidget *play_sound_entry;
 	GtkWidget *play_sound_browse;
 	GtkWidget *play_sound_test;
-	
+
 	GtkWidget *save_pounce;
 
 	/* Buttons */
@@ -106,58 +111,24 @@ cancel_cb(GtkWidget *w, GaimGtkPounceDialog *dialog)
 }
 
 static void
-pounce_update_entryfields(GtkWidget *w, gpointer data)
+pounce_update_entry_fields(void *user_data, const char *filename)
 {
-	const char *filename;
-	GHashTable *args;
-	GtkFileSelection *filesel;
+	GtkWidget *entry = (GtkWidget *)user_data;
 
-	args = (GHashTable *)data;
-	filesel = GTK_FILE_SELECTION(g_hash_table_lookup(args, "filesel"));
-
-	filename = gtk_file_selection_get_filename(filesel);
-
-	if (gaim_gtk_check_if_dir(filename, filesel))
-		return;
-
-	if (filename != NULL)
-		gtk_entry_set_text(GTK_ENTRY(g_hash_table_lookup(args, "entry")),
-						   filename);
-
-	gtk_widget_destroy(GTK_WIDGET(filesel));
-	g_hash_table_destroy(args);
+	gtk_entry_set_text(GTK_ENTRY(entry), filename);
 }
 
 static void
-filesel(GtkWidget *w, gpointer data)
+filesel(GtkWidget *widget, gpointer data)
 {
-	GtkWidget *filesel;
 	GtkWidget *entry;
-	GHashTable *args;
-	
-	entry = (GtkWidget *)data;	
+	const gchar *name;
 
-	filesel = gtk_file_selection_new(_("Select a file"));
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel),
-									gtk_entry_get_text(GTK_ENTRY(entry)));
-	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
-	gtk_file_selection_set_select_multiple(GTK_FILE_SELECTION(filesel), FALSE);
+	entry = (GtkWidget *)data;
+	name = gtk_entry_get_text(GTK_ENTRY(entry));
 
-	args = g_hash_table_new(g_str_hash, g_str_equal);
-	g_hash_table_insert(args, "filesel", filesel);
-	g_hash_table_insert(args, "entry",   entry);
-
-	g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
-					 "clicked",
-					 G_CALLBACK(pounce_update_entryfields), args);
-	g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button),
-							 "clicked",
-							 G_CALLBACK(g_hash_table_destroy), args);
-	g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button),
-							 "clicked",
-							 G_CALLBACK(gtk_widget_destroy), filesel);
-
-	gtk_widget_show(filesel);
+	gaim_request_file(entry, _("Select a file"), name, FALSE,
+					  G_CALLBACK(pounce_update_entry_fields), NULL, entry);
 }
 
 static void
@@ -178,13 +149,14 @@ save_pounce_cb(GtkWidget *w, GaimGtkPounceDialog *dialog)
 {
 	const char *name;
 	const char *message, *command, *sound;
-	struct gaim_buddy_list *blist;
-	struct gaim_gtk_buddy_list *gtkblist;
+	GaimBuddyList *blist;
+	GaimGtkBuddyList *gtkblist;
 	GaimPounceEvent events = GAIM_POUNCE_NONE;
 
 	name = gtk_entry_get_text(GTK_ENTRY(dialog->buddy_entry));
 
-	if (*name == '\0') {
+	if (*name == '\0')
+	{
 		gaim_notify_error(NULL, NULL,
 						  _("Please enter a buddy to pounce."), NULL);
 		return;
@@ -224,7 +196,8 @@ save_pounce_cb(GtkWidget *w, GaimGtkPounceDialog *dialog)
 	if (*command == '\0') command = NULL;
 	if (*sound   == '\0') sound   = NULL;
 
-	if (dialog->pounce == NULL) {
+	if (dialog->pounce == NULL)
+	{
 		dialog->pounce = gaim_pounce_new(GAIM_GTK_UI, dialog->account,
 										 name, events);
 	}
@@ -234,7 +207,7 @@ save_pounce_cb(GtkWidget *w, GaimGtkPounceDialog *dialog)
 		gaim_pounce_set_pouncee(dialog->pounce, name);
 	}
 
-	/* Actions*/
+	/* Actions */
 	gaim_pounce_action_set_enabled(dialog->pounce, "open-window",
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->open_win)));
 	gaim_pounce_action_set_enabled(dialog->pounce, "popup-notify",
@@ -253,6 +226,18 @@ save_pounce_cb(GtkWidget *w, GaimGtkPounceDialog *dialog)
 	gaim_pounce_action_set_attribute(dialog->pounce, "play-sound",
 									 "filename", sound);
 
+	/* Set the defaults for next time. */
+	gaim_prefs_set_bool("/gaim/gtk/pounces/default_actions/open-window",
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->open_win)));
+	gaim_prefs_set_bool("/gaim/gtk/pounces/default_actions/popup-notify",
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->popup)));
+	gaim_prefs_set_bool("/gaim/gtk/pounces/default_actions/send-message",
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->send_msg)));
+	gaim_prefs_set_bool("/gaim/gtk/pounces/default_actions/execute-command",
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->exec_cmd)));
+	gaim_prefs_set_bool("/gaim/gtk/pounces/default_actions/play-sound",
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->play_sound)));
+
 	gaim_pounce_set_save(dialog->pounce,
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->save_pounce)));
 
@@ -278,56 +263,6 @@ pounce_choose_cb(GtkWidget *item, GaimAccount *account,
 	dialog->account = account;
 }
 
-#if 0
-static GtkWidget *
-pounce_user_menu(GaimGtkPounceDialog *dialog)
-{
-	GaimAccount *account;
-	GaimPlugin *prpl;
-	GtkWidget *opt_menu;
-	GtkWidget *menu;
-	GtkWidget *item;
-	GList *l;
-	char buf[2048];
-	int count, place = 0;
-
-	opt_menu = gtk_option_menu_new();
-	menu = gtk_menu_new();
-
-	for (l = gaim_accounts_get_all(), count = 0;
-		 l != NULL;
-		 l = l->next, count++) {
-
-		account = (GaimAccount *)l->data;
-
-		prpl = gaim_find_prpl(account->protocol);
-
-		g_snprintf(buf, sizeof(buf), "%s (%s)", account->username,
-				   (prpl && prpl->info->name)
-				   ? prpl->info->name : _("Unknown"));
-
-		item = gtk_menu_item_new_with_label(buf);
-		g_object_set_data(G_OBJECT(item), "user_data", account);
-
-		g_signal_connect(G_OBJECT(item), "activate",
-						 G_CALLBACK(pounce_choose_cb), dialog);
-
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-		gtk_widget_show(item);
-
-		if (dialog->account == account) {
-			gtk_menu_item_activate(GTK_MENU_ITEM(item));
-			place = count;
-		}
-	}
-
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(opt_menu), menu);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(opt_menu), place);
-
-	return opt_menu;
-}
-#endif
-
 static void
 buddy_changed_cb(GtkEntry *entry, GaimGtkPounceDialog *dialog)
 {
@@ -335,9 +270,75 @@ buddy_changed_cb(GtkEntry *entry, GaimGtkPounceDialog *dialog)
 		return;
 
 	gtk_widget_set_sensitive(dialog->save_button,
-			*gtk_entry_get_text(entry) != '\0');
+		*gtk_entry_get_text(entry) != '\0');
 }
 
+static void
+pounce_dnd_recv(GtkWidget *widget, GdkDragContext *dc, gint x, gint y,
+				GtkSelectionData *sd, guint info, guint t, gpointer data)
+{
+	GaimGtkPounceDialog *dialog;
+
+	if (sd->target == gdk_atom_intern("GAIM_BLIST_NODE", FALSE))
+	{
+		GaimBlistNode *node = NULL;
+		GaimBuddy *buddy;
+
+		memcpy(&node, sd->data, sizeof(node));
+
+		if (GAIM_BLIST_NODE_IS_CONTACT(node))
+			buddy = gaim_contact_get_priority_buddy((GaimContact *)node);
+		else if (GAIM_BLIST_NODE_IS_BUDDY(node))
+			buddy = (GaimBuddy *)node;
+		else
+			return;
+
+		dialog = (GaimGtkPounceDialog *)data;
+
+		gtk_entry_set_text(GTK_ENTRY(dialog->buddy_entry), buddy->name);
+		dialog->account = buddy->account;
+		gaim_gtk_account_option_menu_set_selected(dialog->account_menu, buddy->account);
+
+		gtk_drag_finish(dc, TRUE, (dc->action == GDK_ACTION_MOVE), t);
+	}
+	else if (sd->target == gdk_atom_intern("application/x-im-contact", FALSE))
+	{
+		char *protocol = NULL;
+		char *username = NULL;
+		GaimAccount *account;
+
+		if (gaim_gtk_parse_x_im_contact(sd->data, FALSE, &account,
+										&protocol, &username, NULL))
+		{
+			if (account == NULL)
+			{
+				gaim_notify_error(NULL, NULL,
+					_("You are not currently signed on with an account that "
+					  "can add that buddy."), NULL);
+			}
+			else
+			{
+				dialog = (GaimGtkPounceDialog *)data;
+
+				gtk_entry_set_text(GTK_ENTRY(dialog->buddy_entry), username);
+				dialog->account = account;
+				gaim_gtk_account_option_menu_set_selected(dialog->account_menu, account);
+			}
+		}
+
+		if (username != NULL) g_free(username);
+		if (protocol != NULL) g_free(protocol);
+
+		gtk_drag_finish(dc, TRUE, (dc->action == GDK_ACTION_MOVE), t);
+	}
+}
+
+static const GtkTargetEntry dnd_targets[] =
+{
+	{"GAIM_BLIST_NODE", GTK_TARGET_SAME_APP, 0},
+	{"application/x-im-contact", 0, 1}
+};
+
 void
 gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 						   GaimPounce *cur_pounce)
@@ -358,17 +359,24 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 
 	dialog = g_new0(GaimGtkPounceDialog, 1);
 
-	if (cur_pounce != NULL) {
+	if (cur_pounce != NULL)
+	{
 		dialog->pounce  = cur_pounce;
 		dialog->account = gaim_pounce_get_pouncer(cur_pounce);
 	}
-	else if (account != NULL) {
+	else if (account != NULL)
+	{
 		dialog->pounce  = NULL;
 		dialog->account = account;
 	}
-	else {
+	else
+	{
+		GaimConnection *gc;
+
+		gc = (GaimConnection *)gaim_connections_get_all()->data;
+
 		dialog->pounce  = NULL;
-		dialog->account = gaim_accounts_get_all()->data;
+		dialog->account = gaim_connection_get_account(gc);
 	}
 
 	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
@@ -383,7 +391,6 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 						  ? _("New Buddy Pounce") : _("Edit Buddy Pounce")));
 
 	gtk_container_set_border_width(GTK_CONTAINER(window), 12);
-	gtk_widget_realize(window);
 
 	g_signal_connect(G_OBJECT(window), "delete_event",
 					 G_CALLBACK(delete_win_cb), dialog);
@@ -418,24 +425,27 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 
 	gtk_box_pack_start(GTK_BOX(hbox), dialog->account_menu, FALSE, FALSE, 0);
 	gtk_widget_show(dialog->account_menu);
+	gaim_set_accessible_label (dialog->account_menu, label);
 
 	/* Buddy: */
 	hbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
 
-	label = gtk_label_new_with_mnemonic(_("_Buddy Name:"));
+	label = gtk_label_new_with_mnemonic(_("_Buddy name:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
 	gtk_size_group_add_widget(sg, label);
 
 	dialog->buddy_entry = gtk_entry_new();
+
 	gtk_box_pack_start(GTK_BOX(hbox), dialog->buddy_entry, TRUE, TRUE, 0);
 	gtk_widget_show(dialog->buddy_entry);
 
 	g_signal_connect(G_OBJECT(dialog->buddy_entry), "changed",
 			 G_CALLBACK(buddy_changed_cb), dialog);
+	gaim_set_accessible_label (dialog->buddy_entry, label);
 
 	if (cur_pounce != NULL) {
 		gtk_entry_set_text(GTK_ENTRY(dialog->buddy_entry),
@@ -454,21 +464,21 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 	gtk_widget_show(table);
 
 	dialog->signon =
-		gtk_check_button_new_with_label(_("Sign on"));
+		gtk_check_button_new_with_mnemonic(_("Si_gn on"));
 	dialog->signoff =
-		gtk_check_button_new_with_label(_("Sign off"));
+		gtk_check_button_new_with_mnemonic(_("Sign _off"));
 	dialog->away =
-		gtk_check_button_new_with_label(_("Away"));
+		gtk_check_button_new_with_mnemonic(_("A_way"));
 	dialog->away_return =
-		gtk_check_button_new_with_label(_("Return from away"));
+		gtk_check_button_new_with_mnemonic(_("_Return from away"));
 	dialog->idle =
-		gtk_check_button_new_with_label(_("Idle"));
+		gtk_check_button_new_with_mnemonic(_("_Idle"));
 	dialog->idle_return =
-		gtk_check_button_new_with_label(_("Return from idle"));
+		gtk_check_button_new_with_mnemonic(_("Retur_n from idle"));
 	dialog->typing =
-		gtk_check_button_new_with_label(_("Buddy starts typing"));
+		gtk_check_button_new_with_mnemonic(_("Buddy starts _typing"));
 	dialog->stop_typing =
-		gtk_check_button_new_with_label(_("Buddy stops typing"));
+		gtk_check_button_new_with_mnemonic(_("Buddy stops t_yping"));
 
 	gtk_table_attach(GTK_TABLE(table), dialog->signon,      0, 1, 0, 1,
 					 GTK_FILL, 0, 0, 0);
@@ -504,19 +514,24 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 	gtk_table_set_col_spacings(GTK_TABLE(table), 12);
 	gtk_widget_show(table);
 
-	dialog->open_win = gtk_check_button_new_with_label(_("Open an IM window"));
-	dialog->popup = gtk_check_button_new_with_label(_("Popup notification"));
-	dialog->send_msg = gtk_check_button_new_with_label(_("Send a message"));
-	dialog->exec_cmd = gtk_check_button_new_with_label(_("Execute a command"));
-	dialog->play_sound = gtk_check_button_new_with_label(_("Play a sound"));
-	
-	dialog->send_msg_entry   = gtk_entry_new();
-	dialog->exec_cmd_entry   = gtk_entry_new();
-	dialog->exec_cmd_browse = gtk_button_new_with_label(_("Browse"));
-	dialog->play_sound_entry = gtk_entry_new();
-	dialog->play_sound_browse = gtk_button_new_with_label(_("Browse"));
-	dialog->play_sound_test = gtk_button_new_with_label(_("Test"));
-	
+	dialog->open_win
+		= gtk_check_button_new_with_mnemonic(_("Op_en an IM window"));
+	dialog->popup
+		= gtk_check_button_new_with_mnemonic(_("_Popup notification"));
+	dialog->send_msg
+		= gtk_check_button_new_with_mnemonic(_("Send a _message"));
+	dialog->exec_cmd
+		= gtk_check_button_new_with_mnemonic(_("E_xecute a command"));
+	dialog->play_sound
+		= gtk_check_button_new_with_mnemonic(_("P_lay a sound"));
+
+	dialog->send_msg_entry    = gtk_entry_new();
+	dialog->exec_cmd_entry    = gtk_entry_new();
+	dialog->exec_cmd_browse   = gtk_button_new_with_mnemonic(_("B_rowse..."));
+	dialog->play_sound_entry  = gtk_entry_new();
+	dialog->play_sound_browse = gtk_button_new_with_mnemonic(_("Bro_wse..."));
+	dialog->play_sound_test   = gtk_button_new_with_mnemonic(_("Pre_view"));
+
 	gtk_widget_set_sensitive(dialog->send_msg_entry,   FALSE);
 	gtk_widget_set_sensitive(dialog->exec_cmd_entry,   FALSE);
 	gtk_widget_set_sensitive(dialog->exec_cmd_browse, FALSE);
@@ -566,7 +581,7 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 	exec_widgets = g_ptr_array_new();
 	g_ptr_array_add(exec_widgets,dialog->exec_cmd_entry);
 	g_ptr_array_add(exec_widgets,dialog->exec_cmd_browse);
-	
+
 	g_signal_connect(G_OBJECT(dialog->exec_cmd), "clicked",
 					 G_CALLBACK(gtk_toggle_sensitive_array),
 					 exec_widgets);
@@ -588,17 +603,17 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 	g_signal_connect(G_OBJECT(dialog->play_sound_test), "clicked",
 					 G_CALLBACK(pounce_test_sound),
 					 dialog->play_sound_entry);
-	
+
 	g_signal_connect(G_OBJECT(dialog->send_msg_entry), "activate",
 					 G_CALLBACK(save_pounce_cb), dialog);
 	g_signal_connect(G_OBJECT(dialog->exec_cmd_entry), "activate",
 					 G_CALLBACK(save_pounce_cb), dialog);
 	g_signal_connect(G_OBJECT(dialog->play_sound_entry), "activate",
 					 G_CALLBACK(save_pounce_cb), dialog);
-	
+
 	/* Now the last part, where we have the Save checkbox */
 	dialog->save_pounce = gtk_check_button_new_with_mnemonic(
-		_("_Save this pounce after activation"));
+		_("Sav_e this pounce after activation"));
 
 	gtk_box_pack_start(GTK_BOX(vbox2), dialog->save_pounce, FALSE, FALSE, 0);
 
@@ -617,6 +632,8 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 	/* Delete button */
 	button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
 	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
+	if (cur_pounce == NULL)
+		gtk_widget_set_sensitive(button, FALSE);
 	gtk_widget_show(button);
 
 	g_signal_connect(G_OBJECT(button), "clicked",
@@ -634,15 +651,35 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 	dialog->save_button = button = gtk_button_new_from_stock(GTK_STOCK_SAVE);
 	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
-	
+
 	g_signal_connect(G_OBJECT(button), "clicked",
 					 G_CALLBACK(save_pounce_cb), dialog);
 
 	if (*gtk_entry_get_text(GTK_ENTRY(dialog->buddy_entry)) == '\0')
 		gtk_widget_set_sensitive(button, FALSE);
 
+	/* Setup drag-and-drop */
+	gtk_drag_dest_set(window,
+					  GTK_DEST_DEFAULT_MOTION |
+					  GTK_DEST_DEFAULT_DROP,
+					  dnd_targets,
+					  sizeof(dnd_targets) / sizeof(GtkTargetEntry),
+					  GDK_ACTION_COPY);
+	gtk_drag_dest_set(dialog->buddy_entry,
+					  GTK_DEST_DEFAULT_MOTION |
+					  GTK_DEST_DEFAULT_DROP,
+					  dnd_targets,
+					  sizeof(dnd_targets) / sizeof(GtkTargetEntry),
+					  GDK_ACTION_COPY);
+
+	g_signal_connect(G_OBJECT(window), "drag_data_received",
+					 G_CALLBACK(pounce_dnd_recv), dialog);
+	g_signal_connect(G_OBJECT(dialog->buddy_entry), "drag_data_received",
+					 G_CALLBACK(pounce_dnd_recv), dialog);
+
 	/* Set the values of stuff. */
-	if (cur_pounce != NULL) {
+	if (cur_pounce != NULL)
+	{
 		GaimPounceEvent events = gaim_pounce_get_events(cur_pounce);
 		const char *value;
 
@@ -681,28 +718,83 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 
 		if ((value = gaim_pounce_action_get_attribute(cur_pounce,
 													  "send-message",
-													  "message")) != NULL) {
-
+													  "message")) != NULL)
+		{
 			gtk_entry_set_text(GTK_ENTRY(dialog->send_msg_entry), value);
 		}
 
 		if ((value = gaim_pounce_action_get_attribute(cur_pounce,
 													  "execute-command",
-													  "command")) != NULL) {
-
+													  "command")) != NULL)
+		{
 			gtk_entry_set_text(GTK_ENTRY(dialog->exec_cmd_entry), value);
 		}
 
 		if ((value = gaim_pounce_action_get_attribute(cur_pounce,
 													  "play-sound",
-													  "filename")) != NULL) {
+													  "filename")) != NULL)
+		{
 			gtk_entry_set_text(GTK_ENTRY(dialog->play_sound_entry), value);
 		}
 	}
-	else {
+	else
+	{
+		GaimBuddy *buddy = NULL;
+
+		if (name != NULL)
+			buddy = gaim_find_buddy(account, name);
+
 		/* Set some defaults */
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->send_msg), TRUE);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->signon), TRUE);
+		if (buddy == NULL)
+		{
+			gtk_toggle_button_set_active(
+				GTK_TOGGLE_BUTTON(dialog->signon), TRUE);
+		}
+		else
+		{
+			if (!GAIM_BUDDY_IS_ONLINE(buddy))
+			{
+				gtk_toggle_button_set_active(
+					GTK_TOGGLE_BUTTON(dialog->signon), TRUE);
+			}
+			else
+			{
+				gboolean default_set = FALSE;
+
+				if (buddy->idle)
+				{
+					gtk_toggle_button_set_active(
+						GTK_TOGGLE_BUTTON(dialog->idle_return), TRUE);
+
+					default_set = TRUE;
+				}
+
+				if (buddy->uc & UC_UNAVAILABLE)
+				{
+					gtk_toggle_button_set_active(
+						GTK_TOGGLE_BUTTON(dialog->away_return), TRUE);
+
+					default_set = TRUE;
+				}
+
+				if (!default_set)
+				{
+					gtk_toggle_button_set_active(
+						GTK_TOGGLE_BUTTON(dialog->signon), TRUE);
+				}
+			}
+		}
+
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->open_win),
+			gaim_prefs_get_bool("/gaim/gtk/pounces/default_actions/open-window"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->popup),
+			gaim_prefs_get_bool("/gaim/gtk/pounces/default_actions/popup-notify"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->send_msg),
+			gaim_prefs_get_bool("/gaim/gtk/pounces/default_actions/send-message"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->exec_cmd),
+			gaim_prefs_get_bool("/gaim/gtk/pounces/default_actions/execute-command"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->play_sound),
+			gaim_prefs_get_bool("/gaim/gtk/pounces/default_actions/play-sound"));
 	}
 
 	gtk_widget_show_all(vbox2);
@@ -710,7 +802,7 @@ gaim_gtkpounce_dialog_show(GaimAccount *account, const char *name,
 }
 
 static void
-new_pounce_cb(GtkWidget *w, struct buddy *b)
+new_pounce_cb(GtkWidget *w, GaimBuddy *b)
 {
 	if (b == NULL)
 		gaim_gtkpounce_dialog_show(NULL, NULL, NULL);
@@ -730,7 +822,7 @@ edit_pounce_cb(GtkWidget *w, GaimPounce *pounce)
 	gaim_gtkpounce_dialog_show(NULL, NULL, pounce);
 }
 
-static void
+static gboolean
 fill_menu(GtkWidget *menu, GCallback cb)
 {
 	GtkWidget *image;
@@ -738,20 +830,29 @@ fill_menu(GtkWidget *menu, GCallback cb)
 	GdkPixbuf *pixbuf, *scale;
 	GaimPounce *pounce;
 	const char *buddy;
+	gboolean has_items = FALSE;
 	GList *bp;
 
-	for (bp = gaim_pounces_get_all(); bp != NULL; bp = bp->next) {
+	for (bp = gaim_pounces_get_all(); bp != NULL; bp = bp->next)
+	{
 		pounce = (GaimPounce *)bp->data;
 		buddy = gaim_pounce_get_pouncee(pounce);
 
+		/* Check if account is online, if not skip it */
+		if (!gaim_account_is_connected(pounce->pouncer))
+			continue;
+
+		has_items = TRUE;
+
 		/* Build the menu item */
 		item = gtk_image_menu_item_new_with_label(buddy);
 
 		/* Create a pixmap for the protocol icon. */
 		pixbuf = create_prpl_icon(gaim_pounce_get_pouncer(pounce));
-		if(pixbuf) {
+		if (pixbuf != NULL)
+		{
 			scale = gdk_pixbuf_scale_simple(pixbuf, 16, 16,
-					GDK_INTERP_BILINEAR);
+											GDK_INTERP_BILINEAR);
 
 			/* Now convert it to GtkImage */
 			image = gtk_image_new_from_pixbuf(scale);
@@ -768,6 +869,8 @@ fill_menu(GtkWidget *menu, GCallback cb)
 		/* Set our callbacks. */
 		g_signal_connect(G_OBJECT(item), "activate", cb, pounce);
 	}
+
+	return has_items;
 }
 
 void
@@ -775,13 +878,17 @@ gaim_gtkpounce_menu_build(GtkWidget *menu)
 {
 	GtkWidget *remmenu;
 	GtkWidget *item;
-	GList *l;
+	GList *children, *l;
+	gboolean has_items;
 
-	for (l = gtk_container_get_children(GTK_CONTAINER(menu));
-		 l != NULL;
-		 l = l->next) {
+	g_return_if_fail(menu != NULL);
 
-		gtk_widget_destroy(GTK_WIDGET(l->data));
+	if ((children = gtk_container_get_children(GTK_CONTAINER(menu))) != NULL)
+	{
+		for (l = children; l != NULL; l = l->next)
+			gtk_widget_destroy(GTK_WIDGET(l->data));
+
+		g_list_free(children);
 	}
 
 	/* "New Buddy Pounce" */
@@ -798,7 +905,10 @@ gaim_gtkpounce_menu_build(GtkWidget *menu)
 	/* "Remove Buddy Pounce" menu */
 	remmenu = gtk_menu_new();
 
-	fill_menu(remmenu, G_CALLBACK(delete_pounce_cb));
+	has_items = fill_menu(remmenu, G_CALLBACK(delete_pounce_cb));
+
+	if (!has_items)
+		gtk_widget_set_sensitive(item, FALSE);
 
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), remmenu);
 	gtk_widget_show(remmenu);
@@ -817,96 +927,180 @@ pounce_cb(GaimPounce *pounce, GaimPounceEvent events, void *data)
 {
 	GaimConversation *conv;
 	GaimAccount *account;
+	GaimBuddy *buddy;
 	const char *pouncee;
+	const char *alias;
 
 	pouncee = gaim_pounce_get_pouncee(pounce);
 	account = gaim_pounce_get_pouncer(pounce);
 
-	if (gaim_pounce_action_is_enabled(pounce, "open-window")) {
+	buddy = gaim_find_buddy(account, pouncee);
+
+	alias = gaim_buddy_get_alias(buddy);
+
+	if (gaim_pounce_action_is_enabled(pounce, "open-window"))
+	{
 		conv = gaim_find_conversation_with_account(pouncee, account);
 
 		if (conv == NULL)
 			conv = gaim_conversation_new(GAIM_CONV_IM, account, pouncee);
 	}
 
-	if (gaim_pounce_action_is_enabled(pounce, "popup-notify")) {
+	if (gaim_pounce_action_is_enabled(pounce, "popup-notify"))
+	{
 		char tmp[1024];
+		const char *name_shown;
+		char *escaped;
 
+		escaped = g_markup_escape_text(alias, -1);
+
+		/*
+		 * Here we place the protocol name in the pounce dialog to lessen
+		 * confusion about what protocol a pounce is for.
+		 */
 		g_snprintf(tmp, sizeof(tmp),
-				   (events & GAIM_POUNCE_TYPING) ? _("%s has started typing to you") :
-				   (events & GAIM_POUNCE_SIGNON) ? _("%s has signed on") :
-				   (events & GAIM_POUNCE_IDLE_RETURN) ? _("%s has returned from being idle") :
-				   (events & GAIM_POUNCE_AWAY_RETURN) ? _("%s has returned from being away") :
-				   (events & GAIM_POUNCE_TYPING_STOPPED) ? _("%s has stopped typing to you") :
-				   (events & GAIM_POUNCE_SIGNOFF) ? _("%s has signed off") :
-				   (events & GAIM_POUNCE_IDLE) ? _("%s has become idle") :
-				   (events & GAIM_POUNCE_AWAY) ? _("%s has gone away.") :
+				   (events & GAIM_POUNCE_TYPING) ?
+				   _("%s has started typing to you (%s)") :
+				   (events & GAIM_POUNCE_SIGNON) ?
+				   _("%s has signed on (%s)") :
+				   (events & GAIM_POUNCE_IDLE_RETURN) ?
+				   _("%s has returned from being idle (%s)") :
+				   (events & GAIM_POUNCE_AWAY_RETURN) ?
+				   _("%s has returned from being away (%s)") :
+				   (events & GAIM_POUNCE_TYPING_STOPPED) ?
+				   _("%s has stopped typing to you (%s)") :
+				   (events & GAIM_POUNCE_SIGNOFF) ?
+				   _("%s has signed off (%s)") :
+				   (events & GAIM_POUNCE_IDLE) ?
+				   _("%s has become idle (%s)") :
+				   (events & GAIM_POUNCE_AWAY) ?
+				   _("%s has gone away. (%s)") :
 				   _("Unknown pounce event. Please report this!"),
-				   pouncee);
+				   escaped, gaim_account_get_protocol_name(account));
 
-		gaim_notify_info(NULL, NULL, tmp, NULL);
+		g_free(escaped);
+
+		/*
+		 * Ok here is where I change the second argument, title, from
+		 * NULL to the account name if that's all we have or the account
+		 * alias if we have that
+		 */
+		if ((name_shown = gaim_account_get_alias(account)) == NULL)
+			name_shown = gaim_account_get_username(account);
+
+		gaim_notify_info(NULL, name_shown, tmp, gaim_date_full());
 	}
 
-	if (gaim_pounce_action_is_enabled(pounce, "send-message")) {
+	if (gaim_pounce_action_is_enabled(pounce, "send-message"))
+	{
 		const char *message;
 
 		message = gaim_pounce_action_get_attribute(pounce, "send-message",
 												   "message");
 
-		if (message != NULL) {
+		if (message != NULL)
+		{
 			conv = gaim_find_conversation_with_account(pouncee, account);
 
 			if (conv == NULL)
 				conv = gaim_conversation_new(GAIM_CONV_IM, account, pouncee);
 
-			gaim_conversation_write(conv, NULL, message, -1,
+			gaim_conversation_write(conv, NULL, message,
 									GAIM_MESSAGE_SEND, time(NULL));
 
-			serv_send_im(account->gc, (char *)pouncee, (char *)message, -1, 0);
+			serv_send_im(account->gc, (char *)pouncee, (char *)message, 0);
 		}
 	}
 
-	if (gaim_pounce_action_is_enabled(pounce, "execute-command")) {
+	if (gaim_pounce_action_is_enabled(pounce, "execute-command"))
+	{
 		const char *command;
 
-		command = gaim_pounce_action_get_attribute(pounce, "execute-command",
-												   "command");
+		command = gaim_pounce_action_get_attribute(pounce,
+				"execute-command", "command");
 
-		if (command != NULL) {
+		if (command != NULL)
+		{
 #ifndef _WIN32
-			int pid = fork();
+			char *localecmd = g_locale_from_utf8(command, -1, NULL,
+					NULL, NULL);
 
-			if (pid == 0) {
-				char *args[4];
+			if (localecmd != NULL)
+			{
+				int pid = fork();
 
-				args[0] = "sh";
-				args[1] = "-c";
-				args[2] = (char *)command;
-				args[3] = NULL;
+				if (pid == 0) {
+					char *args[4];
 
-				execvp(args[0], args);
+					args[0] = "sh";
+					args[1] = "-c";
+					args[2] = (char *)localecmd;
+					args[3] = NULL;
 
-				_exit(0);
+					execvp(args[0], args);
+
+					_exit(0);
+				}
+				g_free(localecmd);
 			}
-#else
-			STARTUPINFO StartInfo;
-			PROCESS_INFORMATION ProcInfo;
+#else /* !_WIN32 */
+			PROCESS_INFORMATION pi;
+			BOOL retval;
+			gchar *message = NULL;
 
-			memset(&ProcInfo, 0, sizeof(ProcInfo));
-			memset(&StartInfo, 0 , sizeof(StartInfo));
-			StartInfo.cb = sizeof(StartInfo);
-			CreateProcess(NULL, (char *)command, NULL, NULL, 0, 0, NULL, NULL, &StartInfo, &ProcInfo);
-                        gaim_debug(GAIM_DEBUG_INFO, "pounce", "Pounce execute command called for: %s\n", command);
+			memset(&pi, 0, sizeof(pi));
 
-#endif
+			if (G_WIN32_HAVE_WIDECHAR_API ()) {
+				STARTUPINFOW si;
+				wchar_t *wc_cmd = g_utf8_to_utf16(command,
+						-1, NULL, NULL, NULL);
+
+				memset(&si, 0 , sizeof(si));
+				si.cb = sizeof(si);
+
+				retval = CreateProcessW(NULL, wc_cmd, NULL,
+						NULL, 0, 0, NULL, NULL,
+						&si, &pi);
+				g_free(wc_cmd);
+			} else {
+				STARTUPINFOA si;
+				char *l_cmd = g_locale_from_utf8(command,
+						-1, NULL, NULL, NULL);
+
+				memset(&si, 0 , sizeof(si));
+				si.cb = sizeof(si);
+
+				retval = CreateProcessA(NULL, l_cmd, NULL,
+						NULL, 0, 0, NULL, NULL,
+						&si, &pi);
+				g_free(l_cmd);
+			}
+
+			if (retval) {
+				CloseHandle(pi.hProcess);
+				CloseHandle(pi.hThread);
+			} else {
+				message = g_win32_error_message(GetLastError());
+			}
+
+			gaim_debug_info("pounce",
+					"Pounce execute command called for: "
+					"%s\n%s%s%s",
+						command,
+						retval ? "" : "Error: ",
+						retval ? "" : message,
+						retval ? "" : "\n");
+			g_free(message);
+#endif /* !_WIN32 */
 		}
 	}
 
-	if (gaim_pounce_action_is_enabled(pounce, "play-sound")) {
+	if (gaim_pounce_action_is_enabled(pounce, "play-sound"))
+	{
 		const char *sound;
 
-		sound = gaim_pounce_action_get_attribute(pounce, "play-sound",
-												 "filename");
+		sound = gaim_pounce_action_get_attribute(pounce,
+												 "play-sound", "filename");
 
 		if (sound != NULL)
 			gaim_sound_play_file(sound);
@@ -918,8 +1112,8 @@ pounce_cb(GaimPounce *pounce, GaimPounceEvent events, void *data)
 static void
 free_pounce(GaimPounce *pounce)
 {
-	struct gaim_buddy_list *blist;
-	struct gaim_gtk_buddy_list *gtkblist;
+	GaimBuddyList *blist;
+	GaimGtkBuddyList *gtkblist;
 
 	/* Rebuild the pounce menu */
 	blist = gaim_get_blist();
@@ -947,4 +1141,17 @@ gaim_gtk_pounces_init(void)
 {
 	gaim_pounces_register_handler(GAIM_GTK_UI, pounce_cb, new_pounce,
 								  free_pounce);
+
+	gaim_prefs_add_none("/gaim/gtk/pounces");
+	gaim_prefs_add_none("/gaim/gtk/pounces/default_actions");
+	gaim_prefs_add_bool("/gaim/gtk/pounces/default_actions/open-window",
+						FALSE);
+	gaim_prefs_add_bool("/gaim/gtk/pounces/default_actions/popup-notify",
+						FALSE);
+	gaim_prefs_add_bool("/gaim/gtk/pounces/default_actions/send-message",
+						TRUE);
+	gaim_prefs_add_bool("/gaim/gtk/pounces/default_actions/execute-command",
+						FALSE);
+	gaim_prefs_add_bool("/gaim/gtk/pounces/default_actions/play-sound",
+						FALSE);
 }
