@@ -25,6 +25,7 @@
 #include "internal.h"
 #include "gtkgaim.h"
 
+#include "blist.h"
 #include "connection.h"
 #include "debug.h"
 #include "privacy.h"
@@ -185,6 +186,8 @@ build_allow_list(GaimGtkPrivacyDialog *dialog)
 
 	dialog->allow_store = gtk_list_store_new(1, G_TYPE_STRING);
 
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(dialog->allow_store), 0, GTK_SORT_ASCENDING);
+
 	widget = build_list(dialog, dialog->allow_store, &list);
 
 	dialog->allow_list = list;
@@ -202,6 +205,8 @@ build_block_list(GaimGtkPrivacyDialog *dialog)
 
 	dialog->block_store = gtk_list_store_new(1, G_TYPE_STRING);
 
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(dialog->block_store), 0, GTK_SORT_ASCENDING);
+
 	widget = build_list(dialog, dialog->block_store, &list);
 
 	dialog->block_list = list;
@@ -238,6 +243,10 @@ select_account_cb(GtkWidget *dropdown, GaimAccount *account,
 	rebuild_block_list(dialog);
 }
 
+/*
+ * TODO: Setting the permit/deny setting needs to go through privacy.c.
+ *       Even better: the privacy API needs to not suck.
+ */
 static void
 type_changed_cb(GtkOptionMenu *optmenu, GaimGtkPrivacyDialog *dialog)
 {
@@ -260,6 +269,8 @@ type_changed_cb(GtkOptionMenu *optmenu, GaimGtkPrivacyDialog *dialog)
 		gtk_widget_show(dialog->button_box);
 		dialog->in_allow_list = FALSE;
 	}
+
+	gaim_blist_sync();
 }
 
 static void
