@@ -131,6 +131,21 @@ static void gaim_gtk_sound_shutdown(void)
 	sound_initialized = FALSE;
 }
 
+#ifdef USE_NAS_AUDIO
+static gboolean play_file_nas(const char *filename)
+{
+	AuServer *nas_serv;
+	gboolean ret = FALSE;
+
+	if((nas_serv = AuOpenServer(NULL, 0, NULL, 0, NULL, NULL))) {
+		ret = AuSoundPlaySynchronousFromFile(nas_serv, filename, 100);
+		AuCloseServer(nas_serv);
+	}
+
+	return ret;
+}
+#endif /* USE_NAS_AUDIO */
+
 static void gaim_gtk_sound_play_file(const char *filename)
 {
 	const char *method;
@@ -264,8 +279,19 @@ static void gaim_gtk_sound_play_file(const char *filename)
 #else /* _WIN32 */
 	gaim_debug(GAIM_DEBUG_INFO, "sound", "Playing %s\n", filename);
 
-	if (!PlaySound(filename, 0, SND_ASYNC | SND_FILENAME))
-		gaim_debug(GAIM_DEBUG_ERROR, "sound", "Error playing sound.\n");
+	if (G_WIN32_HAVE_WIDECHAR_API ()) {
+		wchar_t *wc_filename = g_utf8_to_utf16(filename,
+				-1, NULL, NULL, NULL);
+		if (!PlaySoundW(wc_filename, NULL, SND_ASYNC | SND_FILENAME))
+			gaim_debug(GAIM_DEBUG_ERROR, "sound", "Error playing sound.\n");
+		g_free(wc_filename);
+	} else {
+		char *l_filename = g_locale_from_utf8(filename,
+				-1, NULL, NULL, NULL);
+		if (!PlaySoundA(l_filename, NULL, SND_ASYNC | SND_FILENAME))
+			gaim_debug(GAIM_DEBUG_ERROR, "sound", "Error playing sound.\n");
+		g_free(l_filename);
+	}
 #endif /* _WIN32 */
 }
 
@@ -348,22 +374,6 @@ static void _pref_sound_method_changed(const char *name, GaimPrefType type,
 #endif /* USE_NAS */
 }
 
-#ifdef USE_NAS_AUDIO
-static gboolean play_file_nas(const char *filename)
-{
-	AuServer *nas_serv;
-	gboolean ret = FALSE;
-
-	if((nas_serv = AuOpenServer(NULL, 0, NULL, 0, NULL, NULL))) {
-		ret = AuSoundPlaySynchronousFromFile(nas_serv, filename, 100);
-		AuCloseServer(nas_serv);
-	}
-
-	return ret;
-}
-
-#endif /* USE_NAS_AUDIO */
-
 void gaim_gtk_sound_set_mute(gboolean mute)
 {
 	mute_sounds = mute;
