@@ -755,13 +755,16 @@ static void
 write_action_parameter(gpointer key, gpointer value, gpointer user_data)
 {
 	const char *name, *param_value;
+	char *escaped;
 	FILE *fp;
 
 	param_value = (const char *)value;
 	name        = (const char *)key;
 	fp          = (FILE *)user_data;
 
-	fprintf(fp, "    <param name='%s'>%s</param>\n", name, param_value);
+	escaped = g_markup_escape_text(param_value, -1);
+	fprintf(fp, "    <param name='%s'>%s</param>\n", name, escaped);
+	g_free(escaped);
 }
 
 static void
@@ -863,16 +866,16 @@ gaim_pounces_sync(void)
 
 	gaim_debug(GAIM_DEBUG_INFO, "pounces", "Writing pounces to disk.\n");
 
-	fp = fopen(user_dir, "r");
+	fp = g_fopen(user_dir, "r");
 
 	if (fp == NULL)
-		mkdir(user_dir, S_IRUSR | S_IWUSR | S_IXUSR);
+		g_mkdir(user_dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	else
 		fclose(fp);
 
 	filename = g_build_filename(user_dir, "pounces.xml.save", NULL);
 
-	if ((fp = fopen(filename, "w")) != NULL) {
+	if ((fp = g_fopen(filename, "w")) != NULL) {
 		GList *l;
 
 		fprintf(fp, "<?xml version='1.0' encoding='UTF-8' ?>\n\n");
@@ -893,16 +896,16 @@ gaim_pounces_sync(void)
 		return;
 	}
 
-	if (stat(filename, &st) || (st.st_size == 0)) {
+	if (g_stat(filename, &st) || (st.st_size == 0)) {
 		gaim_debug_error("pounces", "Failed to save pounces\n");
-		unlink(filename);
+		g_unlink(filename);
 		g_free(filename);
 		return;
 	}
 
 	filename_real = g_build_filename(user_dir, "pounces.xml", NULL);
 
-	if (rename(filename, filename_real) < 0) {
+	if (g_rename(filename, filename_real) < 0) {
 		gaim_debug(GAIM_DEBUG_ERROR, "pounces", "Error renaming %s to %s\n",
 				   filename, filename_real);
 	}
