@@ -1,6 +1,6 @@
 /*
  * gaim - Gadu-Gadu Protocol Plugin
- * $Id: gg.c,v 1.116.2.1 2004/09/03 21:34:18 faceprint Exp $
+ * $Id: gg.c,v 1.116.2.3 2005/04/25 03:35:46 datallah Exp $
  *
  * Copyright (C) 2001 Arkadiusz Mikiewicz <misiek@pld.ORG.PL>
  *
@@ -480,8 +480,7 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 			imsg = charset_convert(e->event.msg.message, "CP1250", "UTF-8");
 			gaim_str_strip_cr(imsg);
 			jmsg = gaim_escape_html(imsg);
-			/* e->event.msg.time - we don't know what this time is for */
-			serv_got_im(gc, user, jmsg, 0, time(NULL));
+			serv_got_im(gc, user, jmsg, 0, e->event.msg.time);
 			g_free(imsg);
 			g_free(jmsg);
 		}
@@ -836,7 +835,6 @@ static void agg_close(GaimConnection *gc)
 static int agg_send_im(GaimConnection *gc, const char *who, const char *msg, GaimConvImFlags flags)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
-	gchar *imsg;
 
 	if (invalid_uin(who)) {
 		gaim_notify_error(gc, NULL,
@@ -846,11 +844,14 @@ static int agg_send_im(GaimConnection *gc, const char *who, const char *msg, Gai
 	}
 
 	if (strlen(msg) > 0) {
-		imsg = charset_convert(msg, "UTF-8", "CP1250");
-		if (gg_send_message(gd->sess, GG_CLASS_CHAT,
+		gchar *imsg = charset_convert(msg, "UTF-8", "CP1250");
+		if (imsg != NULL && strlen(imsg) > 0) {
+			if (gg_send_message(gd->sess, GG_CLASS_CHAT,
 				    strtol(who, (char **)NULL, 10), imsg) < 0)
-			return -1;
-		g_free(imsg);
+				return -1;
+		}
+		if (imsg != NULL)
+			g_free(imsg);
 	}
 	return 1;
 }
