@@ -1,4 +1,4 @@
-/* $Id: libgg.h,v 1.11 2004/04/23 17:24:19 lschiere Exp $ */
+/* $Id: libgg.h,v 1.16 2004/08/25 03:09:08 thekingant Exp $ */
 
 /*
  *  (C) Copyright 2001 Wojtek Kaniewski <wojtekka@irc.pl>,
@@ -29,7 +29,9 @@ extern "C" {
   #define INADDR_NONE 0xffffffff
 #endif
 
-#include <stdint.h>
+#ifdef HAVE_STDINT_H
+#  include <stdint.h>
+#endif
 #include <sys/types.h>
 
 /*
@@ -160,6 +162,7 @@ void gg_free_session(struct gg_session *sess);
 void gg_logoff(struct gg_session *sess);
 int gg_write(struct gg_session *sess, const char *buf, int length);
 int gg_change_status(struct gg_session *sess, int status);
+int gg_change_status_descr(struct gg_session *sess, int status, const char *descr);
 int gg_send_message(struct gg_session *sess, int msgclass, uin_t recipient, const unsigned char *message);
 int gg_send_message_richtext(struct gg_session *sess, int msgclass, uin_t recipient, const unsigned char *message, const unsigned char *format, int formatlen);
 int gg_ping(struct gg_session *sess);
@@ -230,7 +233,7 @@ enum {
 	GG_EVENT_CONN_SUCCESS,
 	GG_EVENT_STATUS60,		/* kto zmieni stan w GG 6.0 */
 	GG_EVENT_NOTIFY60,		/* kto si pojawi w GG 6.0 */
-	GG_EVENT_USERLIST,		/* odpowied listy kontaktw w GG 6.0 */
+	GG_EVENT_USERLIST		/* odpowied listy kontaktw w GG 6.0 */
 };
 
 /*
@@ -446,16 +449,17 @@ void gg_chomp(char *line);
 char *gg_urlencode(const char *str);
 int gg_http_hash(const unsigned char *email, const unsigned char *password);
 
-#define GG_APPMSG_HOST "appmsg.gadugadu.pl"
+#define GG_APPMSG_HOST "appmsg.gadu-gadu.pl"
 #define GG_APPMSG_PORT 80
-#define GG_PUBDIR_HOST "pubdir.gadugadu.pl"
+#define GG_PUBDIR_HOST "pubdir.gadu-gadu.pl"
 #define GG_PUBDIR_PORT 80
-#define GG_REGISTER_HOST "register.gadugadu.pl"
+#define GG_REGISTER_HOST "register.gadu-gadu.pl"
 #define GG_REGISTER_PORT 80
 #define GG_DEFAULT_PORT 8074
 #define GG_HTTPS_PORT 443
 #define GG_HTTP_USERAGENT "Mozilla/4.0 (compatible MSIE 5.0; Windows 98; I)"
 #define GG_HAS_AUDIO_MASK 0x40000000
+#define GG_DEFAULT_CLIENT_VERSION "6, 0, 0, 132"
 
 struct gg_header {
 	unsigned long type;		/* typ pakietu */
@@ -529,6 +533,8 @@ __attribute__ ((packed))
 
 #define GG_STATUS_FRIENDS_MASK 0x8000		/* tylko dla znajomych (4.6) */
 
+#define GG_STATUS_DESCR_MAXSIZE 70
+
 /* GG_S() stan bez uwzgldnienia trybu tylko dla znajomych */
 #define GG_S(x) ((x) & ~GG_STATUS_FRIENDS_MASK)
 
