@@ -49,6 +49,7 @@ static void irc_login_cb(gpointer data, gint source, GaimInputCondition cond);
 static void irc_close(GaimConnection *gc);
 static int irc_im_send(GaimConnection *gc, const char *who, const char *what, GaimConvImFlags flags);
 static int irc_chat_send(GaimConnection *gc, int id, const char *what);
+static void irc_ping_server(GaimConnection *gc);
 static void irc_chat_join (GaimConnection *gc, GHashTable *data);
 static void irc_input_cb(gpointer data, gint source, GaimInputCondition cond);
 
@@ -497,6 +498,16 @@ static int irc_chat_send(GaimConnection *gc, int id, const char *what)
 	return 0;
 }
 
+static void irc_ping_server(GaimConnection *gc)
+{
+	struct irc_conn *irc = gc->proto_data;
+	gchar *buf;
+
+	buf = irc_format(irc, "vv", "PING", irc->server);
+	irc_send(irc, buf);
+	g_free(buf);
+}
+
 static guint irc_nick_hash(const char *nick)
 {
 	char *lc;
@@ -587,6 +598,27 @@ static void irc_roomlist_cancel(GaimRoomlist *list)
 	}
 }
 
+static GaimPluginPrefFrame *
+irc_pref_frame(GaimPlugin *plugin) {
+	GaimPluginPrefFrame *frame;
+	GaimPluginPref *ppref;
+
+	frame = gaim_plugin_pref_frame_new();
+
+	ppref = gaim_plugin_pref_new_with_label(_("IRC"));
+	gaim_plugin_pref_frame_add(frame, ppref);
+
+	ppref = gaim_plugin_pref_new_with_name_and_label("/plugins/prpl/irc/quitmsg",
+													 _("Quit message"));
+	gaim_plugin_pref_frame_add(frame, ppref);
+
+	return frame;
+}
+
+static GaimPluginUiInfo prefs_info = {
+	irc_pref_frame
+};
+
 static GaimPluginProtocolInfo prpl_info =
 {
 	OPT_PROTO_CHAT_TOPIC | OPT_PROTO_PASSWORD_OPTIONAL,
@@ -627,7 +659,7 @@ static GaimPluginProtocolInfo prpl_info =
 	irc_chat_leave,			/* chat_leave */
 	NULL,					/* chat_whisper */
 	irc_chat_send,			/* chat_send */
-	NULL,					/* keepalive */
+	irc_ping_server,		/* keepalive */
 	NULL,					/* register_user */
 	NULL,					/* get_cb_info */
 	NULL,					/* get_cb_away */
@@ -636,7 +668,7 @@ static GaimPluginProtocolInfo prpl_info =
 	NULL,					/* rename_group */
 	NULL,					/* buddy_free */
 	NULL,					/* convo_closed */
-	NULL,					/* normalize */
+	gaim_normalize_nocase,	/* normalize */
 	NULL,					/* set_buddy_icon */
 	NULL,					/* remove_group */
 	NULL,					/* get_cb_real_name */
@@ -675,7 +707,7 @@ static GaimPluginInfo info =
 
 	NULL,                                             /**< ui_info        */
 	&prpl_info,                                       /**< extra_info     */
-	NULL,
+	&prefs_info,                                      /**< prefs_info     */
 	irc_actions
 };
 
@@ -701,6 +733,9 @@ static void _init_plugin(GaimPlugin *plugin)
 
 	_irc_plugin = plugin;
 
+	gaim_prefs_add_none("/plugins/prpl/irc");
+	gaim_prefs_add_string("/plugins/prpl/irc/quitmsg", IRC_DEFAULT_QUIT);
+
 	irc_register_commands();
 }
 
