@@ -56,11 +56,15 @@ static char *irc_mask_userhost(const char *mask)
 
 static void irc_chat_remove_buddy(GaimConversation *convo, char *data[2])
 {
-	char *message = g_strdup_printf("quit: %s", data[1]);
+	char *escaped, *message;
+
+	escaped = g_markup_escape_text(data[1], -1);
+	message = g_strdup_printf("quit: %s", escaped);
 
 	if (gaim_conv_chat_find_user(GAIM_CONV_CHAT(convo), data[0]))
 		gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo), data[0], message);
 
+	g_free(escaped);
 	g_free(message);
 }
 
@@ -72,6 +76,7 @@ void irc_msg_default(struct irc_conn *irc, const char *name, const char *from, c
 void irc_msg_away(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc;
+	char *msg;
 
 	if (!args || !args[1])
 		return;
@@ -83,37 +88,66 @@ void irc_msg_away(struct irc_conn *irc, const char *name, const char *from, char
 	}
 
 	gc = gaim_account_get_connection(irc->account);
-	if (gc)
-		serv_got_im(gc, args[1], args[2], GAIM_CONV_IM_AUTO_RESP, time(NULL));
+	if (gc) {
+		msg = g_markup_escape_text(args[2], -1);
+		serv_got_im(gc, args[1], msg, GAIM_CONV_IM_AUTO_RESP, time(NULL));
+		g_free(msg);
+	}
 }
 
 void irc_msg_badmode(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
+	char *escaped;
 
 	if (!args || !args[1] || !gc)
 		return;
 
-	gaim_notify_error(gc, NULL, _("Bad mode"), args[1]);
+	escaped = g_markup_escape_text(args[1], -1);
+	gaim_notify_error(gc, NULL, _("Bad mode"), escaped);
+	g_free(escaped);
 }
 
 void irc_msg_banned(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
-	char *buf;
+	char *buf, *escaped;
 
 	if (!args || !args[1] || !gc)
 		return;
 
-	buf = g_strdup_printf(_("You are banned from %s."), args[1]);
+	escaped = g_markup_escape_text(args[1], -1);
+	buf = g_strdup_printf(_("You are banned from %s."), escaped);
 	gaim_notify_error(gc, _("Banned"), _("Banned"), buf);
+	g_free(escaped);
+	g_free(buf);
+}
+
+void irc_msg_banfull(struct irc_conn *irc, const char *name, const char *from, char **args)
+{
+	GaimConversation *convo;
+	char *buf, *nick;
+
+	if (!args || !args[0] || !args[1] || !args[2])
+		return;
+
+	convo = gaim_find_conversation_with_account(args[1], irc->account);
+	if (!convo)
+		return;
+
+	nick = g_markup_escape_text(args[2], -1);
+	buf = g_strdup_printf(_("Cannot ban %s: banlist is full"), nick);
+	g_free(nick);
+	gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", buf,
+			     GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG,
+			     time(NULL));
 	g_free(buf);
 }
 
 void irc_msg_chanmode(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConversation *convo;
-	char *buf;
+	char *buf, *escaped;
 
 	if (!args || !args[1] || !args[2])
 		return;
@@ -122,8 +156,10 @@ void irc_msg_chanmode(struct irc_conn *irc, const char *name, const char *from,
 	if (!convo)	/* XXX punt on channels we are not in for now */
 		return;
 
-	buf = g_strdup_printf("mode for %s: %s %s", args[1], args[2], args[3] ? args[3] : "");
+	escaped = (args[3] != NULL) ? g_markup_escape_text(args[3], -1) : NULL;
+	buf = g_strdup_printf("mode for %s: %s %s", args[1], args[2], escaped ? escaped : "");
 	gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", buf, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+	g_free(escaped);
 	g_free(buf);
 
 	return;
@@ -167,7 +203,7 @@ void irc_msg_endwhois(struct irc_conn *irc, const char *name, const char *from,
 	GaimConnection *gc;
 	GString *info;
 	char buffer[256];
-	char *str;
+	char *str, *tmp;
 
 	if (!irc->whois.nick) {
 		gaim_debug(GAIM_DEBUG_WARNING, "irc", "Unexpected End of WHOIS for %s\n", args[1]);
@@ -179,18 +215,20 @@ void irc_msg_endwhois(struct irc_conn *irc, const char *name, const char *from,
 	}
 
 	info = g_string_new("");
-	g_string_append_printf(info, _("<b>%s:</b> %s"), _("Nick"), args[1]);
+	tmp = g_markup_escape_text(args[1], -1);
+	g_string_append_printf(info, _("<b>%s:</b> %s"), _("Nick"), tmp);
+	g_free(tmp);
 	g_string_append_printf(info, "%s%s<br>",
 			       irc->whois.ircop ? _(" <i>(ircop)</i>") : "",
 			       irc->whois.identified ? _(" <i>(identified)</i>") : "");
 	if (irc->whois.away) {
-		char *tmp = g_markup_escape_text(irc->whois.away, strlen(irc->whois.away));
+		tmp = g_markup_escape_text(irc->whois.away, strlen(irc->whois.away));
 		g_free(irc->whois.away);
 		g_string_append_printf(info, _("<b>%s:</b> %s<br>"), _("Away"), tmp);
 		g_free(tmp);
 	}
 	if (irc->whois.userhost) {
-		char *tmp = g_markup_escape_text(irc->whois.name, strlen(irc->whois.name));
+		tmp = g_markup_escape_text(irc->whois.name, strlen(irc->whois.name));
 		g_free(irc->whois.name);
 		g_string_append_printf(info, _("<b>%s:</b> %s<br>"), _("Username"), irc->whois.userhost);
 		g_string_append_printf(info, _("<b>%s:</b> %s<br>"), _("Realname"), tmp);
@@ -301,13 +339,15 @@ void irc_msg_topic(struct irc_conn *irc, const char *name, const char *from, cha
 void irc_msg_unknown(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
-	char *buf;
+	char *buf, *escaped;
 
 	if (!args || !args[1] || !gc)
 		return;
 
-	buf = g_strdup_printf(_("Unknown message '%s'"), args[1]);
+	escaped = g_markup_escape_text(args[1], -1);
+	buf = g_strdup_printf(_("Unknown message '%s'"), escaped);
 	gaim_notify_error(gc, _("Unknown message"), buf, _("Gaim has sent a message the IRC server did not understand."));
+	g_free(escaped);
 	g_free(buf);
 }
 
@@ -330,7 +370,7 @@ void irc_msg_names(struct irc_conn *irc, const char *name, const char *from, cha
 		names = cur = g_string_free(irc->names, FALSE);
 		irc->names = NULL;
 		if (irc->nameconv) {
-			msg = g_strdup_printf(_("Users on %s: %s"), args[1], names);
+			msg = g_strdup_printf(_("Users on %s: %s"), args[1], names ? names : "");
 			if (gaim_conversation_get_type(convo) == GAIM_CONV_CHAT)
 				gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", msg, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 			else
@@ -389,6 +429,7 @@ void irc_msg_names(struct irc_conn *irc, const char *name, const char *from, cha
 void irc_msg_motd(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc;
+	char *escaped;
 	if (!strcmp(name, "375")) {
 		gc = gaim_account_get_connection(irc->account);
 		if (gc)
@@ -398,7 +439,9 @@ void irc_msg_motd(struct irc_conn *irc, const char *name, const char *from, char
 	if (!irc->motd)
 		irc->motd = g_string_new("");
 
-	g_string_append_printf(irc->motd, "%s<br>", args[1]);
+	escaped = g_markup_escape_text(args[1], -1);
+	g_string_append_printf(irc->motd, "%s<br>", escaped);
+	g_free(escaped);
 }
 
 void irc_msg_endmotd(struct irc_conn *irc, const char *name, const char *from, char **args)
@@ -417,20 +460,40 @@ void irc_msg_endmotd(struct irc_conn *irc, const char *name, const char *from, c
 		irc->timer = gaim_timeout_add(45000, (GSourceFunc)irc_blist_timeout, (gpointer)irc);
 }
 
+void irc_msg_time(struct irc_conn *irc, const char *name, const char *from, char **args)
+{
+	GaimConnection *gc;
+	char *time;
+
+	gc = gaim_account_get_connection(irc->account);
+	if (gc == NULL || args == NULL || args[2] == NULL)
+		return;
+
+	time = g_markup_escape_text(args[2], -1);
+	gaim_notify_message(gc, GAIM_NOTIFY_MSG_INFO, _("Time Response"),
+			    _("The IRC server's local time is:"),
+			    time, NULL, NULL);
+	g_free(time);
+}
+
 void irc_msg_nochan(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
+	char *escaped;
 
 	if (gc == NULL || args == NULL || args[1] == NULL)
 		return;
 
-	gaim_notify_error(gc, NULL, _("No such channel"), args[1]);
+	escaped = g_markup_escape_text(args[1], -1);
+	gaim_notify_error(gc, NULL, _("No such channel"), escaped);
+	g_free(escaped);
 }
 
 void irc_msg_nonick(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc;
 	GaimConversation *convo;
+	char *nick;
 
 	convo = gaim_find_conversation_with_account(args[1], irc->account);
 	if (convo) {
@@ -443,7 +506,9 @@ void irc_msg_nonick(struct irc_conn *irc, const char *name, const char *from, ch
 	} else {
 		if ((gc = gaim_account_get_connection(irc->account)) == NULL)
 			return;
-		gaim_notify_error(gc, NULL, _("No such nick or channel"), args[1]);
+		nick = g_markup_escape_text (args[1], -1);
+		gaim_notify_error(gc, NULL, _("No such nick or channel"), nick);
+		g_free (nick);
 	}
 
 	if (irc->whois.nick && !gaim_utf8_strcasecmp(irc->whois.nick, args[1])) {
@@ -609,7 +674,7 @@ void irc_msg_kick(struct irc_conn *irc, const char *name, const char *from, char
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
 	GaimConversation *convo = gaim_find_conversation_with_account(args[0], irc->account);
-	char *nick = irc_mask_nick(from), *buf;
+	char *nick = irc_mask_nick(from), *buf, *reason;
 
 	if (!gc) {
 		g_free(nick);
@@ -622,17 +687,19 @@ void irc_msg_kick(struct irc_conn *irc, const char *name, const char *from, char
 		return;
 	}
 
+	reason = g_markup_escape_text(args[2], -1);
 	if (!gaim_utf8_strcasecmp(gaim_connection_get_display_name(gc), args[1])) {
-		buf = g_strdup_printf(_("You have been kicked by %s: (%s)"), nick, args[2]);
+		buf = g_strdup_printf(_("You have been kicked by %s: (%s)"), nick, reason);
 		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[0], buf, GAIM_MESSAGE_SYSTEM, time(NULL));
 		g_free(buf);
 		serv_got_chat_left(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)));
 	} else {
-		buf = g_strdup_printf(_("Kicked by %s (%s)"), nick, args[2]);
+		buf = g_strdup_printf(_("Kicked by %s (%s)"), nick, reason);
 		gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo), args[1], buf);
 		g_free(buf);
 	}
 
+	g_free(reason);
 	g_free(nick);
 	return;
 }
@@ -643,14 +710,17 @@ void irc_msg_mode(struct irc_conn *irc, const char *name, const char *from, char
 	char *nick = irc_mask_nick(from), *buf;
 
 	if (*args[0] == '#' || *args[0] == '&') {	/* Channel	*/
+		char *escaped;
 		convo = gaim_find_conversation_with_account(args[0], irc->account);
 		if (!convo) {
 			gaim_debug(GAIM_DEBUG_ERROR, "irc", "MODE received for %s, which we are not in\n", args[0]);
 			g_free(nick);
 			return;
 		}
-		buf = g_strdup_printf(_("mode (%s %s) by %s"), args[1], args[2] ? args[2] : "", nick);
+		escaped = (args[2] != NULL) ? g_markup_escape_text(args[2], -1) : NULL;
+		buf = g_strdup_printf(_("mode (%s %s) by %s"), args[1], escaped ? escaped : "", nick);
 		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[0], buf, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+		g_free(escaped);
 		g_free(buf);
 		if(args[2]) {
 			GaimConvChatBuddyFlags newflag, flags;
@@ -699,6 +769,7 @@ void irc_msg_mode(struct irc_conn *irc, const char *name, const char *from, char
 void irc_msg_nick(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
+	GaimConversation *conv;
 	GSList *chats;
 	char *nick = irc_mask_nick(from);
 
@@ -719,9 +790,33 @@ void irc_msg_nick(struct irc_conn *irc, const char *name, const char *from, char
 			gaim_conv_chat_rename_user(chat, nick, args[0]);
 		chats = chats->next;
 	}
+
+	conv = gaim_find_conversation_with_account(nick, irc->account);
+	if (conv != NULL)
+		gaim_conversation_set_name(conv, args[0]);
+
 	g_free(nick);
 }
 
+void irc_msg_badnick(struct irc_conn *irc, const char *name, const char *from, char **args)
+{
+	GaimConnection *gc = gaim_account_get_connection(irc->account);
+	char *escaped, *msg;
+
+	if (gc && gaim_connection_get_state(gc) == GAIM_CONNECTED) {
+		escaped = g_markup_escape_text(args[1], -1);
+		msg = g_strdup_printf(_("Invalid nickname '%s'"), escaped);
+		gaim_notify_error(irc->account, _("Invalid nickname"),
+				  msg,
+				  _("Your selected account name was rejected by the server.  It probably contains invalid characters."));
+		g_free(escaped);
+		g_free(msg);
+	} else {
+		gaim_connection_error(gc,
+				      _("Your selected account name was rejected by the server.  It probably contains invalid characters."));
+	}
+}
+
 void irc_msg_nickused(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	char *newnick, *buf, *end;
@@ -759,8 +854,8 @@ void irc_msg_nochangenick(struct irc_conn *irc, const char *name, const char *fr
 	if (!args || !args[2] || !gc)
 		return;
 
-	msg = g_strdup_printf(_("Could not change nick"));
-	gaim_notify_error(gc, _("Cannot change nick"), msg, args[2]);
+	msg = g_markup_escape_text(args[2], -1);
+	gaim_notify_error(gc, _("Cannot change nick"), _("Could not change nick"), msg);
 	g_free(msg);
 }
 
@@ -768,7 +863,7 @@ void irc_msg_part(struct irc_conn *irc, const char *name, const char *from, char
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
 	GaimConversation *convo;
-	char *nick, *msg;
+	char *nick, *msg, *escaped;
 
 	if (!args || !args[0] || !gc)
 		return;
@@ -779,16 +874,19 @@ void irc_msg_part(struct irc_conn *irc, const char *name, const char *from, char
 		return;
 	}
 
+	escaped = (args[1] && *args[1]) ? g_markup_escape_text(args[1], -1) : NULL;
 	nick = irc_mask_nick(from);
 	if (!gaim_utf8_strcasecmp(nick, gaim_connection_get_display_name(gc))) {
 		msg = g_strdup_printf(_("You have parted the channel%s%s"),
-                                      (args[1] && *args[1]) ? ": " : "", args[1]);
+                                      (args[1] && *args[1]) ? ": " : "",
+									  (escaped && *escaped) ? escaped : "");
 		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[0], msg, GAIM_MESSAGE_SYSTEM, time(NULL));
 		g_free(msg);
 		serv_got_chat_left(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)));
 	} else {
-		gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo), nick, args[1]);
+		gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo), nick, escaped);
 	}
+	g_free(escaped);
 	g_free(nick);
 }
 
@@ -893,14 +991,16 @@ void irc_msg_privmsg(struct irc_conn *irc, const char *name, const char *from, c
 void irc_msg_regonly(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
-	char *msg;
+	char *msg, *reason;
 
 	if (!args || !args[1] || !args[2] || !gc)
 		return;
 
 	msg = g_strdup_printf(_("Cannot join %s:"), args[1]);
-	gaim_notify_error(gc, _("Cannot join channel"), msg, args[2]);
+	reason = g_markup_escape_text(args[2], -1);
+	gaim_notify_error(gc, _("Cannot join channel"), msg, reason);
 	g_free(msg);
+	g_free(reason);
 }
 
 void irc_msg_quit(struct irc_conn *irc, const char *name, const char *from, char **args)
