@@ -26,6 +26,8 @@
 #include "conversation.h"
 #include "notify.h"
 #include "debug.h"
+#include "util.h"
+#include "cmds.h"
 #include "irc.h"
 
 #include <stdio.h>
@@ -69,16 +71,19 @@ static struct _irc_msg {
 	{ "372", "n:", irc_msg_motd },		/* MOTD				*/
 	{ "375", "n:", irc_msg_motd },		/* Start MOTD			*/
 	{ "376", "n:", irc_msg_endmotd },	/* End of MOTD			*/
+	{ "391", "nv:", irc_msg_time },		/* Time reply			*/
 	{ "401", "nt:", irc_msg_nonick },	/* No such nick/chan		*/
 	{ "403", "nc:", irc_msg_nochan },	/* No such channel		*/
 	{ "404", "nt:", irc_msg_nosend },	/* Cannot send to chan		*/
 	{ "421", "nv:", irc_msg_unknown },	/* Unknown command		*/
 	{ "422", "nv:", irc_msg_endmotd },	/* No MOTD available		*/
+	{ "432", "nn:", irc_msg_badnick },	/* Erroneous nickname		*/
 	{ "433", "vn:", irc_msg_nickused },	/* Nickname already in use	*/
 	{ "438", "nn:", irc_msg_nochangenick },	/* Nick may not change		*/
 	{ "442", "nc:", irc_msg_notinchan },	/* Not in channel		*/
 	{ "473", "nc:", irc_msg_inviteonly },	/* Tried to join invite-only	*/
 	{ "474", "nc:", irc_msg_banned },	/* Banned from channel		*/
+	{ "478", "nct:", irc_msg_banfull },	/* Banlist is full		*/
 	{ "482", "nc:", irc_msg_notop },	/* Need to be op to do that	*/
 	{ "501", "n:", irc_msg_badmode },	/* Unknown mode flag		*/
 	{ "506", "nc:", irc_msg_nosend },	/* Must identify to send	*/
@@ -103,71 +108,162 @@ static struct _irc_user_cmd {
 	char *name;
 	char *format;
 	IRCCmdCallback cb;
+	char *help;
 } _irc_cmds[] = {
-	{ "action", ":", irc_cmd_ctcp_action },
-	{ "away", ":", irc_cmd_away },
-	{ "deop", ":", irc_cmd_op },
-	{ "devoice", ":", irc_cmd_op },
-	{ "help", "v", irc_cmd_help },
-	{ "invite", ":", irc_cmd_invite },
-	{ "j", "cv", irc_cmd_join },
-	{ "join", "cv", irc_cmd_join },
-	{ "kick", "n:", irc_cmd_kick },
-	{ "list", ":", irc_cmd_list },
-	{ "me", ":", irc_cmd_ctcp_action },
-	{ "mode", ":", irc_cmd_mode },
-	{ "msg", "t:", irc_cmd_privmsg },
-	{ "names", "c", irc_cmd_names },
-	{ "nick", "n", irc_cmd_nick },
-	{ "op", ":", irc_cmd_op },
-	{ "operwall", ":", irc_cmd_wallops },
-	{ "part", "c:", irc_cmd_part },
-	{ "ping", "n", irc_cmd_ping },
-	{ "query", "n:", irc_cmd_query },
-	{ "quit", ":", irc_cmd_quit },
-	{ "quote", "*", irc_cmd_quote },
-	{ "remove", "n:", irc_cmd_remove },
-	{ "topic", ":", irc_cmd_topic },
-	{ "umode", ":", irc_cmd_mode },
-	{ "voice", ":", irc_cmd_op },
-	{ "wallops", ":", irc_cmd_wallops },
-	{ "whois", "n", irc_cmd_whois },
+	{ "action", ":", irc_cmd_ctcp_action, N_("action &lt;action to perform&gt;:  Perform an action.") },
+	{ "away", ":", irc_cmd_away, N_("away [message]:  Set an away message, or use no message to return from being away.") },
+	{ "deop", ":", irc_cmd_op, N_("deop &lt;nick1&gt; [nick2] ...:  Remove channel operator status from someone. You must be a channel operator to do this.") },
+	{ "devoice", ":", irc_cmd_op, N_("devoice &lt;nick1&gt; [nick2] ...:  Remove channel voice status from someone, preventing them from speaking if the channel is moderated (+m). You must be a channel operator to do this.") },
+	{ "invite", ":", irc_cmd_invite, N_("invite &lt;nick&gt; [room]:  Invite someone to join you in the specified channel, or the current channel.") },
+	{ "j", "cv", irc_cmd_join, N_("j &lt;room1&gt;[,room2][,...] [key1[,key2][,...]]:  Enter one or more channels, optionally providing a channel key for each if needed.") },
+	{ "join", "cv", irc_cmd_join, N_("join &lt;room1&gt;[,room2][,...] [key1[,key2][,...]]:  Enter one or more channels, optionally providing a channel key for each if needed.") },
+	{ "kick", "n:", irc_cmd_kick, N_("kick &lt;nick&gt; [message]:  Remove someone from a channel. You must be a channel operator to do this.") },
+	{ "list", ":", irc_cmd_list, N_("list:  Display a list of chat rooms on the network. <i>Warning, some servers may disconnect you upon doing this.</i>") },
+	{ "me", ":", irc_cmd_ctcp_action, N_("me &lt;action to perform&gt;:  Perform an action.") },
+	{ "mode", ":", irc_cmd_mode, N_("mode &lt;+|-&gt;&lt;A-Za-z&gt; &lt;nick|channel&gt;:  Set or unset a channel or user mode.") },
+	{ "msg", "t:", irc_cmd_privmsg, N_("msg &lt;nick&gt; &lt;message&gt;:  Send a private message to a user (as opposed to a channel).") },
+	{ "names", "c", irc_cmd_names, N_("names [channel]:  List the users currently in a channel.") },
+	{ "nick", "n", irc_cmd_nick, N_("nick &lt;new nickname&gt;:  Change your nickname.") },
+	{ "op", ":", irc_cmd_op, N_("op &lt;nick1&gt; [nick2] ...:  Grant channel operator status to someone. You must be a channel operator to do this.") },
+	{ "operwall", ":", irc_cmd_wallops, N_("operwall &lt;message&gt;:  If you don't know what this is, you probably can't use it.") },
+	{ "part", "c:", irc_cmd_part, N_("part [room] [message]:  Leave the current channel, or a specified channel, with an optional message.") },
+	{ "ping", "n", irc_cmd_ping, N_("ping [nick]:  Asks how much lag a user (or the server if no user specified) has.") },
+	{ "query", "n:", irc_cmd_query, N_("query &lt;nick&gt; &lt;message&gt;:  Send a private message to a user (as opposed to a channel).") },
+	{ "quit", ":", irc_cmd_quit, N_("quit [message]:  Disconnect from the server, with an optional message.") },
+	{ "quote", "*", irc_cmd_quote, N_("quote [...]:  Send a raw command to the server.") },
+	{ "remove", "n:", irc_cmd_remove, N_("remove &lt;nick&gt; [message]:  Remove someone from a room. You must be a channel operator to do this.") },
+	{ "time", "", irc_cmd_time, N_("time: Displays the current local time at the IRC server.") },
+	{ "topic", ":", irc_cmd_topic, N_("topic [new topic]:  View or change the channel topic.") },
+	{ "umode", ":", irc_cmd_mode, N_("umode &lt;+|-&gt;&lt;A-Za-z&gt;:  Set or unset a user mode.") },
+	{ "voice", ":", irc_cmd_op, N_("voice &lt;nick1&gt; [nick2] ...:  Grant channel voice status to someone. You must be a channel operator to do this.") },
+	{ "wallops", ":", irc_cmd_wallops, N_("wallops &lt;message&gt;:  If you don't know what this is, you probably can't use it.") },
+	{ "whois", "tt", irc_cmd_whois, N_("whois [server] &lt;nick&gt;:  Get information on a user.") },
 	{ NULL, NULL, NULL }
 };
 
+static GaimCmdRet irc_parse_gaim_cmd(GaimConversation *conv, const gchar *cmd,
+                                        gchar **args, gchar **error, void *data)
+{
+	GaimConnection *gc;
+	struct irc_conn *irc;
+	struct _irc_user_cmd *cmdent;
+
+	gc = gaim_conversation_get_gc(conv);
+	if (!gc)
+		return GAIM_CMD_RET_FAILED;
+
+	irc = gc->proto_data;
+
+	if ((cmdent = g_hash_table_lookup(irc->cmds, cmd)) == NULL)
+		return GAIM_CMD_RET_FAILED;
+
+	(cmdent->cb)(irc, cmd, gaim_conversation_get_name(conv), (const char **)args);
+
+	return GAIM_CMD_RET_OK;
+}
+
+static void irc_register_command(struct _irc_user_cmd *c)
+{
+	GaimCmdFlag f;
+	char args[10];
+	char *format;
+	int i;
+
+	f = GAIM_CMD_FLAG_CHAT | GAIM_CMD_FLAG_IM | GAIM_CMD_FLAG_PRPL_ONLY
+	    | GAIM_CMD_FLAG_ALLOW_WRONG_ARGS;
+
+	format = c->format;
+
+	for (i = 0; (i < (sizeof(args) - 1)) && *format; i++, format++)
+		switch (*format) {
+		case 'v':
+		case 'n':
+		case 'c':
+		case 't':
+			args[i] = 'w';
+			break;
+		case ':':
+		case '*':
+			args[i] = 's';
+			break;
+		}
+
+	args[i] = '\0';
+
+	gaim_cmd_register(c->name, args, GAIM_CMD_P_PRPL, f, "prpl-irc",
+	                  irc_parse_gaim_cmd, _(c->help), NULL);
+}
+
+void irc_register_commands(void)
+{
+	struct _irc_user_cmd *c;
+
+	for (c = _irc_cmds; c && c->name; c++)
+		irc_register_command(c);
+}
+
 static char *irc_send_convert(struct irc_conn *irc, const char *string)
 {
 	char *utf8;
 	GError *err = NULL;
-	
-	utf8 = g_convert(string, strlen(string), 
-			 gaim_account_get_string(irc->account, "encoding", IRC_DEFAULT_CHARSET),
-			 "UTF-8", NULL, NULL, &err);
+	gchar **encodings;
+	const gchar *enclist;
+
+	enclist = gaim_account_get_string(irc->account, "encoding", IRC_DEFAULT_CHARSET);
+	encodings = g_strsplit(enclist, ",", 2);
+
+	if (encodings[0] == NULL || !strcasecmp("UTF-8", encodings[0])) {
+		g_strfreev(encodings);
+		return g_strdup(string);
+	}
+
+	utf8 = g_convert(string, strlen(string), encodings[0], "UTF-8", NULL, NULL, &err);
 	if (err) {
-		gaim_debug(GAIM_DEBUG_ERROR, "irc", "send conversion error: %s\n", err->message);
-		gaim_debug(GAIM_DEBUG_ERROR, "irc", "Sending raw, which probably isn't right\n");
+		gaim_debug(GAIM_DEBUG_ERROR, "irc", "Send conversion error: %s\n", err->message);
+		gaim_debug(GAIM_DEBUG_ERROR, "irc", "Sending as UTF-8 instead of %s\n", encodings[0]);
 		utf8 = g_strdup(string);
 		g_error_free(err);
 	}
-	
+	g_strfreev(encodings);
+
 	return utf8;
 }
 
 static char *irc_recv_convert(struct irc_conn *irc, const char *string)
 {
-	char *utf8;
-	GError *err = NULL;
-	
-	utf8 = g_convert(string, strlen(string), "UTF-8",
-			 gaim_account_get_string(irc->account, "encoding", IRC_DEFAULT_CHARSET),
-			 NULL, NULL, &err);
-	if (err) {
-		gaim_debug(GAIM_DEBUG_ERROR, "irc", "recv conversion error: %s\n", err->message);
-		utf8 = g_strdup(_("(There was an error converting this message.  Check the 'Encoding' option in the Account Editor)"));
-		g_error_free(err);
+	char *utf8 = NULL;
+	const gchar *charset, *enclist;
+	gchar **encodings;
+	int i;
+
+	enclist = gaim_account_get_string(irc->account, "encoding", IRC_DEFAULT_CHARSET);
+	encodings = g_strsplit(enclist, ",", -1);
+
+	if (encodings[0] == NULL) {
+		g_strfreev(encodings);
+		return gaim_utf8_salvage(string);
 	}
-	
-	return utf8;
+
+	for (i = 0; encodings[i] != NULL; i++) {
+		charset = encodings[i];
+		while (*charset == ' ')
+			charset++;
+
+		if (!strcasecmp("UTF-8", charset)) {
+			if (g_utf8_validate(string, strlen(string), NULL))
+				utf8 = g_strdup(string);
+		} else {
+			utf8 = g_convert(string, strlen(string), "UTF-8", charset, NULL, NULL, NULL);
+		}
+
+		if (utf8) {
+			g_strfreev(encodings);
+			return utf8;
+		}
+	}
+	g_strfreev(encodings);
+
+	return gaim_utf8_salvage(string);
 }
 
 /* XXX tag closings are not necessarily correctly nested here!  If we
@@ -288,10 +384,15 @@ char *irc_mirc2txt (const char *string)
 			result[j++] = result[i];
 		}
 	}
-	result[i] = '\0';
+	result[j] = '\0';
         return result;
 }
 
+gboolean irc_ischannel(const char *string)
+{
+	return (string[0] == '#' || string[0] == '&');
+}
+
 char *irc_parse_ctcp(struct irc_conn *irc, const char *from, const char *to, const char *msg, int notice)
 {
 	GaimConnection *gc;
@@ -420,7 +521,12 @@ void irc_parse_msg(struct irc_conn *irc, char *input)
 		g_free(msg);
 		return;
 	} else if (!strncmp(input, "ERROR ", 6)) {
-		gaim_connection_error(gaim_account_get_connection(irc->account), _("Disconnected."));
+		if (g_utf8_validate(input, -1, NULL)) {
+			char *tmp = g_strdup_printf("%s\n%s", _("Disconnected."), input);
+			gaim_connection_error(gaim_account_get_connection(irc->account), tmp);
+			g_free(tmp);
+		} else
+			gaim_connection_error(gaim_account_get_connection(irc->account), _("Disconnected."));
 		return;
 	}
 
@@ -488,63 +594,6 @@ void irc_parse_msg(struct irc_conn *irc, char *input)
 	g_free(from);
 }
 
-int irc_parse_cmd(struct irc_conn *irc, const char *target, const char *cmdstr)
-{
-	const char *cur, *end, *fmt; 
-	char *tmp, *cmd, **args;
-	struct _irc_user_cmd *cmdent;
-	guint i;
-	int ret;
-
-	cur = cmdstr;
-	end = strchr(cmdstr, ' ');
-	if (!end)
-		end = cur + strlen(cur);
-
-	tmp = g_strndup(cur, end - cur);
-	cmd = g_utf8_strdown(tmp, -1);
-	g_free(tmp);
-
-	if ((cmdent = g_hash_table_lookup(irc->cmds, cmd)) == NULL) {
-		ret = irc_cmd_default(irc, cmd, target, &cmdstr);
-		g_free(cmd);
-		return ret;
-	}
-
-	args = g_new0(char *, strlen(cmdent->format));
-	for (cur = end, fmt = cmdent->format, i = 0; fmt[i] && *cur++; i++) {
-		switch (fmt[i]) {
-		case 'v':
-			if (!(end = strchr(cur, ' '))) end = cur + strlen(cur);
-			args[i] = g_strndup(cur, end - cur);
-			cur += end - cur;
-			break;
-		case 't':
-		case 'n':
-		case 'c':
-			if (!(end = strchr(cur, ' '))) end = cur + strlen(cur);
-			args[i] = g_strndup(cur, end - cur);
-			cur += end - cur;
-			break;
-		case ':':
-		case '*':
-			args[i] = g_strdup(cur);
-			cur = cur + strlen(cur);
-			break;
-		default:
-			gaim_debug(GAIM_DEBUG_ERROR, "irc", "invalid command format character '%c'\n", fmt[i]);
-			break;
-		}
-	}
-	ret = (cmdent->cb)(irc, cmd, target, (const char **)args);
-	for (i = 0; i < strlen(cmdent->format); i++)
-		g_free(args[i]);
-	g_free(args);
-
-	g_free(cmd);
-	return ret;
-}
-
 static void irc_parse_error_cb(struct irc_conn *irc, char *input)
 {
 	gaim_debug(GAIM_DEBUG_WARNING, "irc", "Unrecognized string: %s\n", input);
