@@ -21,7 +21,7 @@
 #include "internal.h"
 #include "debug.h"
 #include "imgstore.h"
-#include "multi.h"
+#include "prpl.h"
 #include "notify.h"
 #include "request.h"
 #include "util.h"
@@ -32,7 +32,6 @@
 #include "jabber.h"
 #include "iq.h"
 #include "presence.h"
-#include "si.h"
 
 
 void jabber_buddy_free(JabberBuddy *jb)
@@ -410,12 +409,13 @@ jabber_format_info(GaimConnection *gc, GaimRequestFields *fields)
 		field = gaim_request_fields_get_field(fields, vc_tp->tag);
 		text  = gaim_request_field_string_get_value(field);
 
-		gaim_debug(GAIM_DEBUG_INFO, "jabber",
-				   "Setting %s to '%s'\n", vc_tp->tag, text);
 
 		if (text != NULL && *text != '\0') {
 			xmlnode *xp;
 
+			gaim_debug(GAIM_DEBUG_INFO, "jabber",
+					"Setting %s to '%s'\n", vc_tp->tag, text);
+
 			if ((xp = insert_tag_to_parent_tag(vc_node,
 											   NULL, vc_tp->tag)) != NULL) {
 
@@ -542,12 +542,15 @@ static void jabber_vcard_parse(JabberStream *js, xmlnode *packet, gpointer data)
 	if(!from)
 		return;
 
+	jb = jabber_buddy_find(js, from, TRUE);
+	if(!jb)
+		return;
+
 	resource_name = jabber_get_resource(from);
 	bare_jid = jabber_get_bare_jid(from);
 
 	b = gaim_find_buddy(js->gc->account, bare_jid);
 
-	jb = jabber_buddy_find(js, from, TRUE);
 	info_text = g_string_new("");
 
 	g_string_append_printf(info_text, "<b>%s:</b> %s<br/>", _("Jabber ID"),
@@ -572,17 +575,23 @@ static void jabber_vcard_parse(JabberStream *js, xmlnode *packet, gpointer data)
 	} else {
 		for(resources = jb->resources; resources; resources = resources->next) {
 			char *purdy = NULL;
+			char *res = NULL;
+
 			jbr = resources->data;
 			if(jbr->status)
 				purdy = gaim_strdup_withhtml(jbr->status);
-			g_string_append_printf(info_text, "<b>%s:</b> %s<br/>",
-					_("Resource"), jbr->name);
-			g_string_append_printf(info_text, "<b>%s:</b> %s%s%s<br/><br/>",
-					_("Status"), jabber_get_state_string(jbr->state),
+			if(jbr->name)
+				res = g_strdup_printf(" (%s)", jbr->name);
+			g_string_append_printf(info_text, "<b>%s%s:</b> %s%s%s<br/><br/>",
+					_("Status"),
+					res ? res : "",
+					jabber_get_state_string(jbr->state),
 					purdy ? ": " : "",
 					purdy ? purdy : "");
 			if(purdy)
 				g_free(purdy);
+			if(res)
+				g_free(res);
 		}
 	}
 
@@ -630,7 +639,6 @@ static void jabber_vcard_parse(JabberStream *js, xmlnode *packet, gpointer data)
 				serv_got_alias(js->gc, from, text);
 				if(b) {
 					gaim_blist_node_set_string((GaimBlistNode*)b, "servernick", text);
-					gaim_blist_save();
 				}
 				g_string_append_printf(info_text, "<b>%s:</b> %s<br/>",
 						_("Nickname"), text);
@@ -775,7 +783,7 @@ static void jabber_vcard_parse(JabberStream *js, xmlnode *packet, gpointer data)
 
 	text = gaim_strdup_withhtml(info_text->str);
 
-	gaim_notify_formatted(NULL, title, _("Jabber Profile"),
+	gaim_notify_userinfo(js->gc, from, title, _("Jabber Profile"),
 			NULL, text, NULL, NULL);
 
 	g_free(title);
@@ -926,10 +934,6 @@ GList *jabber_buddy_menu(GaimBuddy *buddy)
 	if(!jb)
 		return m;
 
-	act = gaim_blist_node_action_new(_("Send File"),
-			jabber_si_xfer_ask_send, NULL);
-	m = g_list_append(m, act);
-
 	/* XXX: fix the NOT ME below */
 
 	if(js->protocol_version == JABBER_PROTO_0_9 /* && NOT ME */) {
