@@ -20,7 +20,7 @@
  */
 #include "internal.h"
 #include "debug.h"
-#include "multi.h" /* for proto_chat_entry */
+#include "prpl.h" /* for proto_chat_entry */
 #include "notify.h"
 #include "request.h"
 #include "roomlist.h"
@@ -36,7 +36,6 @@ GList *jabber_chat_info(GaimConnection *gc)
 {
 	GList *m = NULL;
 	struct proto_chat_entry *pce;
-	JabberStream *js = gc->proto_data;
 
 	pce = g_new0(struct proto_chat_entry, 1);
 	pce->label = _("_Room:");
@@ -46,13 +45,11 @@ GList *jabber_chat_info(GaimConnection *gc)
 	pce = g_new0(struct proto_chat_entry, 1);
 	pce->label = _("_Server:");
 	pce->identifier = "server";
-	pce->def = js->chat_servers ? js->chat_servers->data : "conference.jabber.org";
 	m = g_list_append(m, pce);
 
 	pce = g_new0(struct proto_chat_entry, 1);
 	pce->label = _("_Handle:");
 	pce->identifier = "handle";
-	pce->def = js->user->node;
 	m = g_list_append(m, pce);
 
 	pce = g_new0(struct proto_chat_entry, 1);
@@ -64,6 +61,33 @@ GList *jabber_chat_info(GaimConnection *gc)
 	return m;
 }
 
+GHashTable *jabber_chat_info_defaults(GaimConnection *gc, const char *chat_name)
+{
+	GHashTable *defaults;
+	JabberStream *js = gc->proto_data;
+
+	defaults = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
+
+	g_hash_table_insert(defaults, "handle", g_strdup(js->user->node));
+
+	if (js->chat_servers)
+		g_hash_table_insert(defaults, "server", g_strdup(js->chat_servers->data));
+	else
+		g_hash_table_insert(defaults, "server", g_strdup("conference.jabber.org"));
+
+	if (chat_name != NULL) {
+		JabberID *jid = jabber_id_new(chat_name);
+		if(jid) {
+			g_hash_table_insert(defaults, "room", g_strdup(jid->node));
+			if(jid->domain)
+				g_hash_table_replace(defaults, "server", g_strdup(jid->domain));
+			jabber_id_free(jid);
+		}
+	}
+
+	return defaults;
+}
+
 JabberChat *jabber_chat_find(JabberStream *js, const char *room,
 		const char *server)
 {
@@ -103,6 +127,16 @@ JabberChat *jabber_chat_find_by_id(JabberStream *js, int id)
 	return chat;
 }
 
+JabberChat *jabber_chat_find_by_conv(GaimConversation *conv)
+{
+	GaimAccount *account = gaim_conversation_get_account(conv);
+	GaimConnection *gc = gaim_account_get_connection(account);
+	JabberStream *js = gc->proto_data;
+	int id = gaim_conv_chat_get_id(GAIM_CONV_CHAT(conv));
+
+	return jabber_chat_find_by_id(js, id);
+}
+
 void jabber_chat_invite(GaimConnection *gc, int id, const char *msg,
 		const char *name)
 {
@@ -141,6 +175,20 @@ void jabber_chat_invite(GaimConnection *gc, int id, const char *msg,
 	g_free(room_jid);
 }
 
+void jabber_chat_member_free(JabberChatMember *jcm);
+
+char *jabber_get_chat_name(GHashTable *data) {
+	char *room, *server, *chat_name = NULL;
+
+	room = g_hash_table_lookup(data, "room");
+	server = g_hash_table_lookup(data, "server");
+
+	if (room && server) {
+		chat_name = g_strdup_printf("%s@%s", room, server);
+	}
+	return chat_name;
+}
+
 void jabber_chat_join(GaimConnection *gc, GHashTable *data)
 {
 	JabberChat *chat;
@@ -192,6 +240,9 @@ void jabber_chat_join(GaimConnection *gc, GHashTable *data)
 	chat->server = g_strdup(server);
 	chat->handle = g_strdup(handle);
 
+	chat->members = g_hash_table_new_full(g_str_hash, g_str_equal, NULL,
+			(GDestroyNotify)jabber_chat_member_free);
+
 	g_hash_table_insert(js->chats, room_jid, chat);
 
 	presence = jabber_presence_create(gc->away_state, gc->away);
@@ -221,6 +272,8 @@ void jabber_chat_leave(GaimConnection *gc, int id)
 		return;
 
 	jabber_chat_part(chat, NULL);
+
+	chat->conv = NULL;
 }
 
 void jabber_chat_destroy(JabberChat *chat)
@@ -239,20 +292,14 @@ void jabber_chat_free(JabberChat *chat)
 
 	g_free(chat->room);
 	g_free(chat->server);
+	g_free(chat->handle);
+	g_hash_table_destroy(chat->members);
 	g_free(chat);
 }
 
 gboolean jabber_chat_find_buddy(GaimConversation *conv, const char *name)
 {
-	GList *m = gaim_conv_chat_get_users(GAIM_CONV_CHAT(conv));
-
-	while(m) {
-		if(!strcmp(m->data, name))
-			return TRUE;
-		m = m->next;
-	}
-
-	return FALSE;
+	return gaim_conv_chat_find_user(GAIM_CONV_CHAT(conv), name);
 }
 
 char *jabber_chat_buddy_real_name(GaimConnection *gc, int id, const char *who)
@@ -359,7 +406,7 @@ void jabber_chat_request_room_configure(JabberChat *chat) {
 		return;
 	}
 
-	iq = jabber_iq_new_query(chat->js, JABBER_IQ_SET,
+	iq = jabber_iq_new_query(chat->js, JABBER_IQ_GET,
 			"http://jabber.org/protocol/muc#owner");
 	query = xmlnode_get_child(iq->node, "query");
 	room_jid = g_strdup_printf("%s@%s", chat->room, chat->server);
@@ -521,11 +568,15 @@ void jabber_chat_change_topic(JabberChat *chat, const char *topic)
 		jabber_message_free(jm);
 	} else {
 		const char *cur = gaim_conv_chat_get_topic(GAIM_CONV_CHAT(chat->conv));
-		char *buf;
+		char *buf, *tmp, *tmp2;
 
-		if(cur)
-			buf = g_strdup_printf(_("current topic is: %s"), cur);
-		else
+		if(cur) {
+			tmp = gaim_escape_html(cur);
+			tmp2 = gaim_markup_linkify(tmp);
+			buf = g_strdup_printf(_("current topic is: %s"), tmp2);
+			g_free(tmp);
+			g_free(tmp2);
+		} else
 			buf = g_strdup(_("No topic is set"));
 		gaim_conv_chat_write(GAIM_CONV_CHAT(chat->conv), "", buf,
 				GAIM_MESSAGE_SYSTEM | GAIM_MESSAGE_NO_LOG, time(NULL));
@@ -641,6 +692,14 @@ static void roomlist_disco_result_cb(JabberStream *js, xmlnode *packet, gpointer
 	js->roomlist = NULL;
 }
 
+static void roomlist_cancel_cb(JabberStream *js, const char *server) {
+	if(js->roomlist) {
+		gaim_roomlist_set_in_progress(js->roomlist, FALSE);
+		gaim_roomlist_unref(js->roomlist);
+		js->roomlist = NULL;
+	}
+}
+
 static void roomlist_ok_cb(JabberStream *js, const char *server)
 {
 	JabberIq *iq;
@@ -673,7 +732,7 @@ GaimRoomlist *jabber_roomlist_get_list(GaimConnection *gc)
 	if(js->roomlist)
 		gaim_roomlist_unref(js->roomlist);
 
-	js->roomlist = gaim_roomlist_new(gaim_connection_get_account(gc));
+	js->roomlist = gaim_roomlist_new(gaim_connection_get_account(js->gc));
 
 	f = gaim_roomlist_field_new(GAIM_ROOMLIST_FIELD_STRING, "", "room", TRUE);
 	fields = g_list_append(fields, f);
@@ -686,11 +745,13 @@ GaimRoomlist *jabber_roomlist_get_list(GaimConnection *gc)
 
 	gaim_roomlist_set_fields(js->roomlist, fields);
 
+
 	gaim_request_input(gc, _("Enter a Conference Server"), _("Enter a Conference Server"),
 			_("Select a conference server to query"),
 			js->chat_servers ? js->chat_servers->data : "conference.jabber.org",
 			FALSE, FALSE, NULL,
-			_("Find Rooms"), G_CALLBACK(roomlist_ok_cb), _("Cancel"), NULL, js);
+			_("Find Rooms"), GAIM_CALLBACK(roomlist_ok_cb),
+			_("Cancel"), GAIM_CALLBACK(roomlist_cancel_cb), js);
 
 	return js->roomlist;
 }
@@ -711,5 +772,138 @@ void jabber_roomlist_cancel(GaimRoomlist *list)
 	}
 }
 
+void jabber_chat_member_free(JabberChatMember *jcm)
+{
+	g_free(jcm->handle);
+	g_free(jcm->jid);
+	g_free(jcm);
+}
+
+void jabber_chat_track_handle(JabberChat *chat, const char *handle,
+		const char *jid, const char *affiliation, const char *role)
+{
+	JabberChatMember *jcm = g_new0(JabberChatMember, 1);
+
+	jcm->handle = g_strdup(handle);
+	jcm->jid = g_strdup(jid);
+
+	g_hash_table_replace(chat->members, jcm->handle, jcm);
+
+	/* XXX: keep track of role and affiliation */
+}
+
+void jabber_chat_remove_handle(JabberChat *chat, const char *handle)
+{
+	g_hash_table_remove(chat->members, handle);
+}
+
+gboolean jabber_chat_ban_user(JabberChat *chat, const char *who, const char *why)
+{
+	JabberIq *iq;
+	JabberChatMember *jcm = g_hash_table_lookup(chat->members, who);
+	char *to;
+	xmlnode *query, *item, *reason;
+
+	if(!jcm || !jcm->jid)
+		return FALSE;
+
+	iq = jabber_iq_new_query(chat->js, JABBER_IQ_SET,
+			"http://jabber.org/protocol/muc#admin");
+
+	to = g_strdup_printf("%s@%s", chat->room, chat->server);
+	xmlnode_set_attrib(iq->node, "to", to);
+	g_free(to);
+
+	query = xmlnode_get_child(iq->node, "query");
+	item = xmlnode_new_child(query, "item");
+	xmlnode_set_attrib(item, "jid", jcm->jid);
+	xmlnode_set_attrib(item, "affiliation", "outcast");
+	if(why) {
+		reason = xmlnode_new_child(item, "reason");
+		xmlnode_insert_data(reason, why, -1);
+	}
+
+	jabber_iq_send(iq);
+
+	return TRUE;
+}
+
+
+gboolean jabber_chat_kick_user(JabberChat *chat, const char *who, const char *why)
+{
+	JabberIq *iq;
+	JabberChatMember *jcm = g_hash_table_lookup(chat->members, who);
+	char *to;
+	xmlnode *query, *item, *reason;
+
+	if(!jcm || !jcm->jid)
+		return FALSE;
+
+	iq = jabber_iq_new_query(chat->js, JABBER_IQ_SET,
+			"http://jabber.org/protocol/muc#admin");
+
+	to = g_strdup_printf("%s@%s", chat->room, chat->server);
+	xmlnode_set_attrib(iq->node, "to", to);
+	g_free(to);
+
+	query = xmlnode_get_child(iq->node, "query");
+	item = xmlnode_new_child(query, "item");
+	xmlnode_set_attrib(item, "jid", jcm->jid);
+	xmlnode_set_attrib(item, "role", "none");
+	if(why) {
+		reason = xmlnode_new_child(item, "reason");
+		xmlnode_insert_data(reason, why, -1);
+	}
+
+	jabber_iq_send(iq);
+
+	return TRUE;
+}
+
+static void jabber_chat_disco_traffic_cb(JabberStream *js, xmlnode *packet, gpointer data)
+{
+	JabberChat *chat;
+	xmlnode *query, *x, *error;
+	int id = GPOINTER_TO_INT(data);
+
+	if(!(chat = jabber_chat_find_by_id(js, id)))
+		return;
+
+	if((error = xmlnode_get_child(packet, "error"))) {
+		/* defaults, in case the conference server doesn't
+		 * support this request */
+		chat->xhtml = TRUE;
+		return;
+	}
+
+	if(!(query = xmlnode_get_child(packet, "query")))
+		return;
+
+	for(x = xmlnode_get_child(query, "feature"); x; x = xmlnode_get_next_twin(x)) {
+		const char *var = xmlnode_get_attrib(x, "var");
+
+		if(var && !strcmp(var, "http://jabber.org/protocol/xhtml-im")) {
+			chat->xhtml = TRUE;
+		}
+	}
+}
+
+void jabber_chat_disco_traffic(JabberChat *chat)
+{
+	JabberIq *iq;
+	xmlnode *query;
+
+	iq = jabber_iq_new_query(chat->js, JABBER_IQ_GET,
+			"http://jabber.org/protocol/disco#info");
+
+	query = xmlnode_get_child(iq->node, "query");
+
+	xmlnode_set_attrib(query, "node", "http://jabber.org/protocol/muc#traffic");
+
+	jabber_iq_set_callback(iq, jabber_chat_disco_traffic_cb, GINT_TO_POINTER(chat->id));
+
+	jabber_iq_send(iq);
+}
+
 
 
