@@ -46,6 +46,8 @@ struct bytestreams_streamhost {
 typedef struct _JabberSIXfer {
 	JabberStream *js;
 
+	gboolean accepted;
+
 	char *stream_id;
 	char *iq_id;
 
@@ -74,7 +76,7 @@ jabber_si_xfer_find(JabberStream *js, const char *sid, const char *from)
 	for(xfers = js->file_transfers; xfers; xfers = xfers->next) {
 		GaimXfer *xfer = xfers->data;
 		JabberSIXfer *jsx = xfer->data;
-		if(jsx->stream_id && xfer->who &&
+		if(jsx && jsx->stream_id && xfer->who &&
 				!strcmp(jsx->stream_id, sid) && !strcmp(xfer->who, from))
 			return xfer;
 	}
@@ -186,6 +188,10 @@ void jabber_bytestreams_parse(JabberStream *js, xmlnode *packet)
 		return;
 
 	jsx = xfer->data;
+
+	if(!jsx->accepted)
+		return;
+
 	if(jsx->iq_id)
 		g_free(jsx->iq_id);
 	jsx->iq_id = g_strdup(xmlnode_get_attrib(packet, "id"));
@@ -509,7 +515,7 @@ static void jabber_si_xfer_send_request(GaimXfer *xfer)
 	xmlnode_set_attrib(file, "xmlns",
 			"http://jabber.org/protocol/si/profile/file-transfer");
 	xmlnode_set_attrib(file, "name", xfer->filename);
-	g_snprintf(buf, sizeof(buf), "%d", xfer->size);
+	g_snprintf(buf, sizeof(buf), "%zu", xfer->size);
 	xmlnode_set_attrib(file, "size", buf);
 	/* maybe later we'll do hash and date attribs */
 
@@ -564,6 +570,12 @@ static void jabber_si_xfer_cancel_recv(GaimXfer *xfer)
 	gaim_debug(GAIM_DEBUG_INFO, "jabber", "in jabber_si_xfer_cancel_recv\n");
 }
 
+static void jabber_si_xfer_request_denied(GaimXfer *xfer)
+{
+	/* XXX: let the other side know */
+	jabber_si_xfer_free(xfer);
+}
+
 
 static void jabber_si_xfer_end(GaimXfer *xfer)
 {
@@ -619,6 +631,8 @@ static void jabber_si_xfer_init(GaimXfer *xfer)
 		if(jsx->iq_id)
 			jabber_iq_set_id(iq, jsx->iq_id);
 
+		jsx->accepted = TRUE;
+
 		si = xmlnode_new_child(iq->node, "si");
 		xmlnode_set_attrib(si, "xmlns", "http://jabber.org/protocol/si");
 
@@ -644,36 +658,34 @@ static void jabber_si_xfer_init(GaimXfer *xfer)
 	}
 }
 
-void jabber_si_xfer_ask_send(GaimBlistNode *node, gpointer data)
+void jabber_si_xfer_send(GaimConnection *gc, const char *who, const char *file)
 {
-	GaimBuddy *buddy;
-	GaimConnection *gc;
 	JabberStream *js;
 
 	GaimXfer *xfer;
 	JabberSIXfer *jsx;
 
-	g_return_if_fail(GAIM_BLIST_NODE_IS_BUDDY(node));
-
-	buddy = (GaimBuddy *) node;
-	gc = gaim_account_get_connection(buddy->account);
 	js = gc->proto_data;
 
-	if(!gaim_find_buddy(gc->account, buddy->name) || !jabber_buddy_find(js, buddy->name, FALSE))
+	if(!gaim_find_buddy(gc->account, who) || !jabber_buddy_find(js, who, FALSE))
 		return;
 
-	xfer = gaim_xfer_new(buddy->account, GAIM_XFER_SEND, buddy->name);
+	xfer = gaim_xfer_new(gc->account, GAIM_XFER_SEND, who);
 
 	xfer->data = jsx = g_new0(JabberSIXfer, 1);
 	jsx->js = js;
 
 	gaim_xfer_set_init_fnc(xfer, jabber_si_xfer_init);
 	gaim_xfer_set_cancel_send_fnc(xfer, jabber_si_xfer_cancel_send);
+	gaim_xfer_set_request_denied_fnc(xfer, jabber_si_xfer_request_denied);
 	gaim_xfer_set_end_fnc(xfer, jabber_si_xfer_end);
 
 	js->file_transfers = g_list_append(js->file_transfers, xfer);
 
-	gaim_xfer_request(xfer);
+	if (file)
+		gaim_xfer_request_accepted(xfer, file);
+	else
+		gaim_xfer_request(xfer);
 }
 
 void jabber_si_parse(JabberStream *js, xmlnode *packet)
@@ -681,7 +693,7 @@ void jabber_si_parse(JabberStream *js, xmlnode *packet)
 	JabberSIXfer *jsx;
 	GaimXfer *xfer;
 	xmlnode *si, *file, *feature, *x, *field, *option, *value;
-	const char *stream_id, *filename, *filesize_c, *profile;
+	const char *stream_id, *filename, *filesize_c, *profile, *from;
 	size_t filesize = 0;
 
 	if(!(si = xmlnode_get_child(packet, "si")))
@@ -709,6 +721,15 @@ void jabber_si_parse(JabberStream *js, xmlnode *packet)
 	if(!(x = xmlnode_get_child_with_namespace(feature, "x", "jabber:x:data")))
 		return;
 
+	if(!(from = xmlnode_get_attrib(packet, "from")))
+		return;
+
+	/* if they've already sent us this file transfer with the same damn id
+	 * then we're gonna ignore it, until I think of something better to do
+	 * with it */
+	if((xfer = jabber_si_xfer_find(js, stream_id, from)))
+		return;
+
 	jsx = g_new0(JabberSIXfer, 1);
 
 	for(field = xmlnode_get_child(x, "field"); field; field = xmlnode_get_next_twin(field)) {
@@ -742,8 +763,7 @@ void jabber_si_parse(JabberStream *js, xmlnode *packet)
 	jsx->stream_id = g_strdup(stream_id);
 	jsx->iq_id = g_strdup(xmlnode_get_attrib(packet, "id"));
 
-	xfer = gaim_xfer_new(js->gc->account, GAIM_XFER_RECEIVE,
-			xmlnode_get_attrib(packet, "from"));
+	xfer = gaim_xfer_new(js->gc->account, GAIM_XFER_RECEIVE, from);
 	xfer->data = jsx;
 
 	gaim_xfer_set_filename(xfer, filename);
@@ -752,6 +772,7 @@ void jabber_si_parse(JabberStream *js, xmlnode *packet)
 
 	gaim_xfer_set_init_fnc(xfer, jabber_si_xfer_init);
 	gaim_xfer_set_cancel_recv_fnc(xfer, jabber_si_xfer_cancel_recv);
+	gaim_xfer_set_request_denied_fnc(xfer, jabber_si_xfer_request_denied);
 	gaim_xfer_set_end_fnc(xfer, jabber_si_xfer_end);
 
 	js->file_transfers = g_list_append(js->file_transfers, xfer);
