@@ -28,6 +28,6 @@
 
 void jabber_bytestreams_parse(JabberStream *js, xmlnode *packet);
 void jabber_si_parse(JabberStream *js, xmlnode *packet);
-void jabber_si_xfer_ask_send(GaimBlistNode *node, gpointer data);
+void jabber_si_xfer_send(GaimConnection *gc, const char *who, const char *file);
 
 #endif /* _GAIM_JABBER_SI_H_ */
