@@ -3,8 +3,10 @@
  *
  * gaim
  *
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
- * 
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -27,6 +29,61 @@ typedef struct _MsnMessage MsnMessage;
 #include "session.h"
 #include "user.h"
 
+#include "command.h"
+#include "transaction.h"
+
+typedef void (*MsnMsgCb)(MsnMessage *, void *data);
+
+/*
+typedef enum
+{
+	MSN_MSG_NORMAL,
+	MSN_MSG_SLP_SB,
+	MSN_MSG_SLP_DC
+
+} MsnMsgType;
+*/
+
+typedef enum
+{
+	MSN_MSG_UNKNOWN,
+	MSN_MSG_TEXT,
+	MSN_MSG_TYPING,
+	MSN_MSG_CAPS,
+	MSN_MSG_SLP
+
+} MsnMsgType;
+
+typedef enum
+{
+	MSN_MSG_ERROR_NONE, /**< No error. */
+	MSN_MSG_ERROR_TIMEOUT, /**< The message timedout. */
+	MSN_MSG_ERROR_NAK, /**< The message could not be sent. */
+	MSN_MSG_ERROR_SB, /**< The error comes from the switchboard. */
+	MSN_MSG_ERROR_UNKNOWN /**< An unknown error occurred. */
+
+} MsnMsgErrorType;
+
+typedef struct
+{
+	guint32 session_id;
+	guint32 id;
+	guint64 offset;
+	guint64 total_size;
+	guint32 length;
+	guint32 flags;
+	guint32 ack_id;
+	guint32 ack_sub_id;
+	guint64 ack_size;
+
+} MsnSlpHeader;
+
+typedef struct
+{
+	guint32 value;
+
+} MsnSlpFooter;
+
 /**
  * A message.
  */
@@ -34,22 +91,37 @@ struct _MsnMessage
 {
 	size_t ref_count;           /**< The reference count.       */
 
-	MsnUser *sender;
-	MsnUser *receiver;
+	MsnMsgType type;
 
-	unsigned int tid;
+	gboolean msnslp_message;
+
+	char *remote_user;
 	char flag;
 
-	gboolean incoming;
-
-	size_t size;
-
 	char *content_type;
 	char *charset;
 	char *body;
+	gsize body_len;
+
+	MsnSlpHeader msnslp_header;
+	MsnSlpFooter msnslp_footer;
 
 	GHashTable *attr_table;
 	GList *attr_list;
+
+	gboolean ack_ref;           /**< A flag that states if this message has
+								  been ref'ed for using it in a callback. */
+
+	MsnCommand *cmd;
+	MsnTransaction *trans;
+
+	MsnMsgCb ack_cb; /**< The callback to call when we receive an ACK of this
+					   message. */
+	MsnMsgCb nak_cb; /**< The callback to call when we receive a NAK of this
+					   message. */
+	void *ack_data; /**< The data used by callbacks. */
+
+	MsnMsgErrorType error; /**< The error of the message. */
 };
 
 /**
@@ -57,20 +129,50 @@ struct _MsnMessage
  *
  * @return A new message.
  */
-MsnMessage *msn_message_new(void);
+MsnMessage *msn_message_new(MsnMsgType type);
 
 /**
- * Creates a new message based off a string.
+ * Creates a new, empty MSNSLP message.
+ *
+ * @return A new MSNSLP message.
+ */
+MsnMessage *msn_message_new_msnslp(void);
+
+MsnMessage *msn_message_new_plain(const char *message);
+
+/**
+ * Creates a MSNSLP ack message.
+ *
+ * @param acked_msg The message to acknowledge.
+ *
+ * @return A new MSNSLP ack message.
+ */
+MsnMessage *msn_message_new_msnslp_ack(MsnMessage *acked_msg);
+
+/**
+ * Creates a new message based off a command.
  *
  * @param session The MSN session.
- * @param str     The string.
+ * @param cmd     The command.
  *
  * @return The new message.
  */
-MsnMessage *msn_message_new_from_str(MsnSession *session, const char *str);
+MsnMessage *msn_message_new_from_cmd(MsnSession *session, MsnCommand *cmd);
+
+/**
+ * Parses the payload of a message.
+ *
+ * @param msg         The message.
+ * @param payload     The payload.
+ * @param payload_len The length of the payload.
+ */
+void msn_message_parse_payload(MsnMessage *msg, const char *payload,
+							   size_t payload_len);
 
 /**
  * Destroys a message.
+ *
+ * @param msg The message to destroy.
  */
 void msn_message_destroy(MsnMessage *msg);
 
@@ -95,82 +197,14 @@ MsnMessage *msn_message_ref(MsnMessage *msg);
 MsnMessage *msn_message_unref(MsnMessage *msg);
 
 /**
- * Converts a message to a string.
+ * Generates the payload data of a message.
  *
- * @param msg The message.
+ * @param msg      The message.
+ * @param ret_size The returned size of the payload.
  *
- * @return The string representation of a message.
+ * @return The payload data of the message.
  */
-char *msn_message_build_string(const MsnMessage *msg);
-
-/**
- * Returns TRUE if the message is outgoing.
- *
- * @param msg The message.
- *
- * @return @c TRUE if the message is outgoing, or @c FALSE otherwise.
- */
-gboolean msn_message_is_outgoing(const MsnMessage *msg);
-
-/**
- * Returns TRUE if the message is incoming.
- *
- * @param msg The message.
- *
- * @return @c TRUE if the message is incoming, or @c FALSE otherwise.
- */
-gboolean msn_message_is_incoming(const MsnMessage *msg);
-
-/**
- * Sets the message's sender.
- *
- * @param msg  The message.
- * @param user The sender.
- */
-void msn_message_set_sender(MsnMessage *msg, MsnUser *user);
-
-/**
- * Returns the message's sender.
- *
- * @param msg The message.
- *
- * @return The sender.
- */
-MsnUser *msn_message_get_sender(const MsnMessage *msg);
-
-/**
- * Sets the message's receiver.
- *
- * @param msg  The message.
- * @param user The receiver.
- */
-void msn_message_set_receiver(MsnMessage *msg, MsnUser *user);
-
-/**
- * Returns the message's receiver.
- *
- * @param msg The message.
- *
- * @return The receiver.
- */
-MsnUser *msn_message_get_receiver(const MsnMessage *msg);
-
-/**
- * Sets the message transaction ID.
- *
- * @param msg The message.
- * @param tid The transaction ID.
- */
-void msn_message_set_transaction_id(MsnMessage *msg, unsigned int tid);
-
-/**
- * Returns the message transaction ID.
- *
- * @param msg The message.
- *
- * @return The transaction ID.
- */
-unsigned int msn_message_get_transaction_id(const MsnMessage *msg);
+char *msn_message_gen_payload(MsnMessage *msg, size_t *ret_size);
 
 /**
  * Sets the flag for an outgoing message.
@@ -189,6 +223,7 @@ void msn_message_set_flag(MsnMessage *msg, char flag);
  */
 char msn_message_get_flag(const MsnMessage *msg);
 
+#if 0
 /**
  * Sets the body of a message.
  *
@@ -205,6 +240,25 @@ void msn_message_set_body(MsnMessage *msg, const char *body);
  * @return The body of the message.
  */
 const char *msn_message_get_body(const MsnMessage *msg);
+#endif
+/**
+ * Sets the binary content of the message.
+ *
+ * @param msg  The message.
+ * @param data The binary data.
+ * @param len  The length of the data.
+ */
+void msn_message_set_bin_data(MsnMessage *msg, const void *data, size_t len);
+
+/**
+ * Returns the binary content of the message.
+ *
+ * @param msg The message.
+ * @param len The returned length of the data.
+ *
+ * @return The binary data.
+ */
+const void *msn_message_get_bin_data(const MsnMessage *msg, size_t *len);
 
 /**
  * Sets the content type in a message.
@@ -269,4 +323,14 @@ const char *msn_message_get_attr(const MsnMessage *msg, const char *attr);
  */
 GHashTable *msn_message_get_hashtable_from_body(const MsnMessage *msg);
 
+void msn_message_show_readable(MsnMessage *msg, const char *info,
+							   gboolean text_body);
+
+void msn_message_parse_slp_body(MsnMessage *msg, const char *body,
+								size_t len);
+
+char *msn_message_gen_slp_body(MsnMessage *msg, size_t *ret_size);
+
+char *msn_message_to_string(MsnMessage *msg);
+
 #endif /* _MSN_MSG_H_ */
