@@ -338,6 +338,7 @@ initiate_chat_cb(GaimBlistNode *node, gpointer data)
 	/* TODO: This might move somewhere else, after USR might be */
 	swboard->chat_id = session->conv_seq++;
 	swboard->conv = serv_got_joined_chat(gc, swboard->chat_id, "MSN Chat");
+	swboard->flag = MSN_SB_FLAG_IM;
 
 	gaim_conv_chat_add_user(GAIM_CONV_CHAT(swboard->conv),
 							gaim_account_get_username(buddy->account), NULL, GAIM_CBFLAGS_NONE, TRUE);
@@ -354,7 +355,7 @@ t_msn_xfer_init(GaimXfer *xfer)
 
 	slplink = xfer->data;
 
-	if ((fp = fopen(filename, "rb")) == NULL)
+	if ((fp = g_fopen(filename, "rb")) == NULL)
 	{
 		GaimAccount *account;
 		GaimConnection *gc;
@@ -617,6 +618,7 @@ msn_login(GaimAccount *account)
 
 	if (!gaim_ssl_is_supported())
 	{
+		gc->wants_to_die = TRUE;
 		gaim_connection_error(gc,
 			_("SSL support is needed for MSN. Please install a supported "
 			  "SSL library. See http://gaim.sf.net/faq-ssl.php for more "
@@ -645,7 +647,8 @@ msn_login(GaimAccount *account)
 	if (strcmp(username, gaim_account_get_username(account)))
 		gaim_account_set_username(account, username);
 
-	msn_session_connect(session, host, port, http_method);
+	if (!msn_session_connect(session, host, port, http_method))
+		gaim_connection_error(gc, _("Failed to connect to server."));
 }
 
 static void
@@ -695,7 +698,7 @@ msn_send_im(GaimConnection *gc, const char *who, const char *message,
 		MsnSwitchBoard *swboard;
 
 		session = gc->proto_data;
-		swboard = msn_session_get_swboard(session, who);
+		swboard = msn_session_get_swboard(session, who, MSN_SB_FLAG_IM);
 
 		msn_switchboard_send_msg(swboard, msg, TRUE);
 	}
@@ -713,7 +716,8 @@ msn_send_im(GaimConnection *gc, const char *who, const char *message,
 
 		format = msn_message_get_attr(msg, "X-MMS-IM-Format");
 		msn_parse_format(format, &pre, &post);
-		body_str = g_strdup_printf("%s%s%s", pre, body_enc, post);
+		body_str = g_strdup_printf("%s%s%s", pre ? pre :  "",
+								   body_enc ? body_enc : "", post ? post : "");
 		g_free(body_enc);
 		g_free(pre);
 		g_free(post);
@@ -755,6 +759,8 @@ msn_send_typing(GaimConnection *gc, const char *who, int typing)
 	if (swboard == NULL || !msn_switchboard_can_send(swboard))
 		return 0;
 
+	swboard->flag |= MSN_SB_FLAG_IM;
+
 	msg = msn_message_new(MSN_MSG_TYPING);
 	msn_message_set_content_type(msg, "text/x-msmsgscontrol");
 	msn_message_set_flag(msg, 'U');
@@ -1053,7 +1059,7 @@ msn_chat_invite(GaimConnection *gc, int id, const char *msg,
 
 	session = gc->proto_data;
 
-	swboard = msn_session_find_switch_with_id(session, id);
+	swboard = msn_session_find_swboard_with_id(session, id);
 
 	if (swboard == NULL)
 	{
@@ -1064,6 +1070,8 @@ msn_chat_invite(GaimConnection *gc, int id, const char *msg,
 		swboard->conv = gaim_find_chat(gc, id);
 	}
 
+	swboard->flag |= MSN_SB_FLAG_IM;
+
 	msn_switchboard_request_add_user(swboard, who);
 }
 
@@ -1072,16 +1080,27 @@ msn_chat_leave(GaimConnection *gc, int id)
 {
 	MsnSession *session;
 	MsnSwitchBoard *swboard;
+	GaimConversation *conv;
 
 	session = gc->proto_data;
 
-	swboard = msn_session_find_switch_with_id(session, id);
+	swboard = msn_session_find_swboard_with_id(session, id);
 
 	/* if swboard is NULL we were the only person left anyway */
 	if (swboard == NULL)
 		return;
 
-	msn_switchboard_close(swboard);
+	conv = swboard->conv;
+
+	msn_switchboard_release(swboard, MSN_SB_FLAG_IM);
+
+	/* If other switchboards managed to associate themselves with this
+	 * conv, make sure they know it's gone! */
+	if (conv != NULL)
+	{
+		while ((swboard = msn_session_find_swboard_with_conv(session, conv)) != NULL)
+			swboard->conv = NULL;
+	}
 }
 
 static int
@@ -1096,7 +1115,7 @@ msn_chat_send(GaimConnection *gc, int id, const char *message)
 
 	account = gaim_connection_get_account(gc);
 	session = gc->proto_data;
-	swboard = msn_session_find_switch_with_id(session, id);
+	swboard = msn_session_find_swboard_with_id(session, id);
 
 	if (swboard == NULL)
 		return -EINVAL;
@@ -1104,6 +1123,8 @@ msn_chat_send(GaimConnection *gc, int id, const char *message)
 	if (!swboard->ready)
 		return 0;
 
+	swboard->flag |= MSN_SB_FLAG_IM;
+
 	msn_import_html(message, &msgformat, &msgtext);
 
 	if (strlen(msgtext) + strlen(msgformat) + strlen(VERSION) > 1564)
@@ -1132,14 +1153,17 @@ static void
 msn_keepalive(GaimConnection *gc)
 {
 	MsnSession *session;
-	MsnCmdProc *cmdproc;
 
 	session = gc->proto_data;
 
-	cmdproc = session->notification->cmdproc;
-
 	if (!session->http_method)
+	{
+		MsnCmdProc *cmdproc;
+
+		cmdproc = session->notification->cmdproc;
+
 		msn_cmdproc_send_quick(cmdproc, "PNG", NULL, NULL);
+	}
 }
 
 static void
@@ -1186,20 +1210,31 @@ msn_convo_closed(GaimConnection *gc, const char *who)
 {
 	MsnSession *session;
 	MsnSwitchBoard *swboard;
+	GaimConversation *conv;
 
 	session = gc->proto_data;
 
 	swboard = msn_session_find_swboard(session, who);
 
 	/*
-	 * Don't perform an assertion here. It swboard is NULL, then the
+	 * Don't perform an assertion here. If swboard is NULL, then the
 	 * switchboard was either closed by the other party, or the person
 	 * is talking to himself.
 	 */
 	if (swboard == NULL)
 		return;
 
-	msn_switchboard_close(swboard);
+	conv = swboard->conv;
+
+	msn_switchboard_release(swboard, MSN_SB_FLAG_IM);
+
+	/* If other switchboards managed to associate themselves with this
+	 * conv, make sure they know it's gone! */
+	if (conv != NULL)
+	{
+		while ((swboard = msn_session_find_swboard_with_conv(session, conv)) != NULL)
+			swboard->conv = NULL;
+	}
 }
 
 static void
