@@ -268,6 +268,30 @@ usr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	}
 }
 
+static void
+usr_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
+{
+	MsnErrorType msnerr = 0;
+
+	switch (error)
+	{
+		case 500:
+		case 601:
+		case 910:
+		case 921:
+			msnerr = MSN_ERROR_SERV_UNAVAILABLE;
+			break;
+		case 911:
+			msnerr = MSN_ERROR_AUTH;
+			break;
+		default:
+			return;
+			break;
+	}
+
+	msn_session_set_error(cmdproc->session, msnerr, NULL);
+}
+
 static void
 ver_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -322,7 +346,11 @@ msn_notification_close(MsnNotification *notification)
 {
 	g_return_if_fail(notification != NULL);
 
-	msn_cmdproc_send_quick(notification->cmdproc, "OUT", NULL, NULL);
+	if (!notification->in_use)
+		return;
+
+	if (notification->session->connected)
+		msn_cmdproc_send_quick(notification->cmdproc, "OUT", NULL, NULL);
 
 	msn_notification_disconnect(notification);
 }
@@ -719,6 +747,37 @@ rea_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	gaim_connection_set_display_name(gc, friendly);
 }
 
+static void
+prp_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
+{
+	MsnSession *session = cmdproc->session;
+	const char *type, *value;
+
+	g_return_if_fail(cmd->param_count >= 3);
+
+	type  = cmd->params[2];
+
+	if (cmd->param_count == 4)
+	{
+		value = cmd->params[3];
+		if (!strcmp(type, "PHH"))
+			msn_user_set_home_phone(session->user, gaim_url_decode(value));
+		else if (!strcmp(type, "PHW"))
+			msn_user_set_work_phone(session->user, gaim_url_decode(value));
+		else if (!strcmp(type, "PHM"))
+			msn_user_set_mobile_phone(session->user, gaim_url_decode(value));
+	}
+	else
+	{
+		if (!strcmp(type, "PHH"))
+			msn_user_set_home_phone(session->user, NULL);
+		else if (!strcmp(type, "PHW"))
+			msn_user_set_work_phone(session->user, NULL);
+		else if (!strcmp(type, "PHM"))
+			msn_user_set_mobile_phone(session->user, NULL);
+	}
+}
+
 static void
 reg_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -887,7 +946,7 @@ url_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
 	if (session->passport_info.file != NULL)
 	{
-		unlink(session->passport_info.file);
+		g_unlink(session->passport_info.file);
 		g_free(session->passport_info.file);
 	}
 
@@ -940,10 +999,34 @@ url_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 							 "Error closing temp passport file: %s\n",
 							 strerror(errno));
 
-			unlink(session->passport_info.file);
+			g_unlink(session->passport_info.file);
 			g_free(session->passport_info.file);
 			session->passport_info.file = NULL;
 		}
+#ifdef _WIN32
+		else
+		{
+			/*
+			 * Renaming file with .html extension, so that the
+			 * win32 open_url will work.
+			 */
+			char *tmp;
+
+			if ((tmp =
+				g_strdup_printf("%s.html",
+					session->passport_info.file)) != NULL)
+			{
+				if (g_rename(session->passport_info.file,
+							tmp) == 0)
+				{
+					g_free(session->passport_info.file);
+					session->passport_info.file = tmp;
+				}
+				else
+					g_free(tmp);
+			}
+		}
+#endif
 	}
 }
 /**************************************************************************
@@ -975,7 +1058,8 @@ rng_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	swboard->im_user = g_strdup(cmd->params[4]);
 	/* msn_switchboard_add_user(swboard, cmd->params[4]); */
 
-	msn_switchboard_connect(swboard, host, port);
+	if (!msn_switchboard_connect(swboard, host, port))
+		msn_switchboard_destroy(swboard);
 
 	g_free(host);
 }
@@ -1286,7 +1370,7 @@ msn_notification_init(void)
 	msn_table_add_cmd(cbs_table, "CVR", "CVR", cvr_cmd);
 	msn_table_add_cmd(cbs_table, "VER", "VER", ver_cmd);
 	msn_table_add_cmd(cbs_table, "REA", "REA", rea_cmd);
-	/* msn_table_add_cmd(cbs_table, "PRP", "PRP", prp_cmd); */
+	msn_table_add_cmd(cbs_table, "PRP", "PRP", prp_cmd);
 	/* msn_table_add_cmd(cbs_table, "BLP", "BLP", blp_cmd); */
 	msn_table_add_cmd(cbs_table, "BLP", "BLP", NULL);
 	msn_table_add_cmd(cbs_table, "REG", "REG", reg_cmd);
@@ -1319,15 +1403,7 @@ msn_notification_init(void)
 	msn_table_add_error(cbs_table, "REG", reg_error);
 	msn_table_add_error(cbs_table, "RMG", rmg_error);
 	/* msn_table_add_error(cbs_table, "REA", rea_error); */
-
-	/* I received a '500' error from the notification server just now.
-	 * I think this means 'Service temporarily unavailable' or similar,
-	 * in response to a USR command. We should report this instead of
-	 * 'Error reading from notification server'.
-	 * I'm not going to implement this right now, because we're string
-	 * frozen (and I'd probably break something anyway), so I'll put this
-	 * here as a reminder, or something. Stu. */
-	/* msn_table_add_error(cbs_table, "USR", usr_error); */
+	msn_table_add_error(cbs_table, "USR", usr_error);
 
 	msn_table_add_msg_type(cbs_table,
 						   "text/x-msmsgsprofile",
