@@ -45,6 +45,7 @@ msn_session_new(GaimAccount *account)
 								 gaim_account_get_username(account), NULL);
 
 	session->protocol_ver = 9;
+	session->conv_seq = 1;
 
 	return session;
 }
@@ -87,7 +88,7 @@ msn_session_destroy(MsnSession *session)
 
 	if (session->passport_info.file != NULL)
 	{
-		unlink(session->passport_info.file);
+		g_unlink(session->passport_info.file);
 		g_free(session->passport_info.file);
 	}
 
@@ -157,16 +158,36 @@ msn_session_find_swboard(MsnSession *session, const char *username)
 
 		swboard = l->data;
 
-		if (swboard->im_user != NULL)
-			if (!strcmp(username, swboard->im_user))
-				return swboard;
+		if ((swboard->im_user != NULL) && !strcmp(username, swboard->im_user))
+			return swboard;
 	}
 
 	return NULL;
 }
 
 MsnSwitchBoard *
-msn_session_find_switch_with_id(const MsnSession *session, int chat_id)
+msn_session_find_swboard_with_conv(MsnSession *session, GaimConversation *conv)
+{
+	GList *l;
+
+	g_return_val_if_fail(session  != NULL, NULL);
+	g_return_val_if_fail(conv != NULL, NULL);
+
+	for (l = session->switches; l != NULL; l = l->next)
+	{
+		MsnSwitchBoard *swboard;
+
+		swboard = l->data;
+
+		if (swboard->conv == conv)
+			return swboard;
+	}
+
+	return NULL;
+}
+
+MsnSwitchBoard *
+msn_session_find_swboard_with_id(const MsnSession *session, int chat_id)
 {
 	GList *l;
 
@@ -187,7 +208,8 @@ msn_session_find_switch_with_id(const MsnSession *session, int chat_id)
 }
 
 MsnSwitchBoard *
-msn_session_get_swboard(MsnSession *session, const char *username)
+msn_session_get_swboard(MsnSession *session, const char *username,
+						MsnSBFlag flag)
 {
 	MsnSwitchBoard *swboard;
 
@@ -201,6 +223,8 @@ msn_session_get_swboard(MsnSession *session, const char *username)
 		msn_switchboard_request_add_user(swboard, username);
 	}
 
+	swboard->flag |= flag;
+
 	return swboard;
 }
 
@@ -284,7 +308,7 @@ msn_session_set_error(MsnSession *session, MsnErrorType error,
 	GaimConnection *gc;
 	char *msg;
 
-	gc = session->account->gc;
+	gc = gaim_account_get_connection(session->account);
 
 	switch (error)
 	{
@@ -302,6 +326,11 @@ msn_session_set_error(MsnSession *session, MsnErrorType error,
 			gc->wants_to_die = TRUE;
 			msg = g_strdup(_("You have signed on from another location."));
 			break;
+		case MSN_ERROR_SERV_UNAVAILABLE:
+			msg = g_strdup(_("The MSN servers are temporarily "
+							 "unavailable. Please wait and try "
+							 "again."));
+			break;
 		case MSN_ERROR_SERV_DOWN:
 			msg = g_strdup(_("The MSN servers are going down "
 							 "temporarily."));
@@ -353,7 +382,7 @@ msn_session_set_login_step(MsnSession *session, MsnLoginStep step)
 
 	/* Prevent the connection progress going backwards, eg. if we get
 	 * transferred several times during login */
-	if (session->login_step >= step)
+	if (session->login_step > step)
 		return;
 
 	/* If we're already logged in, we're probably here because of a
@@ -385,10 +414,11 @@ msn_session_finish_login(MsnSession *session)
 	msn_user_set_buddy_icon(session->user,
 							gaim_account_get_buddy_icon(session->account));
 
-	msn_change_status(session, MSN_ONLINE);
+	session->logged_in = TRUE;
+
+	msn_change_status(session, session->state == 0 ? MSN_ONLINE : session->state);
 
 	gaim_connection_set_state(gc, GAIM_CONNECTED);
-	session->logged_in = TRUE;
 
 	/* Sync users */
 	msn_session_sync_users(session);
