@@ -31,9 +31,6 @@
 #include "user.h"
 #include "switchboard.h"
 
-/* #include "slplink.h" */
-/* #include "directconn.h" */
-
 static void send_ok(MsnSlpCall *slpcall, const char *branch,
 					const char *type, const char *content);
 
@@ -124,6 +121,8 @@ msn_xfer_cancel(GaimXfer *xfer)
 
 			g_free(content);
 			msn_slplink_unleash(slpcall->slplink);
+
+			msn_slp_call_destroy(slpcall);
 		}
 	}
 }
@@ -612,10 +611,16 @@ got_ok(MsnSlpCall *slpcall,
 }
 
 MsnSlpCall *
-msn_slp_sip_recv(MsnSlpLink *slplink, const char *body, gsize len)
+msn_slp_sip_recv(MsnSlpLink *slplink, const char *body)
 {
 	MsnSlpCall *slpcall;
 
+	if (body == NULL)
+	{
+		gaim_debug_warning("msn", "received bogus message\n");
+		return NULL;
+	}
+
 	if (!strncmp(body, "INVITE", strlen("INVITE")))
 	{
 		char *branch;
@@ -732,6 +737,21 @@ msn_p2p_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 	session = cmdproc->servconn->session;
 	slplink = msn_session_get_slplink(session, msg->remote_user);
 
+	if (slplink->swboard == NULL)
+	{
+		/* We will need this in order to change its flags. */
+		slplink->swboard = (MsnSwitchBoard *)cmdproc->data;
+		/* If swboard is NULL, something has probably gone wrong earlier on
+		 * I didn't want to do this, but MSN 7 is somehow causing us to crash
+		 * here, I couldn't reproduce it to debug more, and people are
+		 * reporting bugs. Hopefully this doesn't cause more crashes. Stu.
+		 */
+		if (slplink->swboard != NULL)
+			slplink->swboard->slplink = slplink;
+		else
+			gaim_debug_error("msn", "msn_p2p_msg, swboard is NULL, ouch!\n");
+	}
+
 	msn_slplink_process_msg(slplink, msg);
 }
 
@@ -739,7 +759,9 @@ void
 got_emoticon(MsnSlpCall *slpcall,
 			 const char *data, long long size)
 {
+#ifdef MSN_DEBUG_UD
 	gaim_debug_info("msn", "Got smiley: %s\n", slpcall->data_info);
+#endif
 
 #if 0
 	serv_got_smiley(slpcall->slplink->session->account->gc, 
@@ -760,12 +782,18 @@ msn_emoticon_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 	MsnSlpLink *slplink;
 	MsnObject *obj;
 	char **tokens;
-	char *smile;
-	const char *who;
+	char *body_str, *smile;
+	const char *body, *who;
+	size_t body_len;
 
 	session = cmdproc->servconn->session;
 
-	tokens = g_strsplit(msg->body, "\t", 2);
+	body = msn_message_get_bin_data(msg, &body_len);
+	body_str = g_strndup(body, body_len);
+
+	tokens = g_strsplit(body_str, "\t", 2);
+
+	g_free(body_str);
 
 	smile = tokens[0];
 	obj = msn_object_new_from_string(gaim_url_decode(tokens[1]));
@@ -818,7 +846,9 @@ msn_release_buddy_icon_request(MsnUserList *userlist)
 
 	g_return_if_fail(userlist != NULL);
 
+#ifdef MSN_DEBUG_UD
 	gaim_debug_info("msn", "Releasing buddy icon request\n");
+#endif
 
 	while (userlist->buddy_icon_window > 0)
 	{
@@ -839,8 +869,10 @@ msn_release_buddy_icon_request(MsnUserList *userlist)
 		msn_request_user_display(user);
 		userlist->buddy_icon_window--;
 
+#ifdef MSN_DEBUG_UD
 		gaim_debug_info("msn", "buddy_icon_window=%d\n",
 						userlist->buddy_icon_window);
+#endif
 	}
 }
 
@@ -883,13 +915,17 @@ msn_queue_buddy_icon_request(MsnUser *user)
 		userlist = user->userlist;
 		queue = userlist->buddy_icon_requests;
 
+#ifdef MSN_DEBUG_UD
 		gaim_debug_info("msn", "Queueing buddy icon request: %s\n",
 						user->passport);
+#endif
 
 		g_queue_push_tail(queue, user);
 
+#ifdef MSN_DEBUG_UD
 		gaim_debug_info("msn", "buddy_icon_window=%d\n",
 						userlist->buddy_icon_window);
+#endif
 
 		if (userlist->buddy_icon_window > 0)
 			msn_release_buddy_icon_request(userlist);
@@ -908,7 +944,9 @@ got_user_display(MsnSlpCall *slpcall,
 	g_return_if_fail(slpcall != NULL);
 
 	info = slpcall->data_info;
+#ifdef MSN_DEBUG_UD
 	gaim_debug_info("msn", "Got User Display: %s\n", info);
+#endif
 
 	userlist = slpcall->slplink->session->userlist;
 	account = slpcall->slplink->session->account;
@@ -943,7 +981,13 @@ end_user_display(MsnSlpCall *slpcall)
 
 	g_return_if_fail(slpcall != NULL);
 
+#ifdef MSN_DEBUG_UD
 	gaim_debug_info("msn", "End User Display\n");
+#endif
+
+	/* Maybe the slplink was destroyed. */
+	if (slpcall->slplink == NULL)
+		return;
 
 	userlist = slpcall->slplink->session->userlist;
 
@@ -954,8 +998,10 @@ end_user_display(MsnSlpCall *slpcall)
 	/* Free one window slot */
 	userlist->buddy_icon_window++;
 
+#ifdef MSN_DEBUG_UD
 	gaim_debug_info("msn", "buddy_icon_window=%d\n",
 					userlist->buddy_icon_window);
+#endif
 
 	msn_release_buddy_icon_request(userlist);
 }
@@ -993,7 +1039,9 @@ msn_request_user_display(MsnUser *user)
 		const char *my_info = NULL;
 		GSList *sl;
 
+#ifdef MSN_DEBUG_UD
 		gaim_debug_info("msn", "Requesting our own user display\n");
+#endif
 
 		my_obj = msn_user_get_object(session->user);
 
@@ -1021,8 +1069,10 @@ msn_request_user_display(MsnUser *user)
 		/* Free one window slot */
 		session->userlist->buddy_icon_window++;
 
+#ifdef MSN_DEBUG_UD
 		gaim_debug_info("msn", "buddy_icon_window=%d\n",
 						session->userlist->buddy_icon_window);
+#endif
 
 		msn_release_buddy_icon_request(session->userlist);
 	}
