@@ -31,20 +31,17 @@
 /* This is the template that we wrap outgoing messages in, since the other
  * GW Messenger clients expect messages to be in RTF.
  */
-#define RTF_TEMPLATE 	"{\\rtf1\\fbidis\\ansi\\ansicpg1252\\deff0\\deflang1033"\
-                        "{\\fonttbl{\\f0\\fswiss\\fprq2\\fcharset0 "\
-                        "Microsoft Sans Serif;}}\n{\\colortbl ;\\red0"\
-                        "\\green0\\blue0;}\n\\viewkind4\\uc1\\pard\\ltrpar"\
-                        "\\li50\\ri50\\cf1\\f0\\fs20 %s\\par\n}"
+#define RTF_TEMPLATE 	"{\\rtf1\\ansi\n"\
+                        "{\\fonttbl{\\f0\\fnil Unknown;}}\n"\
+						"{\\colortbl ;\\red0\\green0\\blue0;}\n"\
+						"\\uc1\\cf1\\f0\\fs24 %s\\par\n}"
 #define NM_MAX_MESSAGE_SIZE 2048
 
 static NMERR_T nm_process_response(NMUser * user);
-
 static void _update_contact_list(NMUser * user, NMField * fields);
-
-static void
-_handle_multiple_get_details_login_cb(NMUser * user, NMERR_T ret_code,
-									  gpointer resp_data, gpointer user_data);
+static void _handle_multiple_get_details_login_cb(NMUser * user, NMERR_T ret_code,
+												  gpointer resp_data, gpointer user_data);
+static char * nm_rtfize_text(char *text);
 
 /**
  * See header for comments on on "public" functions
@@ -60,7 +57,7 @@ nm_initialize_user(const char *name, const char *server_addr,
 
 	user = g_new0(NMUser, 1);
 
-	user->conn = g_new0(NMConn, 1);
+
 
 	user->contacts =
 		g_hash_table_new_full(g_str_hash, nm_utf8_str_equal,
@@ -74,6 +71,7 @@ nm_initialize_user(const char *name, const char *server_addr,
 												   g_free, g_free);
 
 	user->name = g_strdup(name);
+	user->conn = nm_create_conn(server_addr, port);
 	user->conn->addr = g_strdup(server_addr);
 	user->conn->port = port;
 	user->evt_callback = event_callback;
@@ -86,10 +84,7 @@ nm_initialize_user(const char *name, const char *server_addr,
 void
 nm_deinitialize_user(NMUser * user)
 {
-	NMConn *conn = user->conn;
-
-	g_free(conn->addr);
-	g_free(conn);
+	nm_release_conn(user->conn);
 
 	if (user->contacts) {
 		g_hash_table_destroy(user->contacts);
@@ -123,7 +118,6 @@ nm_send_login(NMUser * user, const char *pwd, const char *my_addr,
 {
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
-	NMRequest *req = NULL;
 
 	if (user == NULL || pwd == NULL || user_agent == NULL) {
 		return NMERR_BAD_PARM;
@@ -146,21 +140,9 @@ nm_send_login(NMUser * user, const char *pwd, const char *my_addr,
 	}
 
 	/* Send the login */
-	rc = nm_send_request(user->conn, "login", fields, &req);
-	if (rc == NM_OK && req != NULL) {
-		nm_request_set_callback(req, callback);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields) {
-		nm_free_fields(&fields);
-	}
-
-	if (req) {
-		nm_release_request(req);
-	}
+	rc = nm_send_request(user->conn, "login", fields, callback, data, NULL);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -170,7 +152,6 @@ nm_send_set_status(NMUser * user, int status, const char *text,
 {
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
-	NMRequest *req = NULL;
 
 	if (user == NULL)
 		return NMERR_BAD_PARM;
@@ -192,21 +173,9 @@ nm_send_set_status(NMUser * user, int status, const char *text,
 									  NMFIELD_TYPE_UTF8);
 	}
 
-	rc = nm_send_request(user->conn, "setstatus", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields) {
-		nm_free_fields(&fields);
-	}
-
-	if (req) {
-		nm_release_request(req);
-	}
+	rc = nm_send_request(user->conn, "setstatus", fields, callback, data, NULL);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -216,7 +185,6 @@ nm_send_multiple_get_details(NMUser * user, GSList *names,
 {
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
-	NMRequest *req = NULL;
 	GSList *node;
 
 	if (user == NULL || names == NULL)
@@ -228,19 +196,9 @@ nm_send_multiple_get_details(NMUser * user, GSList *names,
 									  g_strdup(node->data), NMFIELD_TYPE_UTF8);
 	}
 
-	rc = nm_send_request(user->conn, "getdetails", fields, &req);
-	if (rc == NM_OK) {
-		nm_request_set_callback(req, callback);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
-
-	if (req)
-		nm_release_request(req);
+	rc = nm_send_request(user->conn, "getdetails", fields, callback, data, NULL);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -250,7 +208,6 @@ nm_send_get_details(NMUser * user, const char *name,
 {
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
-	NMRequest *req = NULL;
 
 	if (user == NULL || name == NULL)
 		return NMERR_BAD_PARM;
@@ -262,7 +219,6 @@ nm_send_get_details(NMUser * user, const char *name,
 	} else {
 
 		const char *dn = nm_lookup_dn(user, name);
-
 		if (dn) {
 			fields = nm_field_add_pointer(fields, NM_A_SZ_DN, 0, NMFIELD_METHOD_VALID, 0,
 										  g_strdup(name), NMFIELD_TYPE_DN);
@@ -274,25 +230,15 @@ nm_send_get_details(NMUser * user, const char *name,
 
 	}
 
-	rc = nm_send_request(user->conn, "getdetails", fields, &req);
-	if (rc == NM_OK) {
-		nm_request_set_callback(req, callback);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
-
-	if (req)
-		nm_release_request(req);
+	rc = nm_send_request(user->conn, "getdetails", fields, callback, data, NULL);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
 NMERR_T
 nm_send_create_conference(NMUser * user, NMConference * conference,
-						  nm_response_cb callback, gpointer message)
+						  nm_response_cb callback, gpointer data)
 {
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
@@ -336,29 +282,23 @@ nm_send_create_conference(NMUser * user, NMConference * conference,
 									  NMFIELD_TYPE_DN);
 	}
 
-	rc = nm_send_request(user->conn, "createconf", fields, &req);
+	rc = nm_send_request(user->conn, "createconf", fields, callback, data, &req);
 	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
 		nm_conference_add_ref(conference);
 		nm_request_set_data(req, conference);
-		nm_request_set_user_define(req, message);
-		nm_conn_add_request_item(user->conn, req);
 	}
 
 	if (req)
 		nm_release_request(req);
 
-	if (fields)
-		nm_free_fields(&fields);
-
+	nm_free_fields(&fields);
 	return rc;
 }
 
 NMERR_T
 nm_send_leave_conference(NMUser * user, NMConference * conference,
-						 nm_response_cb callback, gpointer message)
+						 nm_response_cb callback, gpointer data)
 {
-
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
 	NMField *tmp = NULL;
@@ -378,20 +318,14 @@ nm_send_leave_conference(NMUser * user, NMConference * conference,
 	tmp = NULL;
 
 	/* Send the request to the server */
-	rc = nm_send_request(user->conn, "leaveconf", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "leaveconf", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, conference);
-		nm_request_set_user_define(req, message);
-		nm_conn_add_request_item(user->conn, req);
-	}
 
 	if (req)
 		nm_release_request(req);
 
-	if (fields)
-		nm_free_fields(&fields);
-
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -417,24 +351,14 @@ nm_send_join_conference(NMUser * user, NMConference * conference,
 	tmp = NULL;
 
 	/* Send the request to the server */
-	rc = nm_send_request(user->conn, "joinconf", fields, &req);
-
-	/* Set up the request object so that we know what to do
-	 * when we get a response
-	 */
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "joinconf", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, conference);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
 
 	if (req)
 		nm_release_request(req);
 
-	if (fields)
-		nm_free_fields(&fields);
-
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -461,24 +385,14 @@ nm_send_reject_conference(NMUser * user, NMConference * conference,
 	tmp = NULL;
 
 	/* Send the request to the server */
-	rc = nm_send_request(user->conn, "rejectconf", fields, &req);
-
-	/* Set up the request object so that we know what to do
-	 * when we get a response
-	 */
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "rejectconf", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, conference);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
 
 	if (req)
 		nm_release_request(req);
 
-	if (fields)
-		nm_free_fields(&fields);
-
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -515,24 +429,14 @@ nm_send_conference_invite(NMUser *user, NMConference *conference, NMUserRecord *
 									  g_strdup(message), NMFIELD_TYPE_UTF8);
 
 	/* Send the request to the server */
-	rc = nm_send_request(user->conn, "sendinvite", fields, &req);
-
-	/* Set up the request object so that we know what to do
-	 * when we get a response
-	 */
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "sendinvite", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, conference);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
 
 	if (req)
 		nm_release_request(req);
 
-	if (fields)
-		nm_free_fields(&fields);
-
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -540,9 +444,8 @@ NMERR_T
 nm_send_message(NMUser * user, NMMessage * message, nm_response_cb callback)
 {
 	NMERR_T rc = NM_OK;
-	char *text;
+	char *text, *rtfized;
 	NMField *fields = NULL, *tmp = NULL;
-	NMRequest *req = NULL;
 	NMConference *conf;
 	NMUserRecord *user_record;
 	int count, i;
@@ -572,21 +475,24 @@ nm_send_message(NMUser * user, NMMessage * message, nm_response_cb callback)
 		if (strlen(text) > NM_MAX_MESSAGE_SIZE)
 			text[NM_MAX_MESSAGE_SIZE] = 0;
 
+		rtfized = nm_rtfize_text(text);
+
+		gaim_debug_info("novell", "message text is: %s\n", text);
+		gaim_debug_info("novell", "message rtf is: %s\n", rtfized);
+
 		tmp = nm_field_add_pointer(tmp, NM_A_SZ_MESSAGE_BODY, 0, NMFIELD_METHOD_VALID, 0,
-								   g_strdup_printf(RTF_TEMPLATE, text), NMFIELD_TYPE_UTF8);
+								   rtfized, NMFIELD_TYPE_UTF8);
 
 		tmp = nm_field_add_number(tmp, NM_A_UD_MESSAGE_TYPE, 0, NMFIELD_METHOD_VALID, 0,
 								  0, NMFIELD_TYPE_UDWORD);
 
 		tmp = nm_field_add_pointer(tmp, NM_A_SZ_MESSAGE_TEXT, 0, NMFIELD_METHOD_VALID, 0,
-								   g_strdup(text), NMFIELD_TYPE_UTF8);
+								   text, NMFIELD_TYPE_UTF8);
 
 		fields = nm_field_add_pointer(fields, NM_A_FA_MESSAGE, 0, NMFIELD_METHOD_VALID, 0,
 									  tmp, NMFIELD_TYPE_ARRAY);
 		tmp = NULL;
 
-		g_free(text);
-
 		/* Add participants */
 		count = nm_conference_get_participant_count(conf);
 		for (i = 0; i < count; i++) {
@@ -600,21 +506,10 @@ nm_send_message(NMUser * user, NMMessage * message, nm_response_cb callback)
 		}
 
 		/* Send the request */
-		rc = nm_send_request(user->conn, "sendmessage", fields, &req);
-		if (rc == NM_OK && req) {
-			nm_request_set_callback(req, callback);
-			nm_conn_add_request_item(user->conn, req);
-		}
-	}
-
-	if (fields) {
-		nm_free_fields(&fields);
-	}
-
-	if (req) {
-		nm_release_request(req);
+		rc = nm_send_request(user->conn, "sendmessage", fields, callback, NULL, NULL);
 	}
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -625,7 +520,6 @@ nm_send_typing(NMUser * user, NMConference * conf,
 	NMERR_T rc = NM_OK;
 	char *str = NULL;
 	NMField *fields = NULL, *tmp = NULL;
-	NMRequest *req = NULL;
 
 	if (user == NULL || conf == NULL) {
 		return NMERR_BAD_PARM;
@@ -652,19 +546,10 @@ nm_send_typing(NMUser * user, NMConference * conf,
 								 tmp, NMFIELD_TYPE_ARRAY);
 		tmp = NULL;
 
-		rc = nm_send_request(user->conn, "sendtyping", fields, &req);
-		if (rc == NM_OK && req) {
-			nm_request_set_callback(req, callback);
-			nm_conn_add_request_item(user->conn, req);
-		}
+		rc = nm_send_request(user->conn, "sendtyping", fields, callback, NULL, NULL);
 	}
 
-	if (req)
-		nm_release_request(req);
-
-	if (fields)
-		nm_free_fields(&fields);
-
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -712,20 +597,14 @@ nm_send_create_contact(NMUser * user, NMFolder * folder,
 									  g_strdup(display_name), NMFIELD_TYPE_UTF8);
 
 	/* Dispatch the request */
-	rc = nm_send_request(user->conn, "createcontact", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "createcontact", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, contact);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
 
 	if (req)
 		nm_release_request(req);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -753,20 +632,14 @@ nm_send_remove_contact(NMUser * user, NMFolder * folder,
 								  NMFIELD_TYPE_UTF8);
 
 	/* Dispatch the request */
-	rc = nm_send_request(user->conn, "deletecontact", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "deletecontact", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, contact);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
 
 	if (req)
 		nm_release_request(req);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -797,20 +670,14 @@ nm_send_create_folder(NMUser * user, const char *name,
 							 g_strdup("-1"), NMFIELD_TYPE_UTF8);
 
 	/* Dispatch the request */
-	rc = nm_send_request(user->conn, "createfolder", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "createfolder", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, g_strdup(name));
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
 
 	if (req)
 		nm_release_request(req);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -832,20 +699,14 @@ nm_send_remove_folder(NMUser * user, NMFolder * folder,
 								  NMFIELD_TYPE_UTF8);
 
 	/* Dispatch the request */
-	rc = nm_send_request(user->conn, "deletecontact", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "deletecontact", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, folder);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
 
 	if (req)
 		nm_release_request(req);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -870,20 +731,14 @@ nm_send_get_status(NMUser * user, NMUserRecord * user_record,
 								  g_strdup(dn), NMFIELD_TYPE_UTF8);
 
 	/* Dispatch the request */
-	rc = nm_send_request(user->conn, "getstatus", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
+	rc = nm_send_request(user->conn, "getstatus", fields, callback, data, &req);
+	if (rc == NM_OK && req)
 		nm_request_set_data(req, user_record);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
 
 	if (req)
 		nm_release_request(req);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -925,16 +780,15 @@ nm_send_rename_contact(NMUser * user, NMContact * contact,
 									 0, fields, NMFIELD_TYPE_ARRAY);
 			fields = NULL;
 
-			rc = nm_send_request(user->conn, "updateitem", list, &req);
-			if (rc == NM_OK && req) {
-				nm_request_set_callback(req, callback);
+			rc = nm_send_request(user->conn, "updateitem", list, callback, data, &req);
+			if (rc == NM_OK && req)
 				nm_request_set_data(req, contact);
-				nm_request_set_user_define(req, data);
-				nm_conn_add_request_item(user->conn, req);
-			}
 		}
 	}
 
+	if (req)
+		nm_release_request(req);
+
 	if (list)
 		nm_free_fields(&list);
 
@@ -979,16 +833,15 @@ nm_send_rename_folder(NMUser * user, NMFolder * folder, const char *new_name,
 										0, fields, NMFIELD_TYPE_ARRAY);
 			fields = NULL;
 
-			rc = nm_send_request(user->conn, "updateitem", list, &req);
-			if (rc == NM_OK && req) {
-				nm_request_set_callback(req, callback);
+			rc = nm_send_request(user->conn, "updateitem", list, callback, data, &req);
+			if (rc == NM_OK && req)
 				nm_request_set_data(req, folder);
-				nm_request_set_user_define(req, data);
-				nm_conn_add_request_item(user->conn, req);
-			}
 		}
 	}
 
+	if (req)
+		nm_release_request(req);
+
 	if (list)
 		nm_free_fields(&list);
 
@@ -1029,16 +882,15 @@ nm_send_move_contact(NMUser * user, NMContact * contact, NMFolder * folder,
 									NMFIELD_TYPE_UTF8);
 
 		/* Dispatch the request */
-		rc = nm_send_request(user->conn, "movecontact", list, &req);
-		if (rc == NM_OK && req) {
-			nm_request_set_callback(req, callback);
+		rc = nm_send_request(user->conn, "movecontact", list, callback, data, &req);
+		if (rc == NM_OK && req)
 			nm_request_set_data(req, contact);
-			nm_request_set_user_define(req, data);
-			nm_conn_add_request_item(user->conn, req);
 
-		}
 	}
 
+	if (req)
+		nm_release_request(req);
+
 	if (list)
 		nm_free_fields(&list);
 
@@ -1052,7 +904,6 @@ nm_send_create_privacy_item(NMUser *user, const char *who, gboolean allow_list,
 {
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
-	NMRequest *req = NULL;
 	const char *tag;
 
 	if (user == NULL || who == NULL)
@@ -1066,16 +917,9 @@ nm_send_create_privacy_item(NMUser *user, const char *who, gboolean allow_list,
     fields = nm_field_add_pointer(fields, tag, 0, NMFIELD_METHOD_ADD, 0,
 								  g_strdup(who), NMFIELD_TYPE_UTF8);
 
-	rc = nm_send_request(user->conn, "createblock", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
+	rc = nm_send_request(user->conn, "createblock", fields, callback, data, NULL);
 
+	nm_free_fields(&fields);
 	return rc;
 }
 
@@ -1085,7 +929,6 @@ nm_send_remove_privacy_item(NMUser *user, const char *dn, gboolean allow_list,
 {
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
-	NMRequest *req = NULL;
 	const char *tag;
 	GSList **list_ptr, *node;
 
@@ -1109,19 +952,9 @@ nm_send_remove_privacy_item(NMUser *user, const char *dn, gboolean allow_list,
     fields = nm_field_add_pointer(fields, tag, 0, NMFIELD_METHOD_DELETE, 0,
 								  g_strdup(dn), NMFIELD_TYPE_DN);
 
-	rc = nm_send_request(user->conn, "updateblocks", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
-
-	if (rc == NM_OK) {
-	}
-
-	if (fields)
-		nm_free_fields(&fields);
+	rc = nm_send_request(user->conn, "updateblocks", fields, callback, data, NULL);
 
+	nm_free_fields(&fields);
 	return rc;
 
 }
@@ -1132,7 +965,6 @@ nm_send_set_privacy_default(NMUser *user, gboolean default_deny,
 {
 	NMERR_T rc = NM_OK;
 	NMField *fields = NULL;
-	NMRequest *req = NULL;
 
 	if (user == NULL)
 		return NMERR_BAD_PARM;
@@ -1141,15 +973,21 @@ nm_send_set_privacy_default(NMUser *user, gboolean default_deny,
 								  (default_deny ? g_strdup("1") : g_strdup("0")),
 								  NMFIELD_TYPE_UTF8);
 
-	rc = nm_send_request(user->conn, "updateblocks", fields, &req);
-	if (rc == NM_OK && req) {
-		nm_request_set_callback(req, callback);
-		nm_request_set_user_define(req, data);
-		nm_conn_add_request_item(user->conn, req);
-	}
+	rc = nm_send_request(user->conn, "updateblocks", fields, callback, data, NULL);
 
-	if (fields)
-		nm_free_fields(&fields);
+	nm_free_fields(&fields);
+	return rc;
+}
+
+NMERR_T
+nm_send_keepalive(NMUser *user, nm_response_cb callback, gpointer data)
+{
+	NMERR_T rc = NM_OK;
+
+	if (user == NULL)
+		return NMERR_BAD_PARM;
+
+	rc = nm_send_request(user->conn, "ping", NULL, callback, data, NULL);
 
 	return rc;
 }
@@ -1449,6 +1287,7 @@ _create_privacy_list(NMUser * user, NMRequest *request)
 
 	if (need_details) {
 
+		nm_request_add_ref(request);
 		nm_send_multiple_get_details(user, need_details,
 									 _handle_multiple_get_details_login_cb, request);
 
@@ -1648,6 +1487,7 @@ _handle_multiple_get_details_login_cb(NMUser * user, NMERR_T ret_code,
 	if ((cb = nm_request_get_callback(request))) {
 		cb(user, ret_code, nm_request_get_data(request),
 		   nm_request_get_user_define(request));
+		nm_release_request(request);
 	}
 }
 
@@ -1793,8 +1633,8 @@ nm_call_handler(NMUser * user, NMRequest * request, NMField * fields)
 				if (list != NULL) {
 
 					done = FALSE;
-					nm_request_add_ref(request);
 					nm_request_set_user_define(request, list);
+					nm_request_add_ref(request);
 					for (node = list; node; node = node->next) {
 
 						nm_send_get_details(user, (const char *) node->data,
@@ -1909,7 +1749,6 @@ nm_call_handler(NMUser * user, NMRequest * request, NMField * fields)
 
 		cb(user, ret_code, nm_request_get_data(request),
 		   nm_request_get_user_define(request));
-
 	}
 
 	return rc;
@@ -1935,7 +1774,9 @@ nm_process_response(NMUser * user)
 			req = nm_conn_find_request(conn, atoi((char *) field->ptr_value));
 			if (req != NULL) {
 				rc = nm_call_handler(user, req, fields);
+				nm_conn_remove_request_item(conn, req);
 			}
+
 		}
 	}
 
@@ -2220,3 +2061,82 @@ _update_contact_list(NMUser * user, NMField * fields)
 		cursor++;
 	}
 }
+
+static char *
+nm_rtfize_text(char *text)
+{
+	GString *gstr = NULL;
+	unsigned char *pch;
+	char *uni_str = NULL, *rtf = NULL;
+	int bytes;
+	gunichar uc;
+
+	gstr = g_string_sized_new(strlen(text)*2);
+	pch = text;
+	while (*pch) {
+		if ((*pch) <= 0x7F) {
+			switch (*pch) {
+				case '{':
+				case '}':
+				case '\\':
+					gstr = g_string_append_c(gstr, '\\');
+					gstr = g_string_append_c(gstr, *pch);
+					break;
+				case '\n':
+					gstr = g_string_append(gstr, "\\par ");
+					break;
+				default:
+					gstr = g_string_append_c(gstr, *pch);
+					break;
+			}
+			pch++;
+		} else {
+			/* convert the utf-8 character to ucs-4 for rtf encoding */
+			if(*pch <= 0xDF) {
+				uc = ((((gunichar)pch[0]) & 0x001F) << 6) |
+					(((gunichar)pch[1]) & 0x003F);
+				bytes = 2;
+			} else if(*pch <= 0xEF) {
+				uc = ((((gunichar)pch[0]) & 0x000F) << 12) |
+					((((gunichar)pch[1]) & 0x003F) << 6) |
+					(((gunichar)pch[2]) & 0x003F);
+				bytes = 3;
+			} else if (*pch <= 0xF7) {
+				uc = ((((gunichar)pch[0]) & 0x0007) << 18) |
+					((((gunichar)pch[1]) & 0x003F) << 12) |
+					((((gunichar)pch[2]) & 0x003F) << 6) |
+					(((gunichar)pch[3]) & 0x003F);
+				bytes = 4;
+			} else if (*pch <= 0xFB) {
+				uc = ((((gunichar)pch[0]) & 0x0003) << 24) |
+					((((gunichar)pch[1]) & 0x003F) << 18) |
+					((((gunichar)pch[2]) & 0x003F) << 12) |
+					((((gunichar)pch[3]) & 0x003F) << 6) |
+					(((gunichar)pch[4]) & 0x003F);
+				bytes = 5;
+			} else if (*pch <= 0xFD) {
+				uc = ((((gunichar)pch[0]) & 0x0001) << 30) |
+					((((gunichar)pch[1]) & 0x003F) << 24) |
+					((((gunichar)pch[2]) & 0x003F) << 18) |
+					((((gunichar)pch[3]) & 0x003F) << 12) |
+					((((gunichar)pch[4]) & 0x003F) << 6) |
+					(((gunichar)pch[5]) & 0x003F);
+				bytes = 6;
+			} else {
+				/* should never happen ... bogus utf-8! */
+				gaim_debug_info("novell", "bogus utf-8 lead byte: 0x%X\n", pch[0]);
+				uc = 0x003F;
+				bytes = 1;
+			}
+			uni_str = g_strdup_printf("\\u%d?", uc);
+			gaim_debug_info("novell", "unicode escaped char %s\n", uni_str);
+			gstr = g_string_append(gstr, uni_str);
+			pch += bytes;
+			g_free(uni_str);
+		}
+	}
+
+	rtf = g_strdup_printf(RTF_TEMPLATE, gstr->str);
+	g_string_free(gstr, TRUE);
+	return rtf;
+}
