@@ -30,15 +30,14 @@
 #include "request.h"
 #include "network.h"
 #include "privacy.h"
-#include "multi.h"
+#include "version.h"
 
 #define DEFAULT_PORT			8300
 #define NOVELL_CONNECT_STEPS	4
+#define NM_ROOT_FOLDER_NAME "GroupWise Messenger"
 
 static GaimPlugin *my_protocol = NULL;
 
-static gboolean set_permit = FALSE;
-
 static gboolean
 _is_disconnect_error(NMERR_T err);
 
@@ -103,13 +102,12 @@ _login_resp_cb(NMUser * user, NMERR_T ret_code,
 				gaim_account_set_alias(user->client_data, alias);
 		}
 
-		_sync_contact_list(user);
-		_sync_privacy_lists(user);
-
 		/* Tell Gaim that we are connected */
 		gaim_connection_set_state(gc, GAIM_CONNECTED);
 		serv_finish_login(gc);
 
+		_sync_contact_list(user);
+
 		rc = nm_send_set_status(user, NM_STATUS_AVAILABLE, NULL, NULL, NULL,
 								NULL);
 		_check_for_disconnect(user, rc);
@@ -269,8 +267,8 @@ _get_details_resp_setup_buddy(NMUser * user, NMERR_T ret_code,
 		gaim_blist_rename_buddy(buddy,
 								nm_user_record_get_display_id(user_record));
 
-		alias = gaim_get_buddy_alias(buddy);
-		if (alias == NULL || (strcmp(alias, buddy->name) == 0)) {
+		alias = gaim_buddy_get_alias(buddy);
+		if (alias == NULL || *alias == '\0' || (strcmp(alias, buddy->name) == 0)) {
 			gaim_blist_alias_buddy(buddy,
 								   nm_user_record_get_full_name(user_record));
 
@@ -324,6 +322,9 @@ _create_contact_resp_cb(NMUser * user, NMERR_T ret_code,
 			folder_name = nm_folder_get_name(folder);
 		}
 
+		if (*folder_name == '\0')
+			folder_name = NM_ROOT_FOLDER_NAME;
+
 		/* Re-add the buddy now that we got the okay from the server */
 		if (folder_name && (group = gaim_find_group(folder_name))) {
 
@@ -652,7 +653,8 @@ _join_conf_resp_cb(NMUser * user, NMERR_T ret_code,
 				ur = nm_conference_get_participant(conference, i);
 				if (ur) {
 					name = nm_user_record_get_display_id(ur);
-					gaim_conv_chat_add_user(GAIM_CONV_CHAT(chat), name, NULL);
+					gaim_conv_chat_add_user(GAIM_CONV_CHAT(chat), name, NULL, 
+											GAIM_CBFLAGS_NONE, TRUE);
 				}
 			}
 		}
@@ -701,7 +703,7 @@ _get_details_resp_add_privacy_item(NMUser *user, NMERR_T ret_code,
 	GaimConnection *gc;
 	NMUserRecord *user_record = resp_data;
 	char *err;
-	gboolean allowed = (gboolean)user_data;
+	gboolean allowed = GPOINTER_TO_INT(user_data);
 	const char *display_id;
 
 	if (user == NULL)
@@ -848,7 +850,7 @@ _get_details_send_privacy_create(NMUser *user, NMERR_T ret_code,
 	GaimConnection *gc;
 	NMUserRecord *user_record = resp_data;
 	char *err;
-	gboolean allowed = (gboolean)user_data;
+	gboolean allowed = GPOINTER_TO_INT(user_data);
 	const char *dn, *display_id;
 
 	if (user == NULL)
@@ -1148,7 +1150,7 @@ _update_buddy_status(GaimBuddy * buddy, int status, int gmt)
 	GaimConnection *gc = gaim_account_get_connection(buddy->account);
 	int gstatus = status << 1;
 	int idle = 0;
-	int loggedin = 1;
+	gboolean loggedin = TRUE;
 
 	switch (status) {
 		case NM_STATUS_AVAILABLE:
@@ -1159,7 +1161,7 @@ _update_buddy_status(GaimBuddy * buddy, int status, int gmt)
 			gstatus |= UC_UNAVAILABLE;
 			break;
 		case NM_STATUS_OFFLINE:
-			loggedin = 0;
+			loggedin = FALSE;
 			gstatus |= UC_UNAVAILABLE;
 			break;
 		case NM_STATUS_AWAY_IDLE:
@@ -1168,7 +1170,7 @@ _update_buddy_status(GaimBuddy * buddy, int status, int gmt)
 			break;
 		default:
 			gstatus |= UC_UNAVAILABLE;
-			loggedin = 0;
+			loggedin = FALSE;
 			break;
 	}
 
@@ -1190,6 +1192,7 @@ _remove_gaim_buddies(NMUser *user)
 	GSList *rem_list = NULL;
 	GSList *l;
 	NMFolder *folder = NULL;
+	const char *gname = NULL;
 
 	if ((blist = gaim_get_blist())) {
 		for (gnode = blist->root; gnode; gnode = gnode->next) {
@@ -1204,7 +1207,10 @@ _remove_gaim_buddies(NMUser *user)
 						continue;
 					buddy = (GaimBuddy *) bnode;
 					if (buddy->account == user->client_data) {
-						folder = nm_find_folder(user, group->name);
+						gname = group->name;
+						if (strcmp(group->name, NM_ROOT_FOLDER_NAME) == 0)
+							gname = "";
+						folder = nm_find_folder(user, gname);
 						if (folder == NULL ||
 							!nm_folder_find_contact_by_display_id(folder, buddy->name)) {
 							rem_list = g_slist_append(rem_list, buddy);
@@ -1234,13 +1240,21 @@ _add_contacts_to_gaim_blist(NMUser * user, NMFolder * folder)
 	NMERR_T cnt = 0, i;
 	const char *text = NULL;
 	const char *name = NULL;
+	const char *fname = NULL;
 	int status = 0;
 
-	/* Does the Gaim group exist already? */
-	group = gaim_find_group(nm_folder_get_name(folder));
+	/* If this is the root folder give it a name. Gaim does not have the concept of
+	 * a root folder.
+	 */
+	fname = nm_folder_get_name(folder);
+	if (fname == NULL || *fname == '\0') {
+		fname = NM_ROOT_FOLDER_NAME;
+	}
 
+	/* Does the Gaim group exist already? */
+	group = gaim_find_group(fname);
 	if (group == NULL) {
-		group = gaim_group_new(nm_folder_get_name(folder));
+		group = gaim_group_new(fname);
 		gaim_blist_add_group(group, NULL);
 	}
 
@@ -1288,16 +1302,20 @@ _add_contacts_to_gaim_blist(NMUser * user, NMFolder * folder)
 static void
 _add_gaim_buddies(NMUser * user)
 {
-	NMERR_T cnt = 0, i;
+	int cnt = 0, i;
 	NMFolder *root_folder = NULL;
 	NMFolder *folder = NULL;
 
 	root_folder = nm_get_root_folder(user);
 	if (root_folder) {
 
-		/* Add contacts for the sub folders */
+		/* Add sub-folders and contacts to sub-folders...
+		 * iterate throught the sub-folders in reverse order
+		 * because Gaim adds the folders to the front -- so we
+		 * want to add the first folder last
+		 */
 		cnt = nm_folder_get_subfolder_count(root_folder);
-		for (i = 0; i < cnt; i++) {
+		for (i = cnt-1; i >= 0; i--) {
 			folder = nm_folder_get_subfolder(root_folder, i);
 			if (folder) {
 				_add_contacts_to_gaim_blist(user, folder);
@@ -1319,6 +1337,7 @@ _sync_contact_list(NMUser *user)
 	 */
 	_remove_gaim_buddies(user);
 	_add_gaim_buddies(user);
+	user->clist_synched = TRUE;
 }
 
 static void
@@ -1414,6 +1433,30 @@ _sync_privacy_lists(NMUser *user)
 	}
 }
 
+ /* Map known property tags to user-friendly strings */
+static const char *
+_map_property_tag(const char *tag)
+{
+	if (tag == NULL) return NULL;
+
+	if (strcmp(tag, "telephoneNumber") == 0)
+		return _("Telephone Number");
+	else if (strcmp(tag, "L") == 0)
+		return _("Location");
+	else if (strcmp(tag, "OU") == 0)
+		return _("Department");
+	else if (strcmp(tag, "personalTitle") == 0)
+		return _("Personal Title");
+	else if (strcmp(tag, "Title") == 0)
+		return _("Title");
+	else if (strcmp(tag, "mailstop") == 0)
+		return _("Mailstop");
+	else if (strcmp(tag, "Internet EMail Address") == 0)
+		return _("Email Address");
+	else
+		return tag;
+}
+
 /* Display a dialog box showing the properties for the given user record */
 static void
 _show_info(GaimConnection * gc, NMUserRecord * user_record)
@@ -1425,16 +1468,16 @@ _show_info(GaimConnection * gc, NMUserRecord * user_record)
 
 	info_text = g_string_new("");
 
-	tag = _("Userid");
+	tag = _("User ID");
 	value = nm_user_record_get_userid(user_record);
 	if (value) {
-		g_string_append_printf(info_text, "<b>%s:</b> %s<br/>\n", tag, value);
+		g_string_append_printf(info_text, "<b>%s:</b> %s<br>", tag, value);
 	}
 
 /*	tag = _("DN");
 	value = nm_user_record_get_dn(user_record);
 	if (value) {
-	g_string_append_printf(info_text, "<b>%s:</b> %s<br/>\n",
+	g_string_append_printf(info_text, "<b>%s:</b> %s<br>",
 	tag, value);
 	}
 */
@@ -1442,25 +1485,26 @@ _show_info(GaimConnection * gc, NMUserRecord * user_record)
 	tag = _("Full name");
 	value = nm_user_record_get_full_name(user_record);
 	if (value) {
-		g_string_append_printf(info_text, "<b>%s:</b> %s<br/>\n", tag, value);
+		g_string_append_printf(info_text, "<b>%s:</b> %s<br>", tag, value);
 	}
 
 	count = nm_user_record_get_property_count(user_record);
 	for (i = 0; i < count; i++) {
 		property = nm_user_record_get_property(user_record, i);
 		if (property) {
-			tag = nm_property_get_tag(property);
+			tag = _map_property_tag(nm_property_get_tag(property));
 			value = nm_property_get_value(property);
 			if (tag && value) {
-				g_string_append_printf(info_text, "<b>%s:</b> %s<br/>\n",
+				g_string_append_printf(info_text, "<b>%s:</b> %s<br>",
 									   tag, value);
 			}
 			nm_release_property(property);
 		}
 	}
 
-	gaim_notify_formatted(NULL, NULL, _("User Properties"),
-						  NULL, info_text->str, NULL, NULL);
+	gaim_notify_userinfo(gc, nm_user_record_get_userid(user_record), NULL,
+						 _("User Properties"), NULL, info_text->str,
+						 NULL, NULL);
 
 	g_string_free(info_text, TRUE);
 }
@@ -1685,6 +1729,9 @@ _evt_receive_message(NMUser * user, NMEvent * event)
 	GaimConversation *gconv;
 	NMConference *conference;
 	GaimConvImFlags imflags;
+	char *text = NULL;
+
+	text = g_markup_escape_text(nm_event_get_text(event), -1);
 
 	conference = nm_event_get_conference(event);
 	if (conference) {
@@ -1703,7 +1750,7 @@ _evt_receive_message(NMUser * user, NMEvent * event)
 
 				serv_got_im(gaim_account_get_connection(user->client_data),
 							nm_user_record_get_display_id(user_record),
-							nm_event_get_text(event), imflags,
+							text, imflags,
 							nm_event_get_gmt(event));
 
 				gconv =	gaim_find_conversation_with_account(
@@ -1715,8 +1762,7 @@ _evt_receive_message(NMUser * user, NMEvent * event)
 					if (contact) {
 
 						gaim_conversation_set_title(
-							gconv,
-							nm_contact_get_display_name(contact));
+							gconv, nm_contact_get_display_name(contact));
 
 
 					} else {
@@ -1759,12 +1805,12 @@ _evt_receive_message(NMUser * user, NMEvent * event)
 
 				serv_got_chat_in(gaim_account_get_connection(user->client_data),
 								 gaim_conv_chat_get_id(GAIM_CONV_CHAT(chat)),
-								 name,
-								 0, nm_event_get_text(event),
-								 nm_event_get_gmt(event));
+								 name, 0, text, nm_event_get_gmt(event));
 			}
 		}
 	}
+
+	g_free(text);
 }
 
 static void
@@ -1841,7 +1887,8 @@ _evt_conference_invite(NMUser * user, NMEvent * event)
 	parms = g_slist_append(parms, nm_event_get_conference(event));
 
 	/* Prompt the user */
-	gaim_request_action(NULL, title, primary, secondary, -1, parms, 2,
+	gaim_request_action(NULL, title, primary, secondary, 
+						GAIM_DEFAULT_ACTION_NONE, parms, 2,
 						_("Yes"), G_CALLBACK(_join_conference_cb),
 						_("No"), G_CALLBACK(_reject_conference_cb));
 
@@ -1858,7 +1905,6 @@ _evt_conference_joined(NMUser * user, NMEvent * event)
 	NMUserRecord *ur = NULL;
 	const char *name;
 	const char *conf_name;
-	GList *list = NULL;
 
 	gc = gaim_account_get_connection(user->client_data);
 	if (gc == NULL)
@@ -1878,7 +1924,8 @@ _evt_conference_joined(NMUser * user, NMEvent * event)
 					nm_conference_set_data(conference, (gpointer) chat);
 
 					name = nm_user_record_get_display_id(ur);
-					gaim_conv_chat_add_user(GAIM_CONV_CHAT(chat), name, NULL);
+					gaim_conv_chat_add_user(GAIM_CONV_CHAT(chat), name, NULL, 
+											GAIM_CBFLAGS_NONE, TRUE);
 
 				}
 			}
@@ -1888,9 +1935,9 @@ _evt_conference_joined(NMUser * user, NMEvent * event)
 			ur = nm_find_user_record(user, nm_event_get_source(event));
 			if (ur) {
 				name = nm_user_record_get_display_id(ur);
-				list = gaim_conv_chat_get_users(GAIM_CONV_CHAT(chat));
-				if (!g_list_find_custom(list, name, (GCompareFunc)nm_utf8_strcasecmp)) {
-					gaim_conv_chat_add_user(GAIM_CONV_CHAT(chat), name, NULL);
+				if (!gaim_conv_chat_find_user(GAIM_CONV_CHAT(chat), name)) {
+					gaim_conv_chat_add_user(GAIM_CONV_CHAT(chat), name, NULL, 
+											GAIM_CBFLAGS_NONE, TRUE);
 				}
 			}
 		}
@@ -2127,13 +2174,8 @@ novell_close(GaimConnection * gc)
 	user = gc->proto_data;
 	if (user) {
 		conn = user->conn;
-		if (conn) {
-			if (conn->use_ssl && user->conn->ssl_conn) {
-				gaim_ssl_close(user->conn->ssl_conn->data);
-			} else {
-				gaim_input_remove(gc->inpa);
-				close(conn->fd);
-			}
+		if (conn && conn->ssl_conn) {
+			gaim_ssl_close(user->conn->ssl_conn->data);
 		}
 		nm_deinitialize_user(user);
 	}
@@ -2161,7 +2203,7 @@ novell_send_im(GaimConnection * gc, const char *name,
 		return 0;
 
 	/* Create a new message */
-	message = nm_create_message(gaim_markup_strip_html(message_body));
+	message = nm_create_message(message_body);
 
 	/* Need to get the DN for the buddy so we can look up the convo */
 	dn = nm_lookup_dn(user, name);
@@ -2334,7 +2376,7 @@ novell_chat_invite(GaimConnection *gc, int id,
 
 	user_record = nm_find_user_record(user, who);
 	if (user_record == NULL) {
-		rc = nm_send_get_details(user, who, _get_details_resp_send_invite, (gpointer)id);
+		rc = nm_send_get_details(user, who, _get_details_resp_send_invite, GINT_TO_POINTER(id));
 		_check_for_disconnect(user, rc);
 		return;
 	}
@@ -2371,7 +2413,7 @@ novell_chat_send(GaimConnection * gc, int id, const char *text)
 	if (user == NULL)
 		return -1;
 
-	message = nm_create_message(gaim_markup_strip_html(text));
+	message = nm_create_message(text);
 
 	for (cnode = user->conferences; cnode != NULL; cnode = cnode->next) {
 		conference = cnode->data;
@@ -2431,39 +2473,47 @@ novell_chat_send(GaimConnection * gc, int id, const char *text)
 }
 
 static void
-novell_add_buddy(GaimConnection * gc, const char *name, GaimGroup * group)
+novell_add_buddy(GaimConnection * gc, GaimBuddy *buddy, GaimGroup * group)
 {
-	GaimBuddy *buddy;
 	NMFolder *folder = NULL;
 	NMContact *contact;
 	NMUser *user;
 	NMERR_T rc = NM_OK;
+	const char *alias, *gname;
 
-	if (gc == NULL || name == NULL || group == NULL)
+	if (gc == NULL || buddy == NULL || group == NULL)
 		return;
 
 	user = (NMUser *) gc->proto_data;
 	if (user == NULL)
 		return;
 
+	/* If we haven't synched the contact list yet, ignore
+	 * the add_buddy calls. Server side list is the master.
+	 */
+	if (!user->clist_synched)
+		return;
+
 	contact = nm_create_contact();
-	nm_contact_set_dn(contact, name);
+	nm_contact_set_dn(contact, buddy->name);
 
 	/* Remove the GaimBuddy (we will add it back after adding it
 	 * to the server side list). Save the alias if there is one.
 	 */
-	buddy = gaim_find_buddy_in_group(user->client_data, name, group);
-	if (buddy) {
-		const char *alias = gaim_get_buddy_alias(buddy);
+	alias = gaim_buddy_get_alias(buddy);
+	if (alias && strcmp(alias, buddy->name))
+		nm_contact_set_display_name(contact, alias);
 
-		if (alias && strcmp(alias, name))
-			nm_contact_set_display_name(contact, gaim_get_buddy_alias(buddy));
-		gaim_blist_remove_buddy(buddy);
-		buddy = NULL;
+	gaim_blist_remove_buddy(buddy);
+	buddy = NULL;
+
+	if (strcmp(group->name, NM_ROOT_FOLDER_NAME) == 0) {
+		gname = "";
+	} else {
+		gname = group->name;
 	}
 
-
-	folder = nm_find_folder(user, group->name);
+	folder = nm_find_folder(user, gname);
 	if (folder) {
 
 		/* We have everything that we need, so send the createcontact */
@@ -2473,7 +2523,7 @@ novell_add_buddy(GaimConnection * gc, const char *name, GaimGroup * group)
 	} else {
 
 		/* Need to create the folder before we can add the contact */
-		rc = nm_send_create_folder(user, group->name,
+		rc = nm_send_create_folder(user, gname,
 								   _create_folder_resp_add_contact, contact);
 	}
 
@@ -2482,21 +2532,25 @@ novell_add_buddy(GaimConnection * gc, const char *name, GaimGroup * group)
 }
 
 static void
-novell_remove_buddy(GaimConnection * gc, const char *name, const char *group_name)
+novell_remove_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *group)
 {
 	NMContact *contact;
 	NMFolder *folder;
 	NMUser *user;
-	const char *dn;
+	const char *dn, *gname;
 	NMERR_T rc = NM_OK;
 
-	if (gc == NULL || name == NULL || group_name == NULL)
+	if (gc == NULL || buddy == NULL || group == NULL)
 		return;
 
 	user = (NMUser *) gc->proto_data;
-	if (user && (dn = nm_lookup_dn(user, name))) {
-
-		folder = nm_find_folder(user, group_name);
+	if (user && (dn = nm_lookup_dn(user, buddy->name))) {
+		if (strcmp(group->name, NM_ROOT_FOLDER_NAME) == 0) {
+			gname = "";
+		} else {
+			gname = group->name;
+		}
+		folder = nm_find_folder(user, gname);
 		if (folder) {
 			contact = nm_folder_find_contact(folder, dn);
 			if (contact) {
@@ -2514,17 +2568,17 @@ novell_remove_buddy(GaimConnection * gc, const char *name, const char *group_nam
 }
 
 static void
-novell_remove_group(GaimConnection * gc, const char *name)
+novell_remove_group(GaimConnection * gc, GaimGroup *group)
 {
 	NMUser *user;
 	NMERR_T rc = NM_OK;
 
-	if (gc == NULL || name == NULL)
+	if (gc == NULL || group == NULL)
 		return;
 
 	user = (NMUser *) gc->proto_data;
 	if (user) {
-		NMFolder *folder = nm_find_folder(user, name);
+		NMFolder *folder = nm_find_folder(user, group->name);
 
 		if (folder) {
 			rc = nm_send_remove_folder(user, folder,
@@ -2541,7 +2595,7 @@ novell_alias_buddy(GaimConnection * gc, const char *name, const char *alias)
 	NMUser *user;
 	GList *contacts = NULL;
 	GList *cnode = NULL;
-	const char *dn = NULL;
+	const char *dn = NULL, *fname = NULL;
 	NMERR_T rc = NM_OK;
 
 	if (gc == NULL || name == NULL || alias == NULL)
@@ -2555,21 +2609,28 @@ novell_alias_buddy(GaimConnection * gc, const char *name, const char *alias)
 		for (cnode = contacts; cnode != NULL; cnode = cnode->next) {
 			contact = (NMContact *) cnode->data;
 			if (contact) {
-				GaimGroup *group;
+				GaimGroup *group = NULL;
 				GaimBuddy *buddy;
 				NMFolder *folder;
 
 				/* Alias the Gaim buddy? */
 				folder = nm_find_folder_by_id(user,
 											  nm_contact_get_parent_id(contact));
-				if (folder &&
-					(group = gaim_find_group(nm_folder_get_name(folder)))) {
+				if (folder) {
+					fname = nm_folder_get_name(folder);
+					if (*fname == '\0') {
+						fname = NM_ROOT_FOLDER_NAME;
+					}
+					group = gaim_find_group(fname);
+				}
+
+				if (group) {
 					buddy = gaim_find_buddy_in_group(user->client_data,
 													 name, group);
 					if (buddy && strcmp(buddy->alias, alias))
 						gaim_blist_alias_buddy(buddy, alias);
-
 				}
+
 				/* Tell the server to alias the contact */
 				rc = nm_send_rename_contact(user, contact, alias,
 											_rename_contact_resp_cb, NULL);
@@ -2601,11 +2662,23 @@ novell_group_buddy(GaimConnection * gc,
 	if (user && (dn = nm_lookup_dn(user, name))) {
 
 		/* Find the old folder */
-		old_folder = nm_find_folder(user, old_group_name);
+		if (strcmp(old_group_name, NM_ROOT_FOLDER_NAME) == 0) {
+			old_folder = nm_get_root_folder(user);
+			if (nm_folder_find_contact(old_folder, dn) == NULL)
+				old_folder = nm_find_folder(user, old_group_name);
+		} else {
+			old_folder = nm_find_folder(user, old_group_name);
+		}
+
 		if (old_folder && (contact = nm_folder_find_contact(old_folder, dn))) {
 
 			/* Find the new folder */
 			new_folder = nm_find_folder(user, new_group_name);
+			if (new_folder == NULL) {
+				if (strcmp(new_group_name, NM_ROOT_FOLDER_NAME) == 0)
+					new_folder = nm_get_root_folder(user);
+			}
+
 			if (new_folder) {
 
 				/* Tell the server to move the contact to the new folder */
@@ -2633,31 +2706,35 @@ novell_group_buddy(GaimConnection * gc,
 
 static void
 novell_rename_group(GaimConnection * gc, const char *old_name,
-					const char *new_name, GList * tobemoved)
+					GaimGroup *group, GList *moved_buddies)
 {
 	NMERR_T rc = NM_OK;
 	NMFolder *folder;
 	NMUser *user;
 
-	if (gc == NULL || old_name == NULL || new_name == NULL || tobemoved == NULL) {
+	if (gc == NULL || old_name == NULL || group == NULL || moved_buddies == NULL) {
 		return;
 	}
 
 	user = gc->proto_data;
 	if (user) {
 		/* Does new folder exist already? */
-		if (nm_find_folder(user, new_name)) {
-			/* Gaim currently calls novell_group_buddy() for
-			 * for all buddies in the group, so we don't
-			 * need to worry about this situation.
+		if (nm_find_folder(user, group->name)) {
+			/* gaim_blist_rename_group() adds the buddies
+			 * to the new group and removes the old group...
+			 * so there is nothing more to do here.
 			 */
 			return;
 		}
 
-		folder = nm_find_folder(user, old_name);
+		if (strcmp(old_name, NM_ROOT_FOLDER_NAME) == 0) {
+			/* Can't rename the root folder ... need to revisit this */
+			return;
+		}
 
+		folder = nm_find_folder(user, old_name);
 		if (folder) {
-			rc = nm_send_rename_folder(user, folder, new_name,
+			rc = nm_send_rename_folder(user, folder, group->name,
 									   _rename_folder_resp_cb, NULL);
 			_check_for_disconnect(user, rc);
 		}
@@ -3083,8 +3160,9 @@ novell_set_permit_deny(GaimConnection *gc)
 	if (user == NULL)
 		return;
 
-	if (set_permit == FALSE) {
-		set_permit = TRUE;
+	if (user->privacy_synched == FALSE) {
+		_sync_privacy_lists(user);
+		user->privacy_synched = TRUE;
 		return;
 	}
 
@@ -3285,7 +3363,7 @@ static GList *
 novell_blist_node_menu(GaimBlistNode *node)
 {
 	GList *list = NULL;
-	GaimBlistNodeAction *act;	
+	GaimBlistNodeAction *act;
 
 	if(GAIM_BLIST_NODE_IS_BUDDY(node)) {
 		act = gaim_blist_node_action_new(_("Initiate _Chat"),
@@ -3296,65 +3374,88 @@ novell_blist_node_menu(GaimBlistNode *node)
 	return list;
 }
 
+static void
+novell_keepalive(GaimConnection *gc)
+{
+	NMUser *user;
+	NMERR_T rc = NM_OK;
+
+	if (gc == NULL)
+		return;
+
+	user = gc->proto_data;
+	if (user == NULL)
+		return;
+
+	rc = nm_send_keepalive(user, NULL, NULL);
+	_check_for_disconnect(user, rc);
+}
+
 static GaimPluginProtocolInfo prpl_info = {
-	GAIM_PRPL_API_VERSION,
 	0,
-	NULL,
-	NULL,
-	novell_list_icon,
-	novell_list_emblems,
-	novell_status_text,
-	novell_tooltip_text,
-	novell_away_states,
-	novell_blist_node_menu,
+	NULL,						/* user_splits */
+	NULL,						/* protocol_options */
+	NO_BUDDY_ICONS,				/* icon_spec */
+	novell_list_icon,			/* list_icon */
+	novell_list_emblems,		/* list_emblems */
+	novell_status_text,			/* status_text */
+	novell_tooltip_text,		/* tooltip_text */
+	novell_away_states,			/* away_states */
+	novell_blist_node_menu,		/* blist_node_menu */
 	NULL,						/* chat_info */
-	novell_login,
-	novell_close,
-	novell_send_im,
+	NULL,						/* chat_info_defaults */
+	novell_login,				/* login */
+	novell_close,				/* close */
+	novell_send_im,				/* send_im */
 	NULL,						/* set_info */
-	novell_send_typing,
-	novell_get_info,
-	novell_set_away,
-	novell_set_idle,
-	NULL,						/* change pwd */
-	novell_add_buddy,
+	novell_send_typing,			/* send_typing */
+	novell_get_info,			/* get_info */
+	novell_set_away,			/* set_away */
+	novell_set_idle,			/* set_idle */
+	NULL,						/* change_passwd */
+	novell_add_buddy,			/* add_buddy */
 	NULL,						/* add_buddies */
-	novell_remove_buddy,
+	novell_remove_buddy,		/* remove_buddy */
 	NULL,						/* remove_buddies */
-	novell_add_permit,
-	novell_add_deny,
-	novell_rem_permit,
-	novell_rem_deny,
-	novell_set_permit_deny,
+	novell_add_permit,			/* add_permit */
+	novell_add_deny,			/* add_deny */
+	novell_rem_permit,			/* rem_permit */
+	novell_rem_deny,			/* rem_deny */
+	novell_set_permit_deny,		/* set_permit_deny */
 	NULL,						/* warn */
 	NULL,						/* join_chat */
-	NULL,						/* reject_chat ?? */
+	NULL,						/* reject_chat */
+	NULL,					/* get_chat_name */
 	novell_chat_invite,			/* chat_invite */
-	novell_chat_leave,
+	novell_chat_leave,			/* chat_leave */
 	NULL,						/* chat_whisper */
-	novell_chat_send,
-	NULL,						/* keepalive */
+	novell_chat_send,			/* chat_send */
+	novell_keepalive,			/* keepalive */
 	NULL,						/* register_user */
 	NULL,						/* get_cb_info */
-	NULL,						/* get_cb_away_msg */
-	novell_alias_buddy,
-	novell_group_buddy,
-	novell_rename_group,
+	NULL,						/* get_cb_away */
+	novell_alias_buddy,			/* alias_buddy */
+	novell_group_buddy,			/* group_buddy */
+	novell_rename_group,		/* rename_group */
 	NULL,						/* buddy_free */
-	novell_convo_closed,
-	NULL,						/* normalize */
+	novell_convo_closed,		/* convo_closed */
+	gaim_normalize_nocase,		/* normalize */
 	NULL,						/* set_buddy_icon */
-	novell_remove_group,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL
+	novell_remove_group,		/* remove_group */
+	NULL,						/* get_cb_real_name */
+	NULL,						/* set_chat_topic */
+	NULL,						/* find_blist_chat */
+	NULL,						/* roomlist_get_list */
+	NULL,						/* roomlist_cancel */
+	NULL,						/* roomlist_expand_category */
+	NULL,						/* can_receive_file */
+	NULL						/* send_file */
 };
 
 static GaimPluginInfo info = {
-	GAIM_PLUGIN_API_VERSION,		/**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_PROTOCOL,			/**< type           */
 	NULL,					/**< ui_requirement */
 	0,					/**< flags          */
