@@ -77,6 +77,7 @@
 #define AIM_CB_LOC_USERINFO 0x0006
 #define AIM_CB_LOC_WATCHERSUBREQ 0x0007
 #define AIM_CB_LOC_WATCHERNOT 0x0008
+#define AIM_CB_LOC_REQUESTINFOTIMEOUT 0xfffe
 #define AIM_CB_LOC_DEFAULT 0xffff
 
 /*
@@ -93,7 +94,7 @@
 #define AIM_CB_BUD_DEFAULT 0xffff
 
 /*
- * SNAC Family: Messeging Services.
+ * SNAC Family: Messaging Services.
  */ 
 #define AIM_CB_MSG_ERROR 0x0001
 #define AIM_CB_MSG_PARAMINFO 0x0005
@@ -209,7 +210,8 @@
 #define AIM_CB_SSI_ERROR 0x0001
 #define AIM_CB_SSI_REQRIGHTS 0x0002
 #define AIM_CB_SSI_RIGHTSINFO 0x0003
-#define AIM_CB_SSI_REQLIST 0x0005
+#define AIM_CB_SSI_REQDATA 0x0004
+#define AIM_CB_SSI_REQIFCHANGED 0x0005
 #define AIM_CB_SSI_LIST 0x0006
 #define AIM_CB_SSI_ACTIVATE 0x0007
 #define AIM_CB_SSI_ADD 0x0008
@@ -238,6 +240,8 @@
 #define AIM_CB_ATH_LOGINRESPONSE 0x0003
 #define AIM_CB_ATH_AUTHREQ 0x0006
 #define AIM_CB_ATH_AUTHRESPONSE 0x0007
+#define AIM_CB_ATH_SECURID_REQUEST 0x000a
+#define AIM_CB_ATH_SECURID_RESPONSE 0x000b
 
 /*
  * SNAC Family: Email
@@ -286,7 +290,7 @@
 /*
  * SNAC Family: Internal Messages
  *
- * This isn't truely a SNAC family either, but using
+ * This isn't truly a SNAC family either, but using
  * these, we can integrated non-SNAC services into
  * the SNAC-centered libfaim callback structure.
  *
