@@ -35,8 +35,8 @@ faim_internal void aim__shutdownmodules(aim_session_t *sess);
 faim_internal aim_module_t *aim__findmodulebygroup(aim_session_t *sess, fu16_t group);
 faim_internal aim_module_t *aim__findmodule(aim_session_t *sess, const char *name);
 
-faim_internal int buddylist_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int admin_modfirst(aim_session_t *sess, aim_module_t *mod);
+faim_internal int buddylist_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int bos_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int search_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int stats_modfirst(aim_session_t *sess, aim_module_t *mod);
@@ -46,15 +46,16 @@ faim_internal int misc_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int chatnav_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int chat_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int locate_modfirst(aim_session_t *sess, aim_module_t *mod);
-faim_internal int general_modfirst(aim_session_t *sess, aim_module_t *mod);
-faim_internal int ssi_modfirst(aim_session_t *sess, aim_module_t *mod);
+faim_internal int service_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int invite_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int translate_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int popups_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int adverts_modfirst(aim_session_t *sess, aim_module_t *mod);
+faim_internal int odir_modfirst(aim_session_t *sess, aim_module_t *mod);
+faim_internal int bart_modfirst(aim_session_t *sess, aim_module_t *mod);
+faim_internal int ssi_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int icq_modfirst(aim_session_t *sess, aim_module_t *mod);
 faim_internal int email_modfirst(aim_session_t *sess, aim_module_t *mod);
-faim_internal int newsearch_modfirst(aim_session_t *sess, aim_module_t *mod);
 
 faim_internal int aim_genericreq_n(aim_session_t *, aim_conn_t *conn, fu16_t family, fu16_t subtype);
 faim_internal int aim_genericreq_n_snacid(aim_session_t *, aim_conn_t *conn, fu16_t family, fu16_t subtype);
@@ -67,7 +68,7 @@ faim_internal int aim_genericreq_s(aim_session_t *, aim_conn_t *conn, fu16_t fam
 faim_internal int aim_bstream_init(aim_bstream_t *bs, fu8_t *data, int len);
 faim_internal int aim_bstream_empty(aim_bstream_t *bs);
 faim_internal int aim_bstream_curpos(aim_bstream_t *bs);
-faim_internal int aim_bstream_setpos(aim_bstream_t *bs, int off);
+faim_internal int aim_bstream_setpos(aim_bstream_t *bs, unsigned int off);
 faim_internal void aim_bstream_rewind(aim_bstream_t *bs);
 faim_internal int aim_bstream_advance(aim_bstream_t *bs, int n);
 faim_internal fu8_t aimbs_get8(aim_bstream_t *bs);
@@ -95,7 +96,7 @@ faim_internal aim_conn_t *aim_cloneconn(aim_session_t *sess, aim_conn_t *src);
 faim_internal int aim_rxdispatch_rendezvous(aim_session_t *sess, aim_frame_t *fr);
 
 /* rxhandlers.c */
-faim_internal aim_rxcallback_t aim_callhandler(aim_session_t *sess, aim_conn_t *conn, u_short family, u_short type);
+faim_internal aim_rxcallback_t aim_callhandler(aim_session_t *sess, aim_conn_t *conn, fu16_t family, fu16_t type);
 faim_internal int aim_callhandler_noparam(aim_session_t *sess, aim_conn_t *conn, fu16_t family, fu16_t type, aim_frame_t *ptr);
 faim_internal int aim_parse_unknown(aim_session_t *, aim_frame_t *, ...);
 faim_internal void aim_clonehandlers(aim_session_t *sess, aim_conn_t *dest, aim_conn_t *src);
@@ -109,12 +110,9 @@ faim_internal void aim_frame_destroy(aim_frame_t *);
 /* txqueue.c */
 faim_internal aim_frame_t *aim_tx_new(aim_session_t *sess, aim_conn_t *conn, fu8_t framing, fu16_t chan, int datalen);
 faim_internal int aim_tx_enqueue(aim_session_t *, aim_frame_t *);
-faim_internal flap_seqnum_t aim_get_next_txseqnum(aim_conn_t *);
 faim_internal int aim_tx_sendframe(aim_session_t *sess, aim_frame_t *cur);
 faim_internal void aim_tx_cleanqueue(aim_session_t *, aim_conn_t *);
 
-/* XXX - What is this?   faim_internal int aim_tx_printqueue(aim_session_t *); */
-
 /*
  * Generic SNAC structure.  Rarely if ever used.
  */
@@ -187,9 +185,6 @@ typedef struct aim_conn_inside_s {
 
 faim_internal void aim_conn_addgroup(aim_conn_t *conn, fu16_t group);
 
-faim_internal fu32_t aim_getcap(aim_session_t *sess, aim_bstream_t *bs, int len);
-faim_internal int aim_putcap(aim_bstream_t *bs, fu32_t caps);
-
 faim_internal int aim_cachecookie(aim_session_t *sess, aim_msgcookie_t *cookie);
 faim_internal aim_msgcookie_t *aim_uncachecookie(aim_session_t *sess, fu8_t *cookie, int type);
 faim_internal aim_msgcookie_t *aim_mkcookie(fu8_t *, int, void *);
@@ -198,13 +193,17 @@ faim_internal int aim_freecookie(aim_session_t *sess, aim_msgcookie_t *cookie);
 faim_internal int aim_msgcookie_gettype(int reqclass);
 faim_internal int aim_cookie_free(aim_session_t *sess, aim_msgcookie_t *cookie);
 
-faim_internal int aim_extractuserinfo(aim_session_t *sess, aim_bstream_t *bs, aim_userinfo_t *);
+/* 0x0002 - locate.c */
+faim_internal void aim_locate_requestuserinfo(aim_session_t *sess, const char *sn);
+faim_internal fu32_t aim_locate_getcaps(aim_session_t *sess, aim_bstream_t *bs, int len);
+faim_internal fu32_t aim_locate_getcaps_short(aim_session_t *sess, aim_bstream_t *bs, int len);
+faim_internal int aim_putcap(aim_bstream_t *bs, fu32_t caps);
+faim_internal void aim_info_free(aim_userinfo_t *);
+faim_internal int aim_info_extract(aim_session_t *sess, aim_bstream_t *bs, aim_userinfo_t *);
 faim_internal int aim_putuserinfo(aim_bstream_t *bs, aim_userinfo_t *info);
 
 faim_internal int aim_chat_readroominfo(aim_bstream_t *bs, struct aim_chat_roominfo *outinfo);
 
-faim_internal void faimdprintf(aim_session_t *sess, int dlevel, const char *format, ...);
-
 faim_internal void aim_conn_kill_chat(aim_session_t *sess, aim_conn_t *conn);
 
 /* These are all handled internally now. */
@@ -213,8 +212,12 @@ faim_internal int aim_reqrates(aim_session_t *, aim_conn_t *);
 faim_internal int aim_rates_addparam(aim_session_t *, aim_conn_t *);
 faim_internal int aim_rates_delparam(aim_session_t *, aim_conn_t *);
 
+faim_internal void faimdprintf(aim_session_t *sess, int dlevel, const char *format, ...);
+
 #ifndef FAIM_INTERNAL_INSANE
+#undef printf
 #define printf() printf called inside libfaim
+#undef sprintf
 #define sprintf() unbounded sprintf used inside libfaim
 #endif
 
