@@ -53,6 +53,7 @@
 #include <netinet/in.h>
 #include <sys/utsname.h> /* for aim_odc_initiate */
 #include <arpa/inet.h> /* for inet_ntoa */
+#include <limits.h> /* for UINT_MAX */
 #define G_DIR_SEPARATOR '/'
 #endif
 
@@ -60,6 +61,20 @@
 #include "win32dep.h"
 #endif
 
+/*
+ * I really want to switch all our networking code to using IPv6 only,
+ * but that really isn't a good idea at all.  Evan S. of Adium says
+ * OS X sets all connections as "AF_INET6/PF_INET6," even if there is
+ * nothing inherently IPv6 about them.  And I feel like Linux kernel
+ * 2.6.5 is doing the same thing.  So we REALLY should accept
+ * connections if they're showing up as IPv6.  Old OSes (Solaris?)
+ * that might not have full IPv6 support yet will fail if we try
+ * to use PF_INET6 but it isn't defined.  --Mark Doliner
+ */
+#ifndef PF_INET6
+#define PF_INET6 PF_INET
+#endif
+
 struct aim_odc_intdata {
 	fu8_t cookie[8];
 	char sn[MAXSNLEN+1];
@@ -107,7 +122,7 @@ static void aim_oft_dirconvert_fromstupid(char *name)
  * which is simpler and should be more accurate than Josh Myer's original 
  * code. -- wtm
  *
- * This algorithim works every time I have tried it.  The other fails 
+ * This algorithm works every time I have tried it.  The other fails 
  * sometimes.  So, AOL who thought this up?  It has got to be the weirdest 
  * checksum I have ever seen.
  *
@@ -156,85 +171,6 @@ faim_export fu32_t aim_oft_checksum_file(char *filename) {
 	return checksum;
 }
 
-/**
- * Create a listening socket on a given port.
- *
- * XXX - Give the client author the responsibility of setting up a 
- *       listener, then we no longer have a libfaim problem with broken 
- *       solaris *innocent smile* -- jbm
- *
- * @param portnum The port number to bind to.
- * @return The file descriptor of the listening socket.
- */
-static int listenestablish(fu16_t portnum)
-{
-#if HAVE_GETADDRINFO
-	int listenfd;
-	const int on = 1;
-	struct addrinfo hints, *res, *ressave;
-	char serv[5];
-
-	snprintf(serv, sizeof(serv), "%d", portnum);
-	memset(&hints, 0, sizeof(struct addrinfo));
-	hints.ai_flags = AI_PASSIVE;
-	hints.ai_family = AF_UNSPEC;
-	hints.ai_socktype = SOCK_STREAM;
-	if (getaddrinfo(NULL /* any IP */, serv, &hints, &res) != 0) {
-		perror("getaddrinfo");
-		return -1;
-	} 
-	ressave = res;
-	do { 
-		listenfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
-		if (listenfd < 0)
-			continue;
-		setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
-		if (bind(listenfd, res->ai_addr, res->ai_addrlen) == 0)
-			break; /* success */
-		close(listenfd);
-	} while ( (res = res->ai_next) );
-
-	if (!res)
-		return -1;
-
-	freeaddrinfo(ressave);
-#else
-	int listenfd;
-	const int on = 1;
-	struct sockaddr_in sockin;
-
-	if ((listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-		perror("socket");
-		return -1;
-	}
-
-	if (setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on)) != 0) {
-		perror("setsockopt");
-		close(listenfd);
-		return -1;
-	}
-
-	memset(&sockin, 0, sizeof(struct sockaddr_in));
-	sockin.sin_family = AF_INET;
-	sockin.sin_port = htons(portnum);
-
-	if (bind(listenfd, (struct sockaddr *)&sockin, sizeof(struct sockaddr_in)) != 0) {
-		perror("bind");
-		close(listenfd);
-		return -1;
-	}
-#endif
-
-	if (listen(listenfd, 4) != 0) {
-		perror("listen");
-		close(listenfd);
-		return -1;
-	}
-	fcntl(listenfd, F_SETFL, O_NONBLOCK);
-
-	return listenfd;
-}
-
 /**
  * After establishing a listening socket, this is called to accept a connection.  It
  * clones the conn used by the listener, and passes both of these to a signal handler.
@@ -253,12 +189,12 @@ faim_export int aim_handlerendconnect(aim_session_t *sess, aim_conn_t *cur)
 	int ret = 0;
 	aim_conn_t *newconn;
 	char ip[20];
-	int port;
+	unsigned short port;
 
 	if ((acceptfd = accept(cur->fd, &addr, &addrlen)) == -1)
 		return 0; /* not an error */
 
-	if (addr.sa_family != AF_INET) { /* just in case IPv6 really is happening */
+	if ((addr.sa_family != PF_INET) && (addr.sa_family != PF_INET6)) {
 		close(acceptfd);
 		aim_conn_close(cur);
 		return -1;
@@ -282,7 +218,7 @@ faim_export int aim_handlerendconnect(aim_session_t *sess, aim_conn_t *cur)
 
 		priv = (struct aim_odc_intdata *)(newconn->internal = cur->internal);
 		cur->internal = NULL;
-		snprintf(priv->ip, sizeof(priv->ip), "%s:%u", ip, port);
+		snprintf(priv->ip, sizeof(priv->ip), "%s:%hu", ip, port);
 
 		if ((userfunc = aim_callhandler(sess, newconn, AIM_CB_FAM_OFT, AIM_CB_OFT_DIRECTIM_ESTABLISHED)))
 			ret = userfunc(sess, NULL, newconn, cur);
@@ -326,7 +262,7 @@ faim_export int aim_odc_send_typing(aim_session_t *sess, aim_conn_t *conn, int t
 	if (!(fr = aim_tx_new(sess, conn, AIM_FRAMETYPE_OFT, 0x0001, 0)))
 		return -ENOMEM;
 	memcpy(fr->hdr.rend.magic, "ODC2", 4);
-	fr->hdr.rend.hdrlen = hdrlen;
+	fr->hdr.rend.hdrlen = hdrlen + 8;
 
 	if (!(hdr = calloc(1, hdrlen))) {
 		aim_frame_destroy(fr);
@@ -386,7 +322,7 @@ faim_export int aim_odc_send_typing(aim_session_t *sess, aim_conn_t *conn, int t
  * @param conn The already-connected ODC connection.
  * @param msg Null-terminated string to send.
  * @param len The length of the message to send, including binary data.
- * @param encoding 0 for ascii, 2 for Unicode, 3 for ISO 8859-1.
+ * @param encoding See the AIM_CHARSET_* defines in aim.h
  * @param isawaymsg 0 if this is not an auto-response, 1 if it is.
  * @return Return 0 if no errors, otherwise return the error number.
  */
@@ -405,7 +341,7 @@ faim_export int aim_odc_send_im(aim_session_t *sess, aim_conn_t *conn, const cha
 		return -ENOMEM;
 
 	memcpy(fr->hdr.rend.magic, "ODC2", 4);
-	fr->hdr.rend.hdrlen = hdrlen;
+	fr->hdr.rend.hdrlen = hdrlen + 8;
 
 	if (!(hdr = calloc(1, hdrlen + len))) {
 		aim_frame_destroy(fr);
@@ -467,7 +403,7 @@ faim_export int aim_odc_send_im(aim_session_t *sess, aim_conn_t *conn, const cha
 
 /**
  * Get the screen name of the peer of a direct connection.
- * 
+ *
  * @param conn The ODC connection.
  * @return The screen name of the dude, or NULL if there was an anomaly.
  */
@@ -478,7 +414,7 @@ faim_export const char *aim_odc_getsn(aim_conn_t *conn)
 	if (!conn || !conn->internal)
 		return NULL;
 
-	if ((conn->type != AIM_CONN_TYPE_RENDEZVOUS) || 
+	if ((conn->type != AIM_CONN_TYPE_RENDEZVOUS) ||
 			(conn->subtype != AIM_CONN_SUBTYPE_OFT_DIRECTIM))
 		return NULL;
 
@@ -487,6 +423,24 @@ faim_export const char *aim_odc_getsn(aim_conn_t *conn)
 	return intdata->sn;
 }
 
+/**
+ * Get the cookie of a direct connection.
+ *
+ * @param conn The ODC connection.
+ * @return The cookie, an 8 byte unterminated string, or NULL if there was an anomaly.
+ */
+faim_export const char *aim_odc_getcookie(aim_conn_t *conn)
+{
+	struct aim_odc_intdata *intdata;
+
+	if (!conn || !conn->internal)
+		return NULL;
+
+	intdata = (struct aim_odc_intdata *)conn->internal;
+
+	return intdata->cookie;
+}
+
 /**
  * Find the conn of a direct connection with the given buddy.
  *
@@ -517,32 +471,31 @@ faim_export aim_conn_t *aim_odc_getconn(aim_session_t *sess, const char *sn)
 /**
  * For those times when we want to open up the direct connection channel ourselves.
  *
- * You'll want to set up some kind of watcher on this socket.  
- * When the state changes, call aim_handlerendconnection with 
- * the connection returned by this.  aim_handlerendconnection 
+ * You'll want to set up some kind of watcher on this socket.
+ * When the state changes, call aim_handlerendconnection with
+ * the connection returned by this.  aim_handlerendconnection
  * will accept the pending connection and stop listening.
  *
  * @param sess The session
  * @param sn The screen name to connect to.
  * @return The new connection.
  */
-faim_export aim_conn_t *aim_odc_initiate(aim_session_t *sess, const char *sn)
+faim_export aim_conn_t *aim_odc_initiate(aim_session_t *sess, const char *sn, int listenfd,
+                                         const fu8_t *localip, fu16_t port, const fu8_t *mycookie)
 {
 	aim_conn_t *newconn;
 	aim_msgcookie_t *cookie;
 	struct aim_odc_intdata *priv;
-	int listenfd;
-	fu16_t port = 4443;
-	fu8_t localip[4];
 	fu8_t ck[8];
 
-	if (aim_util_getlocalip(localip) == -1)
+	if (!localip)
 		return NULL;
 
-	if ((listenfd = listenestablish(port)) == -1)
-		return NULL;
-
-	aim_im_sendch2_odcrequest(sess, ck, sn, localip, port);
+	if (mycookie) {
+		memcpy(ck, mycookie, 8);
+		aim_im_sendch2_odcrequest(sess, ck, TRUE, sn, localip, port);
+	} else
+		aim_im_sendch2_odcrequest(sess, ck, FALSE, sn, localip, port);
 
 	cookie = (aim_msgcookie_t *)calloc(1, sizeof(aim_msgcookie_t));
 	memcpy(cookie->cookie, ck, 8);
@@ -663,7 +616,7 @@ static int handlehdr_odc(aim_session_t *sess, aim_conn_t *conn, aim_frame_t *frr
 			ret = userfunc(sess, &fr, snptr, 0);
 	}
 
-	if (payloadlength) {
+	if ((payloadlength != 0) && (payloadlength != UINT_MAX)) {
 		char *msg;
 		int recvd = 0;
 		int i, isawaymsg;
@@ -719,6 +672,7 @@ faim_export struct aim_oft_info *aim_oft_createinfo(aim_session_t *sess, const f
 	if (sn)
 		new->sn = strdup(sn);
 	new->port = port;
+	new->success = FALSE;
 	new->fh.totfiles = 1;
 	new->fh.filesleft = 1;
 	new->fh.totparts = 1;
@@ -731,8 +685,10 @@ faim_export struct aim_oft_info *aim_oft_createinfo(aim_session_t *sess, const f
 	new->fh.rfcsum = 0xffff0000;
 	new->fh.recvcsum = 0xffff0000;
 	strncpy(new->fh.idstring, "OFT_Windows ICBMFT V1.1 32", 31);
-	if (filename)
+	if (filename) {
 		strncpy(new->fh.name, filename, 63);
+		new->fh.name[63] = '\0';
+	}
 
 	new->next = sess->oft_info;
 	sess->oft_info = new;
@@ -786,16 +742,11 @@ faim_export int aim_oft_destroyinfo(struct aim_oft_info *oft_info)
  *        connection.
  * @return Return 0 if no errors, otherwise return the error number.
  */
-faim_export int aim_sendfile_listen(aim_session_t *sess, struct aim_oft_info *oft_info)
+faim_export int aim_sendfile_listen(aim_session_t *sess, struct aim_oft_info *oft_info, int listenfd)
 {
-	int listenfd;
-
 	if (!oft_info)
 		return -EINVAL;
 
-	if ((listenfd = listenestablish(oft_info->port)) == -1)
-		return 1;
-
 	if (!(oft_info->conn = aim_newconn(sess, AIM_CONN_TYPE_LISTENER, NULL))) {
 		close(listenfd);
 		return -ENOMEM;
@@ -848,6 +799,7 @@ static struct aim_fileheader_t *aim_oft_getheader(aim_bstream_t *bs)
 	fh->nencode = aimbs_get16(bs);
 	fh->nlanguage = aimbs_get16(bs);
 	aimbs_getrawbuf(bs, fh->name, 64); /* XXX - filenames longer than 64B */
+	fh->name[63] = '\0';
 
 	return fh;
 } 
@@ -925,9 +877,7 @@ faim_export int aim_oft_sendheader(aim_session_t *sess, fu16_t type, struct aim_
 	fh->lnameoffset = 0x1a;
 	fh->lsizeoffset = 0x10;
 
-	/* apparently 0 is ASCII, 2 is UCS-2 */
-	/* it is likely that 3 is ISO 8859-1 */
-	/* I think "nlanguage" might be the same thing as "subenc" in im.c */
+	/* These should be the same as charset and charsubset in ICBMs */
 	fh->nencode = 0x0000;
 	fh->nlanguage = 0x0000;
 #endif
@@ -943,7 +893,7 @@ faim_export int aim_oft_sendheader(aim_session_t *sess, fu16_t type, struct aim_
 	}
 
 	memcpy(fr->hdr.rend.magic, "OFT2", 4);
-	fr->hdr.rend.hdrlen = aim_bstream_curpos(&fr->data);
+	fr->hdr.rend.hdrlen = aim_bstream_curpos(&fr->data) + 8;
 
 	aim_tx_enqueue(sess, fr);
 
