@@ -2,7 +2,7 @@
  * Family 0x0004 - Routines for sending/receiving Instant Messages.
  *
  * Note the term ICBM (Inter-Client Basic Message) which blankets
- * all types of genericly routed through-server messages.  Within
+ * all types of generically routed through-server messages.  Within
  * the ICBM types (family 4), a channel is defined.  Each channel
  * represents a different type of message.  Channel 1 is used for
  * what would commonly be called an "instant message".  Channel 2
@@ -62,21 +62,20 @@ static int aim_im_puticbm(aim_bstream_t *bs, const fu8_t *c, fu16_t ch, const ch
  *
  *
  * Heres the current collection:
- *  0501 0003 0101 0101 01	AOL Mobile Communicator, WinAIM 1.0.414
- *  0501 0003 0101 0201 01	WinAIM 2.0.847, 2.1.1187, 3.0.1464, 
+ *  0501 0003 0101 0101 01		AOL Mobile Communicator, WinAIM 1.0.414
+ *  0501 0003 0101 0201 01		WinAIM 2.0.847, 2.1.1187, 3.0.1464, 
  *					4.3.2229, 4.4.2286
- *  0501 0004 0101 0102 0101	WinAIM 4.1.2010, libfaim (right here)
- *  0501 0003 0101 02		WinAIM 5
- *  0501 0001 01		iChat x.x
- *  0501 0001 0101 01		AOL v6.0, CompuServe 2000 v6.0, any
- *				TOC client
- *  0501 0002 0106		WinICQ 5.45.1.3777.85
+ *  0501 0004 0101 0102 0101		WinAIM 4.1.2010, libfaim (right here)
+ *  0501 0003 0101 02			WinAIM 5
+ *  0501 0001 01			iChat x.x, mobile buddies
+ *  0501 0001 0101 01			AOL v6.0, CompuServe 2000 v6.0, any TOC client
+ *  0501 0002 0106			WinICQ 5.45.1.3777.85
  *
  * Note that in this function, only the feature bytes are tested, since
  * the rest will always be the same.
  *
  */
-faim_export fu16_t aim_im_fingerprint(fu8_t *msghdr, int len)
+faim_export fu16_t aim_im_fingerprint(const fu8_t *msghdr, int len)
 {
 	static const struct {
 		fu16_t clientid;
@@ -119,7 +118,7 @@ faim_export fu16_t aim_im_fingerprint(fu8_t *msghdr, int len)
 /** 
  * Subtype 0x0002 - Set ICBM parameters.
  *
- * I definitly recommend sending this.  If you don't, you'll be stuck
+ * I definitely recommend sending this.  If you don't, you'll be stuck
  * with the rather unreasonable defaults.
  * 
  */
@@ -167,7 +166,7 @@ faim_export int aim_im_reqparams(aim_session_t *sess)
 	if (!sess || !(conn = aim_conn_findbygroup(sess, 0x0004)))
 		return -EINVAL;
 
-	return aim_genericreq_n(sess, conn, 0x0004, 0x0004);
+	return aim_genericreq_n_snacid(sess, conn, 0x0004, 0x0004);
 }
 
 /**
@@ -202,12 +201,6 @@ static int aim_im_paraminfo(aim_session_t *sess, aim_module_t *mod, aim_frame_t
  *                        when the message is received (of type 0x0004/0x000c)
  *   AIM_IMFLAGS_OFFLINE--If destination is offline, store it until they are
  *                        online (probably ICQ only).
- *   AIM_IMFLAGS_UNICODE--Instead of ASCII7, the passed message is
- *                        made up of UNICODE duples.  If you set
- *                        this, you'd better be damn sure you know
- *                        what you're doing.
- *   AIM_IMFLAGS_ISO_8859_1 -- The message contains the ASCII8 subset
- *                        known as ISO-8859-1.  
  *
  * Generally, you should use the lowest encoding possible to send
  * your message.  If you only use basic punctuation and the generic
@@ -218,7 +211,7 @@ static int aim_im_paraminfo(aim_session_t *sess, aim_module_t *mod, aim_frame_t
  * notably when the (r) symbol is used), you must use the full UNICODE
  * encoding for your message.  In UNICODE mode, _all_ characters must
  * occupy 16bits, including ones that are not special.  (Remember that
- * the first 128 UNICODE symbols are equivelent to ASCII7, however they
+ * the first 128 UNICODE symbols are equivalent to ASCII7, however they
  * must be prefixed with a zero high order byte.)
  *
  * I strongly discourage the use of UNICODE mode, mainly because none
@@ -255,7 +248,7 @@ faim_export int aim_im_sendch1_ext(aim_session_t *sess, struct aim_sendimext_arg
 		return -EINVAL;
 
 	if (args->flags & AIM_IMFLAGS_MULTIPART) {
-		if (args->mpmsg->numparts <= 0)
+		if (args->mpmsg->numparts == 0)
 			return -EINVAL;
 	} else {
 		if (!args->msg || (args->msglen <= 0))
@@ -381,7 +374,7 @@ faim_export int aim_im_sendch1_ext(aim_session_t *sess, struct aim_sendimext_arg
 
 	/*
 	 * Set the Buddy Icon Requested flag.
-	 * XXX - Everytime?  Surely not...
+	 * XXX - Every time?  Surely not...
 	 */
 	if (args->flags & AIM_IMFLAGS_BUDDYREQ) {
 		aimbs_put16(&fr->data, 0x0009);
@@ -390,9 +383,8 @@ faim_export int aim_im_sendch1_ext(aim_session_t *sess, struct aim_sendimext_arg
 
 	aim_tx_enqueue(sess, fr);
 
-	/* Move this to receive aim_flap_nop and send aim_flap_nop */
-	if (!(sess->flags & AIM_SESS_FLAGS_DONTTIMEOUTONICBM))
-		aim_cleansnacs(sess, 60); /* clean out SNACs over 60sec old */
+	/* clean out SNACs over 60sec old */
+	aim_cleansnacs(sess, 60);
 
 	return 0;
 }
@@ -424,6 +416,94 @@ faim_export int aim_im_sendch1(aim_session_t *sess, const char *sn, fu16_t flags
 	return aim_im_sendch1_ext(sess, &args);
 }
 
+/*
+ * Subtype 0x0006 - Send a chat invitation.
+ */
+faim_export int aim_im_sendch2_chatinvite(aim_session_t *sess, const char *sn, const char *msg, fu16_t exchange, const char *roomname, fu16_t instance)
+{
+	aim_conn_t *conn;
+	aim_frame_t *fr;
+	aim_snacid_t snacid;
+	int i;
+	aim_msgcookie_t *cookie;
+	struct aim_invite_priv *priv;
+	fu8_t ck[8];
+	aim_tlvlist_t *otl = NULL, *itl = NULL;
+	fu8_t *hdr;
+	int hdrlen;
+	aim_bstream_t hdrbs;
+	
+	if (!sess || !(conn = aim_conn_findbygroup(sess, 0x0004)))
+		return -EINVAL;
+
+	if (!sn || !msg || !roomname)
+		return -EINVAL;
+
+	for (i = 0; i < 8; i++)
+		ck[i] = (fu8_t)rand();
+
+	if (!(fr = aim_tx_new(sess, conn, AIM_FRAMETYPE_FLAP, 0x02, 1152+strlen(sn)+strlen(roomname)+strlen(msg))))
+		return -ENOMEM;
+
+	snacid = aim_cachesnac(sess, 0x0004, 0x0006, 0x0000, sn, strlen(sn)+1);
+	aim_putsnac(&fr->data, 0x0004, 0x0006, 0x0000, snacid);
+
+	/* XXX should be uncached by an unwritten 'invite accept' handler */
+	if ((priv = malloc(sizeof(struct aim_invite_priv)))) {
+		priv->sn = strdup(sn);
+		priv->roomname = strdup(roomname);
+		priv->exchange = exchange;
+		priv->instance = instance;
+	}
+
+	if ((cookie = aim_mkcookie(ck, AIM_COOKIETYPE_INVITE, priv)))
+		aim_cachecookie(sess, cookie);
+	else
+		free(priv);
+
+	/* ICBM Header */
+	aimbs_putraw(&fr->data, ck, 8); /* Cookie */
+	aimbs_put16(&fr->data, 0x0002); /* Channel */
+	aimbs_put8(&fr->data, strlen(sn)); /* Screename length */
+	aimbs_putraw(&fr->data, sn, strlen(sn)); /* Screenname */
+
+	/*
+	 * TLV t(0005)
+	 *
+	 * Everything else is inside this TLV.
+	 *
+	 * Sigh.  AOL was rather inconsistent right here.  So we have
+	 * to play some minor tricks.  Right inside the type 5 is some
+	 * raw data, followed by a series of TLVs.  
+	 *
+	 */
+	hdrlen = 2+8+16+6+4+4+strlen(msg)+4+2+1+strlen(roomname)+2;
+	hdr = malloc(hdrlen);
+	aim_bstream_init(&hdrbs, hdr, hdrlen);
+
+	aimbs_put16(&hdrbs, 0x0000); /* Unknown! */
+	aimbs_putraw(&hdrbs, ck, sizeof(ck)); /* I think... */
+	aim_putcap(&hdrbs, AIM_CAPS_CHAT);
+
+	aim_tlvlist_add_16(&itl, 0x000a, 0x0001);
+	aim_tlvlist_add_noval(&itl, 0x000f);
+	aim_tlvlist_add_raw(&itl, 0x000c, strlen(msg), msg);
+	aim_tlvlist_add_chatroom(&itl, 0x2711, exchange, roomname, instance);
+	aim_tlvlist_write(&hdrbs, &itl);
+	
+	aim_tlvlist_add_raw(&otl, 0x0005, aim_bstream_curpos(&hdrbs), hdr);
+
+	aim_tlvlist_write(&fr->data, &otl);
+
+	free(hdr);
+	aim_tlvlist_free(&itl);
+	aim_tlvlist_free(&otl);
+	
+	aim_tx_enqueue(sess, fr);
+
+	return 0;
+}
+
 /**
  * Subtype 0x0006 - Send your icon to a given user.
  *
@@ -506,7 +586,7 @@ faim_export int aim_im_sendch2_icon(aim_session_t *sess, const char *sn, const f
  *
  * You'd think this would be in icq.c, but, well, I'm trying to stick with
  * the one-group-per-file scheme as much as possible.  This could easily
- * be an exception, since Rendezvous IMs are external of the Oscar core, 
+ * be an exception, since Rendezvous IMs are external of the Oscar core,
  * and therefore are undefined.  Really I just need to think of a good way to
  * make an interface similar to what AOL actually uses.  But I'm not using COM.
  *
@@ -594,7 +674,7 @@ faim_export int aim_im_sendch2_rtfmsg(aim_session_t *sess, struct aim_sendrtfmsg
  * Subtype 0x0006 - Send an "I want to directly connect to you" message
  *
  */
-faim_export int aim_im_sendch2_odcrequest(aim_session_t *sess, fu8_t *cookie, const char *sn, const fu8_t *ip, fu16_t port)
+faim_export int aim_im_sendch2_odcrequest(aim_session_t *sess, fu8_t *cookie, int usecookie, const char *sn, const fu8_t *ip, fu16_t port)
 {
 	aim_conn_t *conn;
 	aim_frame_t *fr;
@@ -614,26 +694,30 @@ faim_export int aim_im_sendch2_odcrequest(aim_session_t *sess, fu8_t *cookie, co
 	snacid = aim_cachesnac(sess, 0x0004, 0x0006, 0x0000, NULL, 0);
 	aim_putsnac(&fr->data, 0x0004, 0x0006, 0x0000, snacid);
 
-	/* 
-	 * Generate a random message cookie 
+	/*
+	 * Generate a random message cookie
 	 *
-	 * This cookie needs to be alphanumeric and NULL-terminated to be 
+	 * This cookie needs to be alphanumeric and NULL-terminated to be
 	 * TOC-compatible.
 	 *
 	 * XXX - have I mentioned these should be generated in msgcookie.c?
 	 *
 	 */
-	for (i = 0; i < 7; i++)
-	       	ck[i] = 0x30 + ((fu8_t) rand() % 10);
+
+	if (cookie && usecookie) /* allow user-specified cookie */
+		memcpy(ck, cookie, 8);
+	else
+		for (i = 0; i < 7; i++)
+			ck[i] = 0x30 + ((fu8_t) rand() % 10);
 	ck[7] = '\0';
 
-	if (cookie)
+	if (cookie && !usecookie)
 		memcpy(cookie, ck, 8);
 
 	/* ICBM header */
 	aim_im_puticbm(&fr->data, ck, 0x0002, sn);
 
-	aim_addtlvtochain_noval(&tl, 0x0003);
+	aim_tlvlist_add_noval(&tl, 0x0003);
 
 	hdrlen = 2+8+16+6+8+6+4;
 	hdr = malloc(hdrlen);
@@ -641,22 +725,22 @@ faim_export int aim_im_sendch2_odcrequest(aim_session_t *sess, fu8_t *cookie, co
 
 	aimbs_put16(&hdrbs, 0x0000);
 	aimbs_putraw(&hdrbs, ck, 8);
-	aim_putcap(&hdrbs, AIM_CAPS_IMIMAGE);
+	aim_putcap(&hdrbs, AIM_CAPS_DIRECTIM);
 
-	aim_addtlvtochain16(&itl, 0x000a, 0x0001);
-	aim_addtlvtochain_raw(&itl, 0x0003, 4, ip);
-	aim_addtlvtochain16(&itl, 0x0005, port);
-	aim_addtlvtochain_noval(&itl, 0x000f);
+	aim_tlvlist_add_16(&itl, 0x000a, 0x0001);
+	aim_tlvlist_add_raw(&itl, 0x0003, 4, ip);
+	aim_tlvlist_add_16(&itl, 0x0005, port);
+	aim_tlvlist_add_noval(&itl, 0x000f);
 	
-	aim_writetlvchain(&hdrbs, &itl);
+	aim_tlvlist_write(&hdrbs, &itl);
 
-	aim_addtlvtochain_raw(&tl, 0x0005, aim_bstream_curpos(&hdrbs), hdr);
+	aim_tlvlist_add_raw(&tl, 0x0005, aim_bstream_curpos(&hdrbs), hdr);
 
-	aim_writetlvchain(&fr->data, &tl);
+	aim_tlvlist_write(&fr->data, &tl);
 
 	free(hdr);
-	aim_freetlvchain(&itl);
-	aim_freetlvchain(&tl);
+	aim_tlvlist_free(&itl);
+	aim_tlvlist_free(&tl);
 
 	aim_tx_enqueue(sess, fr);
 
@@ -688,11 +772,12 @@ faim_export int aim_im_sendch2_sendfile_ask(aim_session_t *sess, struct aim_oft_
 		int buflen;
 		aim_bstream_t bs;
 
-		aim_addtlvtochain16(&subtl, 0x000a, 0x0001);
-		aim_addtlvtochain_noval(&subtl, 0x000f);
-/*		aim_addtlvtochain_raw(&subtl, 0x000e, 2, "en");
-		aim_addtlvtochain_raw(&subtl, 0x000d, 8, "us-ascii");
-		aim_addtlvtochain_raw(&subtl, 0x000c, 24, "Please accept this file."); */
+		aim_tlvlist_add_16(&subtl, 0x000a, 0x0001);
+		aim_tlvlist_add_noval(&subtl, 0x000f);
+/*		aim_tlvlist_add_raw(&subtl, 0x000e, 2, "en");
+		aim_tlvlist_add_raw(&subtl, 0x000d, 8, "us-ascii");
+		aim_tlvlist_add_raw(&subtl, 0x000c, 24, "Please accept this file."); */
+		/* XXX - Change oft_info->clientip to an array of 4 bytes */
 		if (oft_info->clientip) {
 			fu8_t ip[4];
 			char *nexttoken;
@@ -703,9 +788,9 @@ faim_export int aim_im_sendch2_sendfile_ask(aim_session_t *sess, struct aim_oft_
 				nexttoken = strtok(NULL, ".");
 				i++;
 			}
-			aim_addtlvtochain_raw(&subtl, 0x0003, 4, ip);
+			aim_tlvlist_add_raw(&subtl, 0x0003, 4, ip);
 		}
-		aim_addtlvtochain16(&subtl, 0x0005, oft_info->port);
+		aim_tlvlist_add_16(&subtl, 0x0005, oft_info->port);
 
 		/* TLV t(2711) */
 		buflen = 2+2+4+strlen(oft_info->fh.name)+1;
@@ -719,7 +804,7 @@ faim_export int aim_im_sendch2_sendfile_ask(aim_session_t *sess, struct aim_oft_
 		aimbs_putraw(&bs, oft_info->fh.name, strlen(oft_info->fh.name));
 		aimbs_put8(&bs, 0x00);
 
-		aim_addtlvtochain_raw(&subtl, 0x2711, bs.len, bs.data);
+		aim_tlvlist_add_raw(&subtl, 0x2711, bs.len, bs.data);
 		free(buf);
 	}
 
@@ -729,22 +814,22 @@ faim_export int aim_im_sendch2_sendfile_ask(aim_session_t *sess, struct aim_oft_
 		aim_bstream_t bs;
 
 		/* TLV t(0005) - Encompasses everything from above. Gee. */
-		buflen = 2+8+16+aim_sizetlvchain(&subtl);
+		buflen = 2+8+16+aim_tlvlist_size(&subtl);
 		buf = malloc(buflen);
 		aim_bstream_init(&bs, buf, buflen);
 		aimbs_put16(&bs, AIM_RENDEZVOUS_PROPOSE);
 		aimbs_putraw(&bs, oft_info->cookie, 8);
 		aim_putcap(&bs, AIM_CAPS_SENDFILE);
-		aim_writetlvchain(&bs, &subtl);
-		aim_freetlvchain(&subtl);
-		aim_addtlvtochain_raw(&tl, 0x0005, bs.len, bs.data);
+		aim_tlvlist_write(&bs, &subtl);
+		aim_tlvlist_free(&subtl);
+		aim_tlvlist_add_raw(&tl, 0x0005, bs.len, bs.data);
 		free(buf);
 
 		/* TLV t(0003) - Request an ack */
-		aim_addtlvtochain_noval(&tl, 0x0003);
+		aim_tlvlist_add_noval(&tl, 0x0003);
 	}
 
-	if (!(fr = aim_tx_new(sess, conn, AIM_FRAMETYPE_FLAP, 0x02, 10 + 11+strlen(oft_info->sn) + aim_sizetlvchain(&tl))))
+	if (!(fr = aim_tx_new(sess, conn, AIM_FRAMETYPE_FLAP, 0x02, 10 + 11+strlen(oft_info->sn) + aim_tlvlist_size(&tl))))
 		return -ENOMEM;
 
 	snacid = aim_cachesnac(sess, 0x0004, 0x0006, AIM_SNACFLAGS_DESTRUCTOR, oft_info->cookie, sizeof(oft_info->cookie));
@@ -754,8 +839,8 @@ faim_export int aim_im_sendch2_sendfile_ask(aim_session_t *sess, struct aim_oft_
 	aim_im_puticbm(&fr->data, oft_info->cookie, 0x0002, oft_info->sn);
 
 	/* All that crap from above (the 0x0005 TLV and the 0x0003 TLV) */
-	aim_writetlvchain(&fr->data, &tl);
-	aim_freetlvchain(&tl);
+	aim_tlvlist_write(&fr->data, &tl);
+	aim_tlvlist_free(&tl);
 
 	aim_tx_enqueue(sess, fr);
 
@@ -1033,14 +1118,14 @@ static int outgoingim(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, a
 	snlen = aimbs_get8(bs);
 	sn = aimbs_getstr(bs, snlen);
 
-	tlvlist = aim_readtlvchain(bs);
+	tlvlist = aim_tlvlist_read(bs);
 
-	if (aim_gettlv(tlvlist, 0x0003, 1))
+	if (aim_tlv_gettlv(tlvlist, 0x0003, 1))
 		icbmflags |= AIM_IMFLAGS_ACK;
-	if (aim_gettlv(tlvlist, 0x0004, 1))
+	if (aim_tlv_gettlv(tlvlist, 0x0004, 1))
 		icbmflags |= AIM_IMFLAGS_AWAY;
 
-	if ((msgblock = aim_gettlv(tlvlist, 0x0002, 1))) {
+	if ((msgblock = aim_tlv_gettlv(tlvlist, 0x0002, 1))) {
 		aim_bstream_t mbs;
 		int featurelen, msglen;
 
@@ -1065,7 +1150,7 @@ static int outgoingim(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, a
 		ret = userfunc(sess, rx, channel, sn, msg, icbmflags, flag1, flag2);
 
 	free(sn);
-	aim_freetlvchain(&tlvlist);
+	aim_tlvlist_free(&tlvlist);
 
 	return ret;
 }
@@ -1103,7 +1188,7 @@ static int outgoingim(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, a
  * open source clients (like encryption or something) -- see faimtest for
  * examples of how to do this.
  *
- * I would definitly recommend avoiding this feature unless you really
+ * I would definitely recommend avoiding this feature unless you really
  * know what you are doing, and/or you have something neat to do with it.
  *
  */
@@ -1223,12 +1308,13 @@ faim_export void aim_mpmsg_free(aim_session_t *sess, aim_mpmsg_t *mpm)
  * suspicious.
  *
  */
-static int incomingim_ch1_parsemsgs(aim_session_t *sess, fu8_t *data, int len, struct aim_incomingim_ch1_args *args)
+static int incomingim_ch1_parsemsgs(aim_session_t *sess, aim_userinfo_t *userinfo, fu8_t *data, int len, struct aim_incomingim_ch1_args *args)
 {
+	/* Should this be ASCII -> UNICODE -> Custom */
 	static const fu16_t charsetpri[] = {
-		0x0000, /* ASCII first */
-		0x0003, /* then ISO-8859-1 */
-		0x0002, /* UNICODE as last resort */
+		AIM_CHARSET_ASCII, /* ASCII first */
+		AIM_CHARSET_CUSTOM, /* then ISO-8859-1 */
+		AIM_CHARSET_UNICODE, /* UNICODE as last resort */
 	};
 	static const int charsetpricount = 3;
 	int i;
@@ -1246,6 +1332,11 @@ static int incomingim_ch1_parsemsgs(aim_session_t *sess, fu8_t *data, int len, s
 
 		/* Message string length, including character set info. */
 		msglen = aimbs_get16(&mbs);
+		if (msglen > aim_bstream_empty(&mbs))
+		{
+			faimdprintf(sess, 0, "Received an IM containing an invalid message part from %s.  They are probably trying to do something malicious.", userinfo->sn);
+			break;
+		}
 
 		/* Character set info */
 		flag1 = aimbs_get16(&mbs);
@@ -1297,15 +1388,6 @@ static int incomingim_ch1_parsemsgs(aim_session_t *sess, fu8_t *data, int len, s
 			args->charsubset = sec->charsubset;
 
 			/* Set up the simple flags */
-			if (args->charset == 0x0000)
-				; /* ASCII */
-			else if (args->charset == 0x0002)
-				args->icbmflags |= AIM_IMFLAGS_UNICODE;
-			else if (args->charset == 0x0003)
-				args->icbmflags |= AIM_IMFLAGS_ISO_8859_1;
-			else if (args->charset == 0xffff)
-				; /* no encoding (yeep!) */
-
 			if (args->charsubset == 0x0000)
 				; /* standard subencoding? */
 			else if (args->charsubset == 0x000b)
@@ -1334,7 +1416,7 @@ static int incomingim_ch1(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	aim_rxcallback_t userfunc;
 	int ret = 0;
 	struct aim_incomingim_ch1_args args;
-	int endpos;
+	unsigned int endpos;
 
 	memset(&args, 0, sizeof(args));
 
@@ -1345,11 +1427,17 @@ static int incomingim_ch1(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	 * I've changed it to process the TLVs in-place.  This avoids lots
 	 * of per-IM memory allocations.
 	 */
-	while (aim_bstream_empty(bs)) {
-
+	while (aim_bstream_empty(bs))
+	{
 		type = aimbs_get16(bs);
 		length = aimbs_get16(bs);
 
+		if (length > aim_bstream_empty(bs))
+		{
+			faimdprintf(sess, 0, "Received an IM containing an invalid message part from %s.  They are probably trying to do something malicious.\n", userinfo->sn);
+			break;
+		}
+
 		endpos = aim_bstream_curpos(bs) + length;
 
 		if (type == 0x0002) { /* Message Block */
@@ -1367,16 +1455,26 @@ static int incomingim_ch1(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 			aimbs_get8(bs); /* 01 */
 
 			args.featureslen = aimbs_get16(bs);
-			/* XXX XXX this is all evil! */
-			args.features = bs->data + bs->offset;
-			aim_bstream_advance(bs, args.featureslen);
-			args.icbmflags |= AIM_IMFLAGS_CUSTOMFEATURES;
+			if (args.featureslen > aim_bstream_empty(bs))
+			{
+				faimdprintf(sess, 0, "Received an IM containing an invalid message part from %s.  They are probably trying to do something malicious.\n", userinfo->sn);
+				break;
+			}
+			if (args.featureslen == 0)
+			{
+				args.features = NULL;
+			}
+			else
+			{
+				args.features = aimbs_getraw(bs, args.featureslen);
+				args.icbmflags |= AIM_IMFLAGS_CUSTOMFEATURES;
+			}
 
 			/*
 			 * The rest of the TLV contains one or more message
 			 * blocks...
 			 */
-			incomingim_ch1_parsemsgs(sess, bs->data + bs->offset /* XXX evil!!! */, length - 2 - 2 - args.featureslen, &args);
+			incomingim_ch1_parsemsgs(sess, userinfo, bs->data + bs->offset /* XXX evil!!! */, length - 2 - 2 - args.featureslen, &args);
 
 		} else if (type == 0x0003) { /* Server Ack Requested */
 
@@ -1421,8 +1519,17 @@ static int incomingim_ch1(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 
 		} else if (type == 0x0017) {
 
+			free(args.extdata);
 			args.extdatalen = length;
-			args.extdata = aimbs_getraw(bs, args.extdatalen);
+			if (args.extdatalen > aim_bstream_empty(bs))
+			{
+				faimdprintf(sess, 0, "Received an IM containing an invalid message part from %s.  They are probably trying to do something malicious.\n", userinfo->sn);
+				break;
+			}
+			if (args.extdatalen == 0)
+				args.extdata = NULL;
+			else
+				args.extdata = aimbs_getraw(bs, args.extdatalen);
 
 		} else {
 			faimdprintf(sess, 0, "incomingim_ch1: unknown TLV 0x%04x (len %d)\n", type, length);
@@ -1430,7 +1537,7 @@ static int incomingim_ch1(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 
 		/*
 		 * This is here to protect ourselves from ourselves.  That
-		 * is, if something above doesn't completly parse its value
+		 * is, if something above doesn't completely parse its value
 		 * section, or, worse, overparses it, this will set the
 		 * stream where it needs to be in order to land on the next
 		 * TLV when the loop continues.
@@ -1444,6 +1551,7 @@ static int incomingim_ch1(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 		ret = userfunc(sess, rx, channel, userinfo, &args);
 
 	aim_mpmsg_free(sess, &args.mpmsg);
+	free(args.features);
 	free(args.extdata);
 
 	return ret;
@@ -1563,7 +1671,6 @@ static void incomingim_ch2_icqserverrelay_free(aim_session_t *sess, struct aim_i
 static void incomingim_ch2_icqserverrelay(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim_modsnac_t *snac, aim_userinfo_t *userinfo, struct aim_incomingim_ch2_args *args, aim_bstream_t *servdata)
 {
 	fu16_t hdrlen, anslen, msglen;
-	fu16_t msgtype;
 
 	hdrlen = aimbs_getle16(servdata);
 	aim_bstream_advance(servdata, hdrlen);
@@ -1571,8 +1678,8 @@ static void incomingim_ch2_icqserverrelay(aim_session_t *sess, aim_module_t *mod
 	hdrlen = aimbs_getle16(servdata);
 	aim_bstream_advance(servdata, hdrlen);
 
-	msgtype = aimbs_getle16(servdata);
-	
+	args->info.rtfmsg.msgtype = aimbs_getle16(servdata);
+
 	anslen = aimbs_getle32(servdata);
 	aim_bstream_advance(servdata, anslen);
 
@@ -1585,9 +1692,6 @@ static void incomingim_ch2_icqserverrelay(aim_session_t *sess, aim_module_t *mod
 	hdrlen = aimbs_getle32(servdata);
 	aim_bstream_advance(servdata, hdrlen);
 
-	/* XXX - This is such a hack. */
-	args->reqclass = AIM_CAPS_ICQRTF;
-
 	args->destructor = (void *)incomingim_ch2_icqserverrelay_free;
 
 	return;
@@ -1653,23 +1757,23 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	memset(&args, 0, sizeof(args));
 
 	/*
-	 * There's another block of TLVs embedded in the type 5 here. 
+	 * There's another block of TLVs embedded in the type 5 here.
 	 */
-	block1 = aim_gettlv(tlvlist, 0x0005, 1);
+	block1 = aim_tlv_gettlv(tlvlist, 0x0005, 1);
 	aim_bstream_init(&bbs, block1->value, block1->length);
 
 	/*
 	 * First two bytes represent the status of the connection.
 	 *
 	 * 0 is a request, 1 is a cancel, 2 is an accept
-	 */ 
+	 */
 	args.status = aimbs_get16(&bbs);
 
 	/*
 	 * Next comes the cookie.  Should match the ICBM cookie.
 	 */
 	cookie2 = aimbs_getraw(&bbs, 8);
-	if (memcmp(cookie, cookie2, 8) != 0) 
+	if (memcmp(cookie, cookie2, 8) != 0)
 		faimdprintf(sess, 0, "rend: warning cookies don't match!\n");
 	memcpy(args.cookie, cookie2, 8);
 	free(cookie2);
@@ -1678,7 +1782,7 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	 * The next 16bytes are a capability block so we can
 	 * identify what type of rendezvous this is.
 	 */
-	args.reqclass = aim_getcap(sess, &bbs, 0x10);
+	args.reqclass = aim_locate_getcaps(sess, &bbs, 0x10);
 
 	/* 
 	 * What follows may be TLVs or nothing, depending on the
@@ -1686,19 +1790,19 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	 *
 	 * Ack packets for instance have nothing more to them.
 	 */
-	list2 = aim_readtlvchain(&bbs);
+	list2 = aim_tlvlist_read(&bbs);
 
 	/*
 	 * IP address to proxy the file transfer through.
 	 *
 	 * XXX - I don't like this.  Maybe just read in an int?  Or inet_ntoa...
 	 */
-	if (aim_gettlv(list2, 0x0002, 1)) {
+	if (aim_tlv_gettlv(list2, 0x0002, 1)) {
 		aim_tlv_t *iptlv;
 
-		iptlv = aim_gettlv(list2, 0x0002, 1);
+		iptlv = aim_tlv_gettlv(list2, 0x0002, 1);
 		if (iptlv->length == 4)
-			snprintf(proxyip, sizeof(proxyip), "%hhd.%hhd.%hhd.%hhd",
+			snprintf(proxyip, sizeof(proxyip), "%hhu.%hhu.%hhu.%hhu",
 				iptlv->value[0], iptlv->value[1],
 				iptlv->value[2], iptlv->value[3]);
 	}
@@ -1706,12 +1810,12 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	/*
 	 * IP address from the perspective of the client.
 	 */
-	if (aim_gettlv(list2, 0x0003, 1)) {
+	if (aim_tlv_gettlv(list2, 0x0003, 1)) {
 		aim_tlv_t *iptlv;
 
-		iptlv = aim_gettlv(list2, 0x0003, 1);
+		iptlv = aim_tlv_gettlv(list2, 0x0003, 1);
 		if (iptlv->length == 4)
-			snprintf(clientip, sizeof(clientip), "%hhd.%hhd.%hhd.%hhd",
+			snprintf(clientip, sizeof(clientip), "%hhu.%hhu.%hhu.%hhu",
 				iptlv->value[0], iptlv->value[1],
 				iptlv->value[2], iptlv->value[3]);
 	}
@@ -1721,12 +1825,12 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	 *
 	 * This is added by the server.
 	 */
-	if (aim_gettlv(list2, 0x0004, 1)) {
+	if (aim_tlv_gettlv(list2, 0x0004, 1)) {
 		aim_tlv_t *iptlv;
 
-		iptlv = aim_gettlv(list2, 0x0004, 1);
+		iptlv = aim_tlv_gettlv(list2, 0x0004, 1);
 		if (iptlv->length == 4)
-			snprintf(verifiedip, sizeof(verifiedip), "%hhd.%hhd.%hhd.%hhd",
+			snprintf(verifiedip, sizeof(verifiedip), "%hhu.%hhu.%hhu.%hhu",
 				iptlv->value[0], iptlv->value[1],
 				iptlv->value[2], iptlv->value[3]);
 	}
@@ -1734,47 +1838,51 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	/*
 	 * Port number for something.
 	 */
-	if (aim_gettlv(list2, 0x0005, 1))
-		args.port = aim_gettlv16(list2, 0x0005, 1);
+	if (aim_tlv_gettlv(list2, 0x0005, 1))
+		args.port = aim_tlv_get16(list2, 0x0005, 1);
 
 	/*
-	 * Something to do with ft -- two bytes
+	 * Something to do with ft? -- two bytes
 	 * 0x0001 - "I want to send you this file"
 	 * 0x0002 - "I will accept this file from you"
+	 * 0x0002 - Also used in ICQ Lite Beta 4.0 URLs
 	 */
-	if (aim_gettlv(list2, 0x000a, 1))
+	if (aim_tlv_gettlv(list2, 0x000a, 1))
 		;
 
 	/*
 	 * Error code.
 	 */
-	if (aim_gettlv(list2, 0x000b, 1))
-		args.errorcode = aim_gettlv16(list2, 0x000b, 1);
+	if (aim_tlv_gettlv(list2, 0x000b, 1))
+		args.errorcode = aim_tlv_get16(list2, 0x000b, 1);
 
 	/*
 	 * Invitation message / chat description.
 	 */
-	if (aim_gettlv(list2, 0x000c, 1))
-		args.msg = aim_gettlv_str(list2, 0x000c, 1);
+	if (aim_tlv_gettlv(list2, 0x000c, 1)) {
+		args.msg = aim_tlv_getstr(list2, 0x000c, 1);
+		args.msglen = aim_tlv_getlength(list2, 0x000c, 1);
+	}
 
 	/*
 	 * Character set.
 	 */
-	if (aim_gettlv(list2, 0x000d, 1))
-		args.encoding = aim_gettlv_str(list2, 0x000d, 1);
+	if (aim_tlv_gettlv(list2, 0x000d, 1))
+		args.encoding = aim_tlv_getstr(list2, 0x000d, 1);
 	
 	/*
 	 * Language.
 	 */
-	if (aim_gettlv(list2, 0x000e, 1))
-		args.language = aim_gettlv_str(list2, 0x000e, 1);
+	if (aim_tlv_gettlv(list2, 0x000e, 1))
+		args.language = aim_tlv_getstr(list2, 0x000e, 1);
 
 	/*
 	 * Unknown -- no value
 	 *
 	 * Maybe means we should connect directly to transfer the file?
+	 * Also used in ICQ Lite Beta 4.0 URLs.  Also empty.
 	 */
-	if (aim_gettlv(list2, 0x000f, 1))
+	if (aim_tlv_gettlv(list2, 0x000f, 1))
 		;
 
 	/*
@@ -1782,7 +1890,7 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	 *
 	 * Maybe means we should proxy the file transfer through an AIM server?
 	 */
-	if (aim_gettlv(list2, 0x0010, 1))
+	if (aim_tlv_gettlv(list2, 0x0010, 1))
 		;
 
 	if (strlen(proxyip))
@@ -1794,11 +1902,12 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 
 	/*
 	 * This must be present in PROPOSALs, but will probably not
-	 * exist in CANCELs and ACCEPTs.
+	 * exist in CANCELs and ACCEPTs.  Also exists in ICQ Lite
+	 * Beta 4.0 URLs (AIM_CAPS_ICQSERVERRELAY).
 	 *
 	 * Service Data blocks are module-specific in format.
 	 */
-	if ((servdatatlv = aim_gettlv(list2, 0x2711 /* 10001 */, 1))) {
+	if ((servdatatlv = aim_tlv_gettlv(list2, 0x2711 /* 10001 */, 1))) {
 
 		aim_bstream_init(&sdbs, servdatatlv->value, servdatatlv->length);
 		sdbsptr = &sdbs;
@@ -1820,7 +1929,6 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	else if (args.reqclass & AIM_CAPS_SENDFILE)
 		incomingim_ch2_sendfile(sess, mod, rx, snac, userinfo, &args, sdbsptr);
 
-
 	if ((userfunc = aim_callhandler(sess, rx->conn, snac->family, snac->subtype)))
 		ret = userfunc(sess, rx, channel, userinfo, &args);
 
@@ -1832,7 +1940,7 @@ static int incomingim_ch2(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	free((char *)args.encoding);
 	free((char *)args.language);
 
-	aim_freetlvchain(&list2);
+	aim_tlvlist_free(&list2);
 
 	return ret;
 }
@@ -1848,7 +1956,7 @@ static int incomingim_ch4(aim_session_t *sess, aim_module_t *mod, aim_frame_t *r
 	/*
 	 * Make a bstream for the meaty part.  Yum.  Meat.
 	 */
-	if (!(block = aim_gettlv(tlvlist, 0x0005, 1)))
+	if (!(block = aim_tlv_gettlv(tlvlist, 0x0005, 1)))
 		return -1;
 	aim_bstream_init(&meat, block->value, block->length);
 
@@ -1889,7 +1997,7 @@ static int incomingim(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, a
 	memset(&userinfo, 0x00, sizeof(aim_userinfo_t));
 
 	/*
-	 * Read ICBM Cookie.  And throw away.
+	 * Read ICBM Cookie.
 	 */
 	for (i = 0; i < 8; i++)
 		cookie[i] = aimbs_get8(bs);
@@ -1900,7 +2008,7 @@ static int incomingim(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, a
 	 * Channel 0x0001 is the message channel.  It is 
 	 * used to send basic ICBMs.
 	 *
-	 * Channel 0x0002 is the Rendevous channel, which
+	 * Channel 0x0002 is the Rendezvous channel, which
 	 * is where Chat Invitiations and various client-client
 	 * connection negotiations come from.
 	 *
@@ -1918,17 +2026,16 @@ static int incomingim(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, a
 	 * Note that although this contains TLVs that appear contiguous
 	 * with the TLVs read below, they are two different pieces.  The
 	 * userinfo block contains the number of TLVs that contain user
-	 * information, the rest are not even though there is no seperation.
-	 * aim_extractuserinfo() returns the number of bytes used by the
-	 * userinfo tlvs, so you can start reading the rest of them right
-	 * afterward.  
+	 * information, the rest are not even though there is no separation.
+	 * You can start reading the message TLVs after aim_info_extract() 
+	 * parses out the standard userinfo block.
 	 *
 	 * That also means that TLV types can be duplicated between the
 	 * userinfo block and the rest of the message, however there should
 	 * never be two TLVs of the same type in one block.
 	 * 
 	 */
-	aim_extractuserinfo(sess, bs, &userinfo);
+	aim_info_extract(sess, bs, &userinfo);
 
 	/*
 	 * From here on, its depends on what channel we're on.
@@ -1948,26 +2055,25 @@ static int incomingim(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, a
 		 * Read block of TLVs (not including the userinfo data).  All 
 		 * further data is derived from what is parsed here.
 		 */
-		tlvlist = aim_readtlvchain(bs);
+		tlvlist = aim_tlvlist_read(bs);
 
 		ret = incomingim_ch2(sess, mod, rx, snac, channel, &userinfo, tlvlist, cookie);
 
-		aim_freetlvchain(&tlvlist);
+		aim_tlvlist_free(&tlvlist);
 
 	} else if (channel == 4) {
 		aim_tlvlist_t *tlvlist;
 
-		tlvlist = aim_readtlvchain(bs);
+		tlvlist = aim_tlvlist_read(bs);
 		ret = incomingim_ch4(sess, mod, rx, snac, channel, &userinfo, tlvlist, cookie);
-		aim_freetlvchain(&tlvlist);
+		aim_tlvlist_free(&tlvlist);
 
 	} else {
-
-		faimdprintf(sess, 0, "icbm: ICBM received on an unsupported channel.  Ignoring.\n (chan = %04x)", channel);
-
-		return 0;
+		faimdprintf(sess, 0, "icbm: ICBM received on an unsupported channel.  Ignoring.  (chan = %04x)\n", channel);
 	}
 
+	aim_info_free(&userinfo);
+
 	return ret;
 }
 
@@ -2014,12 +2120,14 @@ static int missedcall(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, a
 	while (aim_bstream_empty(bs)) {	
 
 		channel = aimbs_get16(bs);
-		aim_extractuserinfo(sess, bs, &userinfo);
+		aim_info_extract(sess, bs, &userinfo);
 		nummissed = aimbs_get16(bs);
 		reason = aimbs_get16(bs);
 
 		if ((userfunc = aim_callhandler(sess, rx->conn, snac->family, snac->subtype)))
 			 ret = userfunc(sess, rx, channel, &userinfo, nummissed, reason);
+
+		aim_info_free(&userinfo);
 	}
 
 	return ret;
@@ -2056,9 +2164,9 @@ faim_export int aim_im_denytransfer(aim_session_t *sess, const char *sender, con
 	aimbs_put8(&fr->data, strlen(sender));
 	aimbs_putraw(&fr->data, sender, strlen(sender));
 
-	aim_addtlvtochain16(&tl, 0x0003, code);
-	aim_writetlvchain(&fr->data, &tl);
-	aim_freetlvchain(&tl);
+	aim_tlvlist_add_16(&tl, 0x0003, code);
+	aim_tlvlist_write(&fr->data, &tl);
+	aim_tlvlist_free(&tl);
 
 	aim_tx_enqueue(sess, fr);
 
@@ -2187,7 +2295,7 @@ static int msgack(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim_m
  * and Gaim 0.60 and newer.
  *
  */
-faim_export int aim_im_sendmtn(aim_session_t *sess, fu16_t type1, char *sn, fu16_t type2)
+faim_export int aim_im_sendmtn(aim_session_t *sess, fu16_t type1, const char *sn, fu16_t type2)
 {
 	aim_conn_t *conn;
 	aim_frame_t *fr;
