@@ -1248,6 +1248,14 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 
 			(void)va_arg(vp, SilcChannelEntry);
 			name = va_arg(vp, char *);
+			if (!name) {
+				gaim_notify_error(gc, _("Roomlist"), _("Cannot get room list"),
+						  silc_get_status_message(status));
+				gaim_roomlist_set_in_progress(sg->roomlist, FALSE);
+				gaim_roomlist_unref(sg->roomlist);
+				sg->roomlist = NULL;
+				return;
+			}
 			topic = va_arg(vp, char *);
 			usercount = va_arg(vp, int);
 
@@ -1469,7 +1477,7 @@ silc_connected(SilcClient client, SilcClientConnection conn,
 	case SILC_CLIENT_CONN_SUCCESS_RESUME:
 		gaim_connection_set_state(gc, GAIM_CONNECTED);
 		serv_finish_login(gc);
-		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
+		g_unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
 
 		/* Send any UMODEs configured for account */
 		reject_watch = gaim_account_get_bool(sg->account, "reject-watch", FALSE);
@@ -1489,7 +1497,7 @@ silc_connected(SilcClient client, SilcClientConnection conn,
 		break;
 	case SILC_CLIENT_CONN_ERROR:
 		gaim_connection_error(gc, _("Error during connecting to SILC Server"));
-		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
+		g_unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
 		break;
 
 	case SILC_CLIENT_CONN_ERROR_KE:
@@ -1504,7 +1512,7 @@ silc_connected(SilcClient client, SilcClientConnection conn,
 		gaim_connection_error(gc,
 				      _("Resuming detached session failed. "
 					"Press Reconnect to create new connection."));
-		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
+		g_unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
 		break;
 
 	case SILC_CLIENT_CONN_ERROR_TIMEOUT:
@@ -1531,7 +1539,7 @@ silc_disconnected(SilcClient client, SilcClientConnection conn,
 	SilcGaim sg = gc->proto_data;
 
 	if (sg->resuming && !sg->detaching)
-		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
+		g_unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
 
 	sg->conn = NULL;
 
@@ -1822,7 +1830,7 @@ silc_detach(SilcClient client, SilcClientConnection conn,
 
 	/* Save the detachment data to file. */
 	file = silcgaim_session_file(gaim_account_get_username(sg->account));
-	unlink(file);
+	g_unlink(file);
 	silc_file_writefile(file, detach_data, detach_data_len);
 }
 
