@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2004 Pekka Riikonen
+  Copyright (C) 2004 - 2005 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -71,6 +71,7 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 {
 	char filename[256], file_public_key[256], file_private_key[256];
 	char servfilename[256], clientfilename[256], friendsfilename[256];
+	char pkd[256], prd[256];
 	struct stat st;
 	struct passwd *pw;
 
@@ -91,11 +92,11 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 	/*
 	 * Check ~/.silc directory
 	 */
-	if ((stat(filename, &st)) == -1) {
+	if ((g_stat(filename, &st)) == -1) {
 		/* If dir doesn't exist */
 		if (errno == ENOENT) {
 			if (pw->pw_uid == geteuid()) {
-				if ((mkdir(filename, 0755)) == -1) {
+				if ((g_mkdir(filename, 0755)) == -1) {
 					gaim_debug_error("silc", "Couldn't create '%s' directory\n", filename);
 					return FALSE;
 				}
@@ -122,11 +123,11 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 	/*
 	 * Check ~./silc/serverkeys directory
 	 */
-	if ((stat(servfilename, &st)) == -1) {
+	if ((g_stat(servfilename, &st)) == -1) {
 		/* If dir doesn't exist */
 		if (errno == ENOENT) {
 			if (pw->pw_uid == geteuid()) {
-				if ((mkdir(servfilename, 0755)) == -1) {
+				if ((g_mkdir(servfilename, 0755)) == -1) {
 					gaim_debug_error("silc", "Couldn't create '%s' directory\n", servfilename);
 					return FALSE;
 				}
@@ -145,11 +146,11 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 	/*
 	 * Check ~./silc/clientkeys directory
 	 */
-	if ((stat(clientfilename, &st)) == -1) {
+	if ((g_stat(clientfilename, &st)) == -1) {
 		/* If dir doesn't exist */
 		if (errno == ENOENT) {
 			if (pw->pw_uid == geteuid()) {
-				if ((mkdir(clientfilename, 0755)) == -1) {
+				if ((g_mkdir(clientfilename, 0755)) == -1) {
 					gaim_debug_error("silc", "Couldn't create '%s' directory\n", clientfilename);
 					return FALSE;
 				}
@@ -168,11 +169,11 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 	/*
 	 * Check ~./silc/friends directory
 	 */
-	if ((stat(friendsfilename, &st)) == -1) {
+	if ((g_stat(friendsfilename, &st)) == -1) {
 		/* If dir doesn't exist */
 		if (errno == ENOENT) {
 			if (pw->pw_uid == geteuid()) {
-				if ((mkdir(friendsfilename, 0755)) == -1) {
+				if ((g_mkdir(friendsfilename, 0755)) == -1) {
 					gaim_debug_error("silc", "Couldn't create '%s' directory\n", friendsfilename);
 					return FALSE;
 				}
@@ -191,12 +192,14 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 	/*
 	 * Check Public and Private keys
 	 */
+	g_snprintf(pkd, sizeof(pkd), "%s" G_DIR_SEPARATOR_S "public_key.pub", silcgaim_silcdir());
+	g_snprintf(prd, sizeof(prd), "%s" G_DIR_SEPARATOR_S "private_key.prv", silcgaim_silcdir());
 	g_snprintf(file_public_key, sizeof(file_public_key) - 1, "%s",
-		   gaim_prefs_get_string("/plugins/prpl/silc/pubkey"));
+		   gaim_account_get_string(gc->account, "public-key", pkd));
 	g_snprintf(file_private_key, sizeof(file_public_key) - 1, "%s",
-		   gaim_prefs_get_string("/plugins/prpl/silc/privkey"));
+		   gaim_account_get_string(gc->account, "private-key", prd));
 
-	if ((stat(file_public_key, &st)) == -1) {
+	if ((g_stat(file_public_key, &st)) == -1) {
 		/* If file doesn't exist */
 		if (errno == ENOENT) {
 			gaim_connection_update_progress(gc, _("Creating SILC key pair..."), 1, 5);
@@ -205,7 +208,7 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 					     file_public_key, file_private_key, NULL,
 					     (gc->account->password == NULL) ? "" : gc->account->password,
 						 NULL, NULL, NULL, FALSE);
-			stat(file_public_key, &st);
+			g_stat(file_public_key, &st);
 		} else {
 			gaim_debug_error("silc", "Couldn't stat '%s' public key, error: %s\n",
 							 file_public_key, strerror(errno));
@@ -221,7 +224,7 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 	}
 #endif
 
-	if ((stat(file_private_key, &st)) == -1) {
+	if ((g_stat(file_private_key, &st)) == -1) {
 		/* If file doesn't exist */
 		if (errno == ENOENT) {
 			gaim_connection_update_progress(gc, _("Creating SILC key pair..."), 1, 5);
@@ -230,7 +233,7 @@ gboolean silcgaim_check_silc_dir(GaimConnection *gc)
 					     file_public_key, file_private_key, NULL,
 					     (gc->account->password == NULL) ? "" : gc->account->password,
 						 NULL, NULL, NULL, FALSE);
-			stat(file_private_key, &st);
+			g_stat(file_private_key, &st);
 		} else {
 			gaim_debug_error("silc", "Couldn't stat '%s' private key, error: %s\n",
 							 file_private_key, strerror(errno));
@@ -304,8 +307,8 @@ void silcgaim_show_public_key(SilcGaim sg,
 
 	s = g_string_new("");
 	if (ident->realname)
-		/* Hint for translators: Please check the tabulator width here and in 
-		   the next strings (short strings: 2 tabs, longer strings 1 tab, 
+		/* Hint for translators: Please check the tabulator width here and in
+		   the next strings (short strings: 2 tabs, longer strings 1 tab,
 		   sum: 3 tabs or 24 characters) */
 		g_string_append_printf(s, _("Real Name: \t%s\n"), ident->realname);
 	if (ident->username)
