@@ -1376,6 +1376,33 @@ static void toc_keepalive(GaimConnection *gc)
 	sflap_send(gc, "", 0, TYPE_KEEPALIVE);
 }
 
+static const char *
+toc_normalize(const GaimAccount *account, const char *str)
+{
+	static char buf[BUF_LEN];
+	char *tmp1, *tmp2;
+	int i, j;
+
+	g_return_val_if_fail(str != NULL, NULL);
+
+	strncpy(buf, str, BUF_LEN);
+	for (i=0, j=0; buf[j]; i++, j++)
+	{
+		while (buf[j] == ' ')
+			j++;
+		buf[i] = buf[j];
+	}
+	buf[i] = '\0';
+
+	tmp1 = g_utf8_strdown(buf, -1);
+	tmp2 = g_utf8_normalize(tmp1, -1, G_NORMALIZE_DEFAULT);
+	g_snprintf(buf, sizeof(buf), "%s", tmp2);
+	g_free(tmp2);
+	g_free(tmp1);
+
+	return buf;
+}
+
 static const char *toc_list_icon(GaimAccount *a, GaimBuddy *b)
 {
 	if (!b || (b && b->name && b->name[0] == '+')) {
@@ -1676,7 +1703,7 @@ static void toc_send_file_callback(gpointer data, gint source, GaimInputConditio
 		write(source, ft, 256);
 
 		if (ft->files == 1) {
-			ft->file = fopen(ft->filename, "w");
+			ft->file = g_fopen(ft->filename, "w");
 			if (!ft->file) {
 				buf = g_strdup_printf(_("Could not open %s for writing!"), ft->filename);
 				gaim_notify_error(ft->gc, NULL, buf, strerror(errno));
@@ -1691,7 +1718,7 @@ static void toc_send_file_callback(gpointer data, gint source, GaimInputConditio
 			}
 		} else {
 			buf = g_strdup_printf("%s/%s", ft->filename, ft->hdr.name);
-			ft->file = fopen(buf, "w");
+			ft->file = g_fopen(buf, "w");
 			g_free(buf);
 			if (!ft->file) {
 				buf = g_strdup_printf("Could not open %s/%s for writing!", ft->filename,
@@ -1839,7 +1866,7 @@ static void toc_get_file_callback(gpointer data, gint source, GaimInputCondition
 		read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
 		debug_header(ft);
 
-		stat(ft->filename, &st);
+		g_stat(ft->filename, &st);
 		fortime = localtime(&st.st_mtime);
 		basename = g_path_get_basename(ft->filename);
 		g_snprintf(buf, sizeof(buf), "%2d/%2d/%4d %2d:%2d %8ld %s\r\n",
@@ -1981,7 +2008,7 @@ static void toc_get_file(gpointer a, struct file_transfer *old_ft)
 		return;
 	ft = g_new0(struct file_transfer, 1);
 	ft->filename = g_strdup(dirname);
-	ft->file = fopen(ft->filename, "r");
+	ft->file = g_fopen(ft->filename, "r");
 	if (!ft->file) {
 		buf = g_strdup_printf("Unable to open %s for transfer.", ft->filename);
 		gaim_notify_error(ft->gc, NULL, buf, NULL);
@@ -1990,7 +2017,7 @@ static void toc_get_file(gpointer a, struct file_transfer *old_ft)
 		g_free(ft);
 		return;
 	}
-	if (stat(dirname, &ft->st)) {
+	if (g_stat(dirname, &ft->st)) {
 		buf = g_strdup_printf("Unable to examine %s.", dirname);
 		gaim_notify_error(ft->gc, NULL, buf, NULL);
 		g_free(buf);
@@ -2156,7 +2183,7 @@ static GaimPluginProtocolInfo prpl_info =
 	NULL,					/* rename_group */
 	NULL,					/* buddy_free */
 	NULL,					/* convo_closed */
-	NULL,					/* normalize */
+	toc_normalize,			/* normalize */
 	NULL,					/* set_buddy_icon */
 	NULL,					/* remove_group */
 	NULL,					/* get_cb_real_name */
