@@ -44,13 +44,20 @@
  */
 char *yahoo_string_encode(GaimConnection *gc, const char *str, gboolean *utf8)
 {
+	struct yahoo_data *yd = gc->proto_data;
 	char *ret;
-	char *to_codeset;
+	const char *to_codeset;
+
+	if (yd->jp && utf8 && *utf8)
+		*utf8 = FALSE;
 
 	if (utf8 && *utf8) /* FIXME: maybe don't use utf8 if it'll fit in latin1 */
 		return g_strdup(str);
 
-	to_codeset = "ISO-8859-1";
+	if (yd->jp)
+		to_codeset = "SHIFT_JIS";
+	else
+		to_codeset = "ISO-8859-1";
 
 	ret = g_convert_with_fallback(str, strlen(str), to_codeset, "UTF-8", "?", NULL, NULL, NULL);
 	if (ret)
@@ -69,6 +76,7 @@ char *yahoo_string_encode(GaimConnection *gc, const char *str, gboolean *utf8)
  */
 char *yahoo_string_decode(GaimConnection *gc, const char *str, gboolean utf8)
 {
+	struct yahoo_data *yd = gc->proto_data;
 	char *ret;
 	char *from_codeset;
 
@@ -77,7 +85,10 @@ char *yahoo_string_decode(GaimConnection *gc, const char *str, gboolean utf8)
 			return g_strdup(str);
 	}
 
-	from_codeset = "ISO-8859-1";
+	if (yd->jp)
+		from_codeset = "SHIFT_JIS";
+	else
+		from_codeset = "ISO-8859-1";
 
 	ret = g_convert_with_fallback(str, strlen(str), "UTF-8", from_codeset, NULL, NULL, NULL, NULL);
 
@@ -207,6 +218,8 @@ static int point_to_html(int x)
 		return 6;
 	return 7;
 }
+
+/* The Yahoo size tag is actually an absz tag; convert it to an HTML size, and include both tags */
 static void _font_tags_fix_size(GString *tag, GString *dest)
 {
 	char *x, *end;
@@ -216,10 +229,13 @@ static void _font_tags_fix_size(GString *tag, GString *dest)
 		while (*x && !g_ascii_isdigit(*x))
 			x++;
 		if (*x) {
+			int htmlsize;
+
 			size = strtol(x, &end, 10);
-			size = point_to_html(size);
+			htmlsize = point_to_html(size);
 			g_string_append_len(dest, tag->str, x - tag->str);
-			g_string_append_printf(dest, "%d", size);
+			g_string_append_printf(dest, "%d", htmlsize);
+			g_string_append_printf(dest, "\" absz=\"%d", size);
 			g_string_append(dest, end);
 		} else {
 			g_string_append(dest, tag->str);
@@ -237,7 +253,6 @@ char *yahoo_codes_to_html(const char *x)
 	int i, j, xs, nomoreendtags = 0; /* s/endtags/closinganglebrackets */
 	char *match, *ret;
 
-
 	s = g_string_sized_new(strlen(x));
 
 	for (i = 0, xs = strlen(x); i < xs; i++) {
@@ -332,7 +347,7 @@ char *yahoo_codes_to_html(const char *x)
 
 /* borrowed from gtkimhtml */
 #define MAX_FONT_SIZE 7
-#define POINT_SIZE(x) (_point_sizes [MIN ((x), MAX_FONT_SIZE) - 1])
+#define POINT_SIZE(x) (_point_sizes [MIN ((x > 0 ? x : 1), MAX_FONT_SIZE) - 1])
 static gint _point_sizes [] = { 8, 10, 12, 14, 20, 30, 40 };
 
 enum fatype { size, color, face, junk };
