@@ -26,6 +26,7 @@
 #include "account.h"
 #include "accountopt.h"
 #include "blist.h"
+#include "cmds.h"
 #include "debug.h"
 #include "notify.h"
 #include "privacy.h"
@@ -77,12 +78,6 @@ gboolean yahoo_privacy_check(GaimConnection *gc, const char *who)
 	gboolean permitted=FALSE;
 
 	switch (gc->account->perm_deny) {
-		case 0:
-			gaim_debug_warning("yahoo", "Privacy setting was 0.  If you can "
-							   "reproduce this, please file a bug report.\n");
-			permitted = TRUE;
-			break;
-
 		case GAIM_PRIVACY_ALLOW_ALL:
 			permitted = TRUE;
 			break;
@@ -114,8 +109,8 @@ gboolean yahoo_privacy_check(GaimConnection *gc, const char *who)
 					gaim_debug_info("yahoo",
 					    "%s blocked data received from %s (GAIM_PRIVACY_DENY_USERS)\n",
 					    gc->account->username,who);
-					}
-				break;
+					break;
+				}
 			}
 			break;
 
@@ -929,7 +924,7 @@ static void yahoo_process_message(GaimConnection *gc, struct yahoo_packet *pkt)
 
 		if (!yahoo_privacy_check(gc, im->from)) {
 			gaim_debug_info("yahoo", "Message from %s dropped.\n", im->from);
-			continue;
+			return;
 		}
 
 		m = yahoo_string_decode(gc, im->msg, im->utf8);
@@ -990,21 +985,72 @@ static void yahoo_process_sysmessage(GaimConnection *gc, struct yahoo_packet *pk
 	g_free(escmsg);
 }
 
+struct yahoo_add_request {
+	GaimConnection *gc;
+	char *id;
+	char *who;
+	char *msg;
+};
+
+static void
+yahoo_buddy_add_authorize_cb(struct yahoo_add_request *add_req, const char *msg) {
+	gaim_account_notify_added(add_req->gc->account, add_req->id,
+			add_req->who, NULL, add_req->msg);
+
+	g_free(add_req->id);
+	g_free(add_req->who);
+	g_free(add_req->msg);
+	g_free(add_req);
+}
+
+static void
+yahoo_buddy_add_deny_cb(struct yahoo_add_request *add_req, const char *msg) {
+	struct yahoo_packet *pkt;
+	char *encoded_msg = NULL;
+	struct yahoo_data *yd = add_req->gc->proto_data;
+
+	if (msg)
+		encoded_msg = yahoo_string_encode(add_req->gc, msg, NULL);
+
+	pkt = yahoo_packet_new(YAHOO_SERVICE_REJECTCONTACT,
+			YAHOO_STATUS_AVAILABLE, 0);
+
+	yahoo_packet_hash(pkt, 1, gaim_normalize(add_req->gc->account,
+				gaim_account_get_username(
+					gaim_connection_get_account(
+						add_req->gc))));
+	yahoo_packet_hash(pkt, 7, add_req->who);
+	yahoo_packet_hash(pkt, 14, encoded_msg ? encoded_msg : "");
+
+	yahoo_send_packet(yd, pkt);
+
+	yahoo_packet_free(pkt);
+
+	g_free(encoded_msg);
+
+	g_free(add_req->id);
+	g_free(add_req->who);
+	g_free(add_req->msg);
+	g_free(add_req);
+}
+
 static void yahoo_buddy_added_us(GaimConnection *gc, struct yahoo_packet *pkt) {
-	char *id = NULL;
-	char *who = NULL;
-	char *msg = NULL, *tmpmsg = NULL;
+	struct yahoo_add_request *add_req;
+	char *msg = NULL;
 	GSList *l = pkt->hash;
 
+	add_req = g_new0(struct yahoo_add_request, 1);
+	add_req->gc = gc;
+
 	while (l) {
 		struct yahoo_pair *pair = l->data;
 
 		switch (pair->key) {
 		case 1:
-			id = pair->value;
+			add_req->id = g_strdup(pair->value);
 			break;
 		case 3:
-			who = pair->value;
+			add_req->who = g_strdup(pair->value);
 			break;
 		case 15: /* time, for when they add us and we're offline */
 			break;
@@ -1015,12 +1061,33 @@ static void yahoo_buddy_added_us(GaimConnection *gc, struct yahoo_packet *pkt) {
 		l = l->next;
 	}
 
-	if (id) {
+	if (add_req->id) {
+		char *prompt_msg;
 		if (msg)
-			tmpmsg = yahoo_string_decode(gc, msg, FALSE);
-		gaim_account_notify_added(gc->account, id, who, NULL, tmpmsg);
-		if (tmpmsg)
-			g_free(tmpmsg);
+			add_req->msg = yahoo_string_decode(gc, msg, FALSE);
+
+		/* TODO: this is almost exactly the same as what MSN does,
+		 * this should probably be moved to the core.
+		 * */
+		prompt_msg = g_strdup_printf(_("The user %s wants to add %s to "
+					"his or her buddy list%s%s."),
+				add_req->who, add_req->id,
+				add_req->msg ? ": " : "",
+				add_req->msg ? add_req->msg : "");
+		gaim_request_input(gc, NULL, prompt_msg,
+				_("Message (optional) :"),
+				NULL, TRUE, FALSE, NULL,
+				_("Authorize"), G_CALLBACK(
+					yahoo_buddy_add_authorize_cb),
+				_("Deny"), G_CALLBACK(
+					yahoo_buddy_add_deny_cb),
+				add_req);
+		g_free(prompt_msg);
+	} else {
+		g_free(add_req->id);
+		g_free(add_req->who);
+		/*g_free(add_req->msg);*/
+		g_free(add_req);
 	}
 }
 
@@ -2458,17 +2525,28 @@ static GHashTable *yahoo_login_page_hash(const char *buf, size_t len)
 	const char *c = buf;
 	char *d;
 	char name[64], value[64];
-	int count = sizeof(name)-1;
-	while ((c < (buf + len)) && (c = strstr(c, "<input "))) {
-		c = strstr(c, "name=\"") + strlen("name=\"");
-		for (d = name; *c!='"' && count; c++, d++, count--) 
+	int count;
+	int input_len = strlen("<input ");
+	int name_len = strlen("name=\"");
+	int value_len = strlen("value=\"");
+	while ((len > ((c - buf) + input_len))
+			&& (c = strstr(c, "<input "))) {
+		if (!(c = g_strstr_len(c, len - (c - buf), "name=\"")))
+			continue;
+		c += name_len;
+		count = sizeof(name)-1;
+		for (d = name; (len > ((c - buf) + 1)) && *c!='"'
+				&& count; c++, d++, count--)
 			*d = *c;
 		*d = '\0';
 		count = sizeof(value)-1;
-		d = strstr(c, "value=\"") + strlen("value=\"");
+		if (!(d = g_strstr_len(c, len - (c - buf), "value=\"")))
+			continue;
+		d += value_len;
 		if (strchr(c, '>') < d)
 			break;
-		for (c = d, d = value; *c!='"' && count; c++, d++, count--)
+		for (c = d, d = value; (len > ((c - buf) + 1))
+				&& *c!='"' && count; c++, d++, count--)
 			*d = *c;
 		*d = '\0';
 		g_hash_table_insert(hash, g_strdup(name), g_strdup(value));
@@ -2593,8 +2671,6 @@ static void yahoo_login(GaimAccount *account) {
 			return;
 		}
 	}
-
-
 }
 
 static void yahoo_close(GaimConnection *gc) {
@@ -3214,6 +3290,9 @@ static void yahoo_add_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *foo
 	if (!yd->logged_in)
 		return;
 
+	if (!yahoo_privacy_check(gc, buddy->name))
+		return;
+
 	if (foo)
 		group = foo->name;
 	if (!group) {
@@ -3423,6 +3502,69 @@ static void yahoo_rename_group(GaimConnection *gc, const char *old_name,
 
 static GaimPlugin *my_protocol = NULL;
 
+/********************************* Commands **********************************/
+
+static GaimCmdRet
+yahoogaim_cmd_buzz(GaimConversation *c, const gchar *cmd, gchar **args,
+					gchar **error, void *data) {
+
+	GaimAccount *account = gaim_conversation_get_account(c);
+	const char *username = gaim_account_get_username(account);
+
+	if (*args && args[0])
+		return GAIM_CMD_RET_FAILED;
+
+	gaim_debug(GAIM_DEBUG_INFO, "yahoo",
+	           "Sending <ding> on account %s to buddy %s.\n", username, c->name);
+	gaim_conv_im_send(GAIM_CONV_IM(c), "<ding>");
+	gaim_conv_im_write(GAIM_CONV_IM(c), "", _("Buzz!!"), 
+		GAIM_MESSAGE_NICK|GAIM_MESSAGE_RECV, time(NULL));
+	return GAIM_CMD_RET_OK;
+}
+
+static GaimCmdRet
+yahoogaim_cmd_chat_join(GaimConversation *conv, const char *cmd,
+                        char **args, char **error, void *data)
+{
+	GHashTable *comp;
+	GaimConnection *gc;
+	struct yahoo_data *yd;
+	int id;
+
+	if (!args || !args[0])
+		return GAIM_CMD_RET_FAILED;
+
+	gc = gaim_conversation_get_gc(conv);
+	yd = gc->proto_data;
+	id = yd->conf_id;
+	gaim_debug(GAIM_DEBUG_INFO, "yahoo",
+	           "Trying to join %s \n", args[0]);
+
+	comp = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	g_hash_table_replace(comp, g_strdup("room"),
+	g_strdup_printf("%s", g_ascii_strdown(args[0], strlen(args[0]))));
+	g_hash_table_replace(comp, g_strdup("type"), g_strdup("Chat"));
+
+	yahoo_c_join(gc, comp);
+
+	g_hash_table_destroy(comp);
+	return GAIM_CMD_RET_OK;
+}
+
+/************************** Plugin Initialization ****************************/
+static void
+yahoogaim_register_commands(void)
+{
+	gaim_cmd_register("join", "s", GAIM_CMD_P_PRPL,
+	                  GAIM_CMD_FLAG_IM | GAIM_CMD_FLAG_CHAT |
+	                  GAIM_CMD_FLAG_PRPL_ONLY,
+	                  "prpl-yahoo", yahoogaim_cmd_chat_join,
+	                  _("join &lt;room&gt;:  Join a chat room on the Yahoo network"), NULL);
+	gaim_cmd_register("buzz", "", GAIM_CMD_P_PRPL,
+	                  GAIM_CMD_FLAG_IM | GAIM_CMD_FLAG_PRPL_ONLY,
+	                  "prpl-yahoo", yahoogaim_cmd_buzz,
+	                  _("buzz: Buzz a contact to get their attention"), NULL);
+}
 static GaimPluginProtocolInfo prpl_info =
 {
 	OPT_PROTO_MAIL_CHECK | OPT_PROTO_CHAT_TOPIC,
@@ -3472,7 +3614,7 @@ static GaimPluginProtocolInfo prpl_info =
 	yahoo_rename_group,
 	NULL, /* buddy_free */
 	NULL, /* convo_closed */
-	NULL, /* normalize */
+	gaim_normalize_nocase, /* normalize */
 	yahoo_set_buddy_icon,
 	NULL, /* void (*remove_group)(GaimConnection *gc, const char *group);*/
 	NULL, /* char *(*get_cb_real_name)(GaimConnection *gc, int id, const char *who); */
@@ -3542,9 +3684,12 @@ init_plugin(GaimPlugin *plugin)
 	option = gaim_account_option_int_new(_("File transfer port"), "xfer_port", YAHOO_XFER_PORT);
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
 
-	option = gaim_account_option_string_new(_("Chat Room List Url"), "room_list", YAHOO_ROOMLIST_URL);
+	option = gaim_account_option_string_new(_("Chat Room Locale"), "room_list_locale", YAHOO_ROOMLIST_LOCALE);
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
 #if 0
+	option = gaim_account_option_string_new(_("Chat Room List Url"), "room_list", YAHOO_ROOMLIST_URL);
+	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
+
 	option = gaim_account_option_string_new(_("YCHT Host"), "ycht-server", YAHOO_YCHT_HOST);
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
 
@@ -3553,7 +3698,7 @@ init_plugin(GaimPlugin *plugin)
 #endif
 
 	my_protocol = plugin;
-
+	yahoogaim_register_commands();
 	yahoo_init_colorht();
 }
 
