@@ -73,7 +73,8 @@ static void yahoo_receivefile_connected(gpointer data, gint source, GaimInputCon
 	xfer->fd = source;
 	gaim_xfer_start(xfer, source, NULL, 0);
 
-	buf = g_strdup_printf("GET /%s HTTP/1.0\r\nHost: %s\r\n\r\n", xd->path, xd->host);
+	buf = g_strdup_printf("GET /%s HTTP/1.0\r\nHost: %s\r\n\r\n",
+			      xd->path, xd->host);
 	write(xfer->fd, buf, strlen(buf));
 	g_free(buf);
 
@@ -86,11 +87,12 @@ static void yahoo_sendfile_connected(gpointer data, gint source, GaimInputCondit
 	struct yahoo_xfer_data *xd;
 	struct yahoo_packet *pkt;
 	gchar *size, *post, *buf;
-	int content_length;
+	const char *host;
+	int content_length, port;
 	GaimConnection *gc;
 	GaimAccount *account;
 	struct yahoo_data *yd;
-	char *filename;
+	char *filename, *encoded_filename;
 
 	gaim_debug(GAIM_DEBUG_INFO, "yahoo",
 			   "AAA - in yahoo_sendfile_connected\n");
@@ -123,22 +125,22 @@ static void yahoo_sendfile_connected(gpointer data, gint source, GaimInputCondit
 	yahoo_packet_hash(pkt, 5, xfer->who);
 	yahoo_packet_hash(pkt, 14, "");
 	filename = g_path_get_basename(gaim_xfer_get_local_filename(xfer));
-	yahoo_packet_hash(pkt, 27, filename);
+	encoded_filename = yahoo_string_encode(gc, filename, NULL);
+	yahoo_packet_hash(pkt, 27, encoded_filename);
 	yahoo_packet_hash(pkt, 28, size);
 
 	content_length = YAHOO_PACKET_HDRLEN + yahoo_packet_length(pkt);
 
 	buf = g_strdup_printf("Y=%s; T=%s", yd->cookie_y, yd->cookie_t);
 
-	post = g_strdup_printf("POST /notifyft HTTP/1.0\r\n"
+	host = gaim_account_get_string(account, "xfer_host", YAHOO_XFER_HOST);
+	port = gaim_account_get_int(account, "xfer_port", YAHOO_XFER_PORT);
+	post = g_strdup_printf("POST http://%s:%d/notifyft HTTP/1.0\r\n"
 	                       "Content-length: %" G_GSIZE_FORMAT "\r\n"
 	                       "Host: %s:%d\r\n"
 	                       "Cookie: %s\r\n"
 	                       "\r\n",
-	                       content_length + 4 + gaim_xfer_get_size(xfer),
-			       gaim_account_get_string(account, "xfer_host", YAHOO_XFER_HOST),
-			       gaim_account_get_int(account, "xfer_port", YAHOO_XFER_PORT),
-			       buf);
+			       host, port, content_length + 4 + gaim_xfer_get_size(xfer), host, port, buf);
 	write(xfer->fd, post, strlen(post));
 
 	yahoo_send_packet_special(xfer->fd, pkt, 8);
@@ -149,6 +151,7 @@ static void yahoo_sendfile_connected(gpointer data, gint source, GaimInputCondit
 	g_free(size);
 	g_free(post);
 	g_free(buf);
+	g_free(encoded_filename);
 	g_free(filename);
 }
 
@@ -260,6 +263,11 @@ ssize_t yahoo_xfer_read(char **buffer, GaimXfer *xfer)
 		xd->rxlen += len;
 
 		length = g_strstr_len(xd->rxqueue, len, "Content-length:");
+		/* some proxies re-write this header, changing the capitalization :(
+		 * technically that's allowed since headers are case-insensitive
+		 * [RFC 2616, section 4.2] */
+		if (length == NULL)
+			length = g_strstr_len(xd->rxqueue, len, "Content-Length:");
 		if (length) {
 			end = g_strstr_len(length, length - xd->rxqueue, "\r\n");
 			if (!end)
@@ -417,18 +425,23 @@ void yahoo_process_filetransfer(GaimConnection *gc, struct yahoo_packet *pkt)
 	xfer->data = xfer_data;
 
 	/* Set the info about the incoming file. */
-	if (filename)
-		gaim_xfer_set_filename(xfer, filename);
-	else {
+	if (filename) {
+		char *utf8_filename = yahoo_string_decode(gc, filename, TRUE);
+		gaim_xfer_set_filename(xfer, utf8_filename);
+		g_free(utf8_filename);
+	} else {
 		gchar *start, *end;
 	 	start = g_strrstr(xfer_data->path, "/");
 		if (start)
 			start++;
 		end = g_strrstr(xfer_data->path, "?");
 		if (start && *start && end) {
+			char *utf8_filename;
 			filename = g_strndup(start, end - start);
-			gaim_xfer_set_filename(xfer, filename);
+			utf8_filename = yahoo_string_decode(gc, filename, TRUE);
 			g_free(filename);
+			gaim_xfer_set_filename(xfer, utf8_filename);
+			g_free(utf8_filename);
 			filename = NULL;
 		}
 	}
