@@ -4,16 +4,16 @@
  *	Created by:	Robert French
  *
  *	$Source: /cvsroot/gaim/gaim/src/protocols/zephyr/ZGetWGPort.c,v $
- *	$Author: chipx86 $
+ *	$Author: datallah $
  *
  *	Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
  *	"mit-copyright.h". 
  */
-/* $Header: /cvsroot/gaim/gaim/src/protocols/zephyr/ZGetWGPort.c,v 1.2 2004/04/24 09:02:28 chipx86 Exp $ */
+/* $Header: /cvsroot/gaim/gaim/src/protocols/zephyr/ZGetWGPort.c,v 1.2.2.2 2005/02/12 01:26:13 datallah Exp $ */
 
 #ifndef lint
-static char rcsid_ZGetWGPort_c[] = "$Header: /cvsroot/gaim/gaim/src/protocols/zephyr/ZGetWGPort.c,v 1.2 2004/04/24 09:02:28 chipx86 Exp $";
+static char rcsid_ZGetWGPort_c[] = "$Header: /cvsroot/gaim/gaim/src/protocols/zephyr/ZGetWGPort.c,v 1.2.2.2 2005/02/12 01:26:13 datallah Exp $";
 #endif
 
 #include "internal.h"
