@@ -4,19 +4,19 @@
  *	Created by:	Robert French
  *
  *	$Source: /cvsroot/gaim/gaim/src/protocols/zephyr/ZRecvPkt.c,v $
- *	$Author: warmenhoven $
+ *	$Author: chipx86 $
  *
  *	Copyright (c) 1987,1988 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
  *	"mit-copyright.h". 
  */
-/* $Header: /cvsroot/gaim/gaim/src/protocols/zephyr/ZRecvPkt.c,v 1.1 2001/07/31 01:00:39 warmenhoven Exp $ */
+/* $Header: /cvsroot/gaim/gaim/src/protocols/zephyr/ZRecvPkt.c,v 1.2 2004/04/24 09:02:28 chipx86 Exp $ */
 
 #ifndef lint
-static char rcsid_ZReceivePacket_c[] = "$Header: /cvsroot/gaim/gaim/src/protocols/zephyr/ZRecvPkt.c,v 1.1 2001/07/31 01:00:39 warmenhoven Exp $";
+static char rcsid_ZReceivePacket_c[] = "$Header: /cvsroot/gaim/gaim/src/protocols/zephyr/ZRecvPkt.c,v 1.2 2004/04/24 09:02:28 chipx86 Exp $";
 #endif
 
-#include <internal.h>
+#include "internal.h"
 
 Code_t ZReceivePacket(buffer, ret_len, from)
     ZPacket_t buffer;
