@@ -4,23 +4,23 @@
  *	Created by:	Robert French
  *
  *	$Source: /cvsroot/gaim/gaim/src/protocols/zephyr/Zinternal.c,v $
- *	$Author: faceprint $
+ *	$Author: chipx86 $
  *
  *	Copyright (c) 1987,1988,1991 by the Massachusetts Institute of
  *	Technology.
  *	For copying and distribution information, see the file
  *	"mit-copyright.h". 
  */
-/* $Header: /cvsroot/gaim/gaim/src/protocols/zephyr/Zinternal.c,v 1.3 2003/04/15 04:18:00 faceprint Exp $ */
+/* $Header: /cvsroot/gaim/gaim/src/protocols/zephyr/Zinternal.c,v 1.5 2004/04/24 09:02:28 chipx86 Exp $ */
 
-#include <internal.h>
+#include "internal.h"
 #include <arpa/inet.h>
 #include <sys/socket.h>
 #include <utmp.h>
 
 #ifndef lint
 static const char rcsid_Zinternal_c[] =
-  "$Id: Zinternal.c,v 1.3 2003/04/15 04:18:00 faceprint Exp $";
+  "$Id: Zinternal.c,v 1.5 2004/04/24 09:02:28 chipx86 Exp $";
 static const char copyright[] =
   "Copyright (c) 1987,1988,1991 by the Massachusetts Institute of Technology.";
 #endif
@@ -594,7 +594,7 @@ Code_t Z_FormatHeader(notice, buffer, buffer_len, len, cert_routine)
 
     if (notice->z_port == 0) {
 	if (ZGetFD() < 0) {
-	    retval = ZOpenPort((u_short *)0);
+	    retval = ZOpenPort((unsigned short *)0);
 	    if (retval != ZERR_NONE)
 		return (retval);
 	}
@@ -607,8 +607,8 @@ Code_t Z_FormatHeader(notice, buffer, buffer_len, len, cert_routine)
     notice->z_multinotice = "";
     
     (void) gettimeofday(&notice->z_uid.tv, (struct timezone *)0);
-    notice->z_uid.tv.tv_sec = htonl((u_long) notice->z_uid.tv.tv_sec);
-    notice->z_uid.tv.tv_usec = htonl((u_long) notice->z_uid.tv.tv_usec);
+    notice->z_uid.tv.tv_sec = htonl((unsigned long) notice->z_uid.tv.tv_sec);
+    notice->z_uid.tv.tv_usec = htonl((unsigned long) notice->z_uid.tv.tv_usec);
     
     (void) memcpy(&notice->z_uid.zuid_addr, &__My_addr, sizeof(__My_addr));
 
@@ -877,9 +877,9 @@ Code_t Z_SendFragmentedNotice(notice, len, cert_func, send_func)
 	    (void) gettimeofday(&partnotice.z_uid.tv,
 				(struct timezone *)0);
 	    partnotice.z_uid.tv.tv_sec =
-		htonl((u_long) partnotice.z_uid.tv.tv_sec);
+		htonl((unsigned long) partnotice.z_uid.tv.tv_sec);
 	    partnotice.z_uid.tv.tv_usec =
-		htonl((u_long) partnotice.z_uid.tv.tv_usec);
+		htonl((unsigned long) partnotice.z_uid.tv.tv_usec);
 	    (void) memcpy((char *)&partnotice.z_uid.zuid_addr, &__My_addr, 
 			  sizeof(__My_addr));
 	}
