@@ -168,7 +168,6 @@ Code_t zephyr_subscribe_to(zephyr_account* zephyr, char* class, char *instance,
 }
 
 char *local_zephyr_normalize(zephyr_account* zephyr,const char *);
-static const char *zephyr_normalize(const GaimAccount *, const char *);
 static void zephyr_chat_set_topic(GaimConnection * gc, int id, const char *topic);
 char* zephyr_tzc_deescape_str(const char *message);
 
@@ -380,58 +379,64 @@ static char *html_to_zephyr(const char *message)
 				retcount += 1;
 			} else if (!g_ascii_strncasecmp(message + cnt + 1, "a href=\"mailto:", 15)) {
 				cnt += 16;
-				while (g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
+				while ((message[cnt] != '\0') && g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
 					ret[retcount] = message[cnt];
 					retcount++;
 					cnt++;
 				}
-				cnt += 2;
+				if (message[cnt] != '\0')
+					cnt += 2;
 				/* ignore descriptive string */
-				while (g_ascii_strncasecmp(message + cnt, "</a>", 4) != 0) {
+				while ((message[cnt] != '\0') && g_ascii_strncasecmp(message + cnt, "</a>", 4) != 0) {
 					cnt++;
 				}
-				cnt += 4;
+				if (message[cnt] != '\0')
+					cnt += 4;
 			} else if (!g_ascii_strncasecmp(message + cnt + 1, "a href=\"", 8)) {
 				cnt += 9;
-				while (g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
+				while ((message[cnt] != '\0') && g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
 					ret[retcount] = message[cnt];
 					retcount++;
 					cnt++;
 				}
-				cnt += 2;
+				if (message[cnt] != '\0')
+					cnt += 2;
 				/* ignore descriptive string */
-				while (g_ascii_strncasecmp(message + cnt, "</a>", 4) != 0) {
+				while ((message[cnt] != '\0') && g_ascii_strncasecmp(message + cnt, "</a>", 4) != 0) {
 					cnt++;
 				}
-				cnt += 4;
+				if (message[cnt] != '\0')
+					cnt += 4;
 			} else if (!g_ascii_strncasecmp(message + cnt + 1, "font", 4)) {
 				cnt += 5;
-				while (!g_ascii_strncasecmp(message + cnt, " ", 1))
+				while ((message[cnt] != '\0') && (message[cnt] != ' '))
 					cnt++;
-				if (!g_ascii_strncasecmp(message + cnt, "color=\"", 7)) {
+				if ((message[cnt] != '\0') && !g_ascii_strncasecmp(message + cnt, "color=\"", 7)) {
 					cnt += 7;
 					strncpy(ret + retcount, "@color(", 7);
 					retcount += 7;
-					while (g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
+					while ((message[cnt] != '\0') && g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
 						ret[retcount] = message[cnt];
 						retcount++;
 						cnt++;
 					}
 					ret[retcount] = ')';
 					retcount++;
-					cnt += 2;
+					if (message[cnt] != '\0')
+						cnt += 2;
 				} else if (!g_ascii_strncasecmp(message + cnt, "face=\"", 6)) {
 					cnt += 6;
 					strncpy(ret + retcount, "@font(", 6);
 					retcount += 6;
-					while (g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
+					while ((message[cnt] != '\0') && g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
 						ret[retcount] = message[cnt];
 						retcount++;
 						cnt++;
 					}
 					ret[retcount] = ')';
 					retcount++;
-					cnt += 2;
+					if (message[cnt] != '\0')
+						cnt += 2;
 				} else if (!g_ascii_strncasecmp(message + cnt, "size=\"", 6)) {
 					cnt += 6;
 					if ((message[cnt] == '1') || (message[cnt] == '2')) {
@@ -450,10 +455,11 @@ static char *html_to_zephyr(const char *message)
 					cnt += 3;
 				} else {
 					/* Drop all unrecognized/misparsed font tags */
-					while (g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
+					while ((message[cnt] != '\0') && g_ascii_strncasecmp(message + cnt, "\">", 2) != 0) {
 						cnt++;
 					}
-					cnt += 2;
+					if (message[cnt] != '\0')
+						cnt += 2;
 				}
 			} else if (!g_ascii_strncasecmp(message + cnt + 1, "/i>", 3)
 				   || !g_ascii_strncasecmp(message + cnt + 1, "/b>", 3)) {
@@ -466,7 +472,7 @@ static char *html_to_zephyr(const char *message)
 				retcount += 12;
 			} else {
 				/* Catch all for all unrecognized/misparsed <foo> tage */
-				while (g_ascii_strncasecmp(message + cnt, ">", 1) != 0) {
+				while ((message[cnt] != '\0') && (message[cnt] != '>')) {
 					ret[retcount] = message[cnt];
 					retcount++;
 					cnt++;
@@ -1315,7 +1321,7 @@ static void process_zsubs(zephyr_account *zephyr)
 	gchar buff[BUFSIZ];
 
 	fname = g_strdup_printf("%s/.zephyr.subs", gaim_home_dir());
-	f = fopen(fname, "r");
+	f = g_fopen(fname, "r");
 	if (f) {
 		char **triple;
 		char *recip;
@@ -1411,7 +1417,7 @@ static void process_anyone(GaimConnection *gc)
 	}
 
 	filename = g_strconcat(gaim_home_dir(), "/.anyone", NULL);
-	if ((fd = fopen(filename, "r")) != NULL) {
+	if ((fd = g_fopen(filename, "r")) != NULL) {
 		while (fgets(buff, BUFSIZ, fd)) {
 			strip_comments(buff);
 			if (buff[0]) {
@@ -1716,7 +1722,7 @@ static void write_zsubs(zephyr_account *zephyr)
 	char **triple;
 
 	fname = g_strdup_printf("%s/.zephyr.subs", gaim_home_dir());
-	fd = fopen(fname, "w");
+	fd = g_fopen(fname, "w");
 
 	if (!fd) {
 		g_free(fname);
@@ -1778,7 +1784,7 @@ static void write_anyone(GaimConnection *gc)
 	FILE *fd;
 	zephyr_account* zephyr = gc->proto_data;
 	fname = g_strdup_printf("%s/.anyone", gaim_home_dir());
-	fd = fopen(fname, "w");
+	fd = g_fopen(fname, "w");
 	if (!fd) {
 		g_free(fname);
 		return;
@@ -2042,24 +2048,6 @@ static int zephyr_send_message(zephyr_account *zephyr,char* zclass, char* instan
 	return 1;
 }
 
-static const char *zephyr_normalize(const GaimAccount * account, const char *orig)
-{
-	/* returns the string you gave it. Maybe this function shouldn't be here */
-	char * buf = g_malloc0(80);
-	/*	gaim_debug_error("zephyr","entering zephyr_normalize\n"); */
-
-	if (!g_ascii_strcasecmp(orig, "")) {
-		buf[0] = '\0';
-		return buf;
-	} else {
-		g_snprintf(buf, 80, "%s", orig);
-	}
-	/*	gaim_debug_error("zephyr","leaving zephyr_normalize\n"); */
-
-	return buf;
-}
-
-
 char *local_zephyr_normalize(zephyr_account *zephyr,const char *orig)
 {
 	/* 
@@ -2694,7 +2682,7 @@ static GaimPluginProtocolInfo prpl_info = {
 	NULL,					/* rename_group */
 	NULL,					/* buddy_free */
 	NULL,					/* convo_closed */
-	zephyr_normalize,			/* normalize */
+	NULL,					/* normalize */
 	NULL,					/* XXX set_buddy_icon */
 	NULL,					/* remove_group */
 	NULL,					/* XXX get_cb_real_name */
