@@ -4,8 +4,9 @@
  *
  * gaim
  *
- * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +26,7 @@
 #define _GAIM_PROXY_H_
 
 #include <glib.h>
+#include "eventloop.h"
 
 /**
  * A type of proxy connection.
@@ -40,16 +42,6 @@ typedef enum
 
 } GaimProxyType;
 
-/**
- * An input condition.
- */
-typedef enum
-{
-	GAIM_INPUT_READ  = 1 << 0,  /**< A read condition.  */
-	GAIM_INPUT_WRITE = 1 << 1   /**< A write condition. */
-
-} GaimInputCondition;
-
 /**
  * Information on proxy settings.
  */
@@ -64,8 +56,6 @@ typedef struct
 
 } GaimProxyInfo;
 
-typedef void (*GaimInputFunction)(gpointer, gint, GaimInputCondition);
-
 
 #include "account.h"
 
@@ -88,38 +78,38 @@ GaimProxyInfo *gaim_proxy_info_new(void);
 /**
  * Destroys a proxy information structure.
  *
- * @param proxy The proxy information structure to destroy.
+ * @param info The proxy information structure to destroy.
  */
 void gaim_proxy_info_destroy(GaimProxyInfo *info);
 
 /**
  * Sets the type of proxy.
  *
- * @param proxy The proxy information.
- * @param type  The proxy type.
+ * @param info The proxy information.
+ * @param type The proxy type.
  */
 void gaim_proxy_info_set_type(GaimProxyInfo *info, GaimProxyType type);
 
 /**
  * Sets the proxy host.
  *
- * @param proxy The proxy information.
- * @param host  The host.
+ * @param info The proxy information.
+ * @param host The host.
  */
 void gaim_proxy_info_set_host(GaimProxyInfo *info, const char *host);
 
 /**
  * Sets the proxy port.
  *
- * @param proxy The proxy information.
- * @param port  The port.
+ * @param info The proxy information.
+ * @param port The port.
  */
 void gaim_proxy_info_set_port(GaimProxyInfo *info, int port);
 
 /**
  * Sets the proxy username.
  *
- * @param proxy    The proxy information.
+ * @param info     The proxy information.
  * @param username The username.
  */
 void gaim_proxy_info_set_username(GaimProxyInfo *info, const char *username);
@@ -127,7 +117,7 @@ void gaim_proxy_info_set_username(GaimProxyInfo *info, const char *username);
 /**
  * Sets the proxy password.
  *
- * @param proxy    The proxy information.
+ * @param info     The proxy information.
  * @param password The password.
  */
 void gaim_proxy_info_set_password(GaimProxyInfo *info, const char *password);
@@ -135,7 +125,7 @@ void gaim_proxy_info_set_password(GaimProxyInfo *info, const char *password);
 /**
  * Returns the proxy's type.
  *
- * @param The proxy information.
+ * @param info The proxy information.
  *
  * @return The type.
  */
@@ -144,7 +134,7 @@ GaimProxyType gaim_proxy_info_get_type(const GaimProxyInfo *info);
 /**
  * Returns the proxy's host.
  *
- * @param The proxy information.
+ * @param info The proxy information.
  *
  * @return The host.
  */
@@ -153,7 +143,7 @@ const char *gaim_proxy_info_get_host(const GaimProxyInfo *info);
 /**
  * Returns the proxy's port.
  *
- * @param The proxy information.
+ * @param info The proxy information.
  *
  * @return The port.
  */
@@ -162,7 +152,7 @@ int gaim_proxy_info_get_port(const GaimProxyInfo *info);
 /**
  * Returns the proxy's username.
  *
- * @param The proxy information.
+ * @param info The proxy information.
  *
  * @return The username.
  */
@@ -171,7 +161,7 @@ const char *gaim_proxy_info_get_username(const GaimProxyInfo *info);
 /**
  * Returns the proxy's password.
  *
- * @param The proxy information.
+ * @param info The proxy information.
  *
  * @return The password.
  */
@@ -203,38 +193,32 @@ GaimProxyInfo *gaim_global_proxy_get_info(void);
  */
 void gaim_proxy_init(void);
 
-/**
- * Adds an input handler.
- *
- * @param source    The input source.
- * @param cond      The condition type.
- * @param func      The callback function for data.
- * @param user_data User-specified data.
- *
- * @return The resulting handle.
- */
-gint gaim_input_add(int source, GaimInputCondition cond,
-					GaimInputFunction func, gpointer user_data);
-
-/**
- * Removes an input handler.
- *
- * @param handle The handle of the input handler.
- */
-void gaim_input_remove(gint handle);
-
 /**
  * Makes a connection to the specified host and port.
  *
  * @param account The account making the connection.
  * @param host    The destination host.
- * @Param port    The destination port.
- * @Param func    The input handler function.
+ * @param port    The destination port.
+ * @param func    The input handler function.
+ * @param data    User-defined data.
+ *
+ * @return 0 for success, -1 for failure
+ */
+int gaim_proxy_connect(GaimAccount *account, const char *host, int port,
+					   GaimInputFunction func, gpointer data);
+
+/**
+ * Makes a connection through a SOCKS5 proxy.
+ *
+ * @param gpi     The GaimProxyInfo specifying the proxy settings
+ * @param host    The destination host.
+ * @param port    The destination port.
+ * @param func    The input handler function.
  * @param data    User-defined data.
  *
  * @return The socket handle.
  */
-int gaim_proxy_connect(GaimAccount *account, const char *host, int port,
+int gaim_proxy_connect_socks5(GaimProxyInfo *gpi, const char *host, int port,
 					   GaimInputFunction func, gpointer data);
 
 /*@}*/
